@@ -31,9 +31,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <Foundation/NSArray.h>
 #include <Foundation/NSCalendarDate.h>
-#include <Foundation/NSData.h>
-#include <Foundation/NSDate.h>
-#include <Foundation/NSTimeZone.h>
 
 #include <SOGo/NSString+Utilities.h>
 #include <SOGo/NSData+Crypto.h>
@@ -65,11 +62,7 @@ static NSArray *easCommandParameters = nil;
 
 - (NSString *) activeSyncRepresentationInContext: (WOContext *) context
 {
-  NSString *s;
-
-  s = [self safeString];
-
-  return [s stringByEscapingHTMLString];
+  return [self safeStringByEscapingXMLString: YES];
 }
 
 - (int) activeSyncFolderType
@@ -142,6 +135,27 @@ static NSArray *easCommandParameters = nil;
   return o;
 }
 
+- (NSString *) protocolVersion
+{
+  NSMutableArray *components;
+
+  components = [NSMutableArray arrayWithArray: [[[self componentsSeparatedByString: @"?"] lastObject] componentsSeparatedByString: @"&"]];
+
+  // We handle BASE64 encoded queryStrings. See http://msdn.microsoft.com/en-us/library/ee160227%28v=exchg.80%29.aspx for details.
+  if ([components count] == 1)
+    {
+      NSData *queryString;
+      const char* qs_bytes;
+
+      queryString = [[components objectAtIndex: 0] dataByDecodingBase64];
+      qs_bytes = (const char*)[queryString bytes];
+
+      return [NSString stringWithFormat:@"%.1f", (float)((uint8_t)qs_bytes[0]/10)];
+    }
+
+  return nil;
+}
+
 - (NSString *) _valueForParameter: (NSString *) theParameter
 {
   NSMutableArray *components;
@@ -322,6 +336,18 @@ static NSArray *easCommandParameters = nil;
   return s;
 }
 
+- (BOOL) acceptsMultiPart
+{
+  NSString *s;
+
+  s = [self _valueForParameter: @"OPTIONS="];
+
+  if (s && [s rangeOfString: @"AcceptMultiPart" options: NSCaseInsensitiveSearch].location != NSNotFound)
+    return YES;
+
+  return NO;
+}
+
 
 //
 // FIXME: combine with our OpenChange code.
