@@ -29,26 +29,27 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 #include "SOGoMailObject+ActiveSync.h"
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSException.h>
 #import <Foundation/NSString.h>
+#import <Foundation/NSCharacterSet.h>
 
 #import <NGCards/iCalCalendar.h>
 #import <NGCards/iCalDateTime.h>
 #import <NGCards/iCalEvent.h>
-#import <NGCards/iCalPerson.h>
-#import <NGCards/iCalTimeZone.h>
 
 #import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NGQuotedPrintableCoding.h>
+
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSString+Encoding.h>
 #import <NGImap4/NGImap4Envelope.h>
 #import <NGImap4/NGImap4EnvelopeAddress.h>
 #import <NGImap4/NSString+Imap4.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
+#import <NGObjWeb/WOApplication.h>
+#import <NGObjWeb/WORequest.h>
 
 #import <NGMime/NGMimeBodyPart.h>
 #import <NGMime/NGMimeFileData.h>
@@ -58,8 +59,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #import <NGMail/NGMimeMessage.h>
 #import <NGMail/NGMimeMessageGenerator.h>
 
-#import <Mailer/SOGoMailLabel.h>
-
 #import <SOGo/SOGoUserDefaults.h>
 
 #include "iCalTimeZone+ActiveSync.h"
@@ -69,11 +68,51 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <Appointments/iCalEntityObject+SOGo.h>
 #include <Appointments/iCalPerson+SOGo.h>
-#include <Mailer/NSString+Mail.h>
 #include <Mailer/SOGoMailBodyPart.h>
+#include <Mailer/NSString+Mail.h>
+
 #include <SOGo/SOGoUser.h>
 #include <SOGo/NSString+Utilities.h>
 
+#import <Appointments/SOGoAptMailNotification.h>
+
+
+
+unsigned char strToChar(char a, char b) {
+    char encoder[3] = {'\0','\0','\0'};
+    encoder[0] = a;
+    encoder[1] = b;
+    return (char) strtol(encoder,NULL,16);
+}
+
+@interface NSString (NSStringExtensions)
+- (NSData *) decodeFromHexidecimal;
+@end
+
+@implementation NSString (NSStringExtensions)
+
+- (NSData *) decodeFromHexidecimal;
+{
+    const char * bytes = [self cStringUsingEncoding: NSUTF8StringEncoding];
+    NSUInteger length = strlen(bytes);
+    unsigned char * r = (unsigned char *) malloc(length / 2 + 1);
+    unsigned char * index = r;
+
+    while ((*bytes) && (*(bytes +1))) {
+        *index = strToChar(*bytes, *(bytes +1));
+        index++;
+        bytes+=2;
+    }
+    *index = '\0';
+
+    NSData * result = [NSData dataWithBytes: r length: length / 2];
+    free(r);
+
+    return result;
+}
+
+@end
+
 typedef struct {
   uint32_t dwLowDateTime;
   uint32_t dwHighDateTime;
@@ -88,7 +127,7 @@ struct GlobalObjectId {
   FILETIME                  CreationTime;
   uint8_t                   X[8];
   uint32_t                  Size;
-  uint8_t*                  Data;
+  uint8_t Data[0];
 };
 
 @implementation SOGoMailObject (ActiveSync)
@@ -115,39 +154,40 @@ struct GlobalObjectId {
 //
 // The GlobalObjId is documented here: http://msdn.microsoft.com/en-us/library/ee160198(v=EXCHG.80).aspx
 //
+
 - (NSData *) _computeGlobalObjectIdFromEvent: (iCalEvent *) event
 {
   NSData *binPrefix, *globalObjectId, *uidAsASCII;
   NSString *prefix, *uid;
-
-  struct GlobalObjectId newGlobalId;
+  struct GlobalObjectId *newGlobalId;
   const char *bytes;
-  
+
+  uid = [event uid];
+  uidAsASCII = [uid dataUsingEncoding: NSASCIIStringEncoding];
+  newGlobalId = (struct GlobalObjectId*)calloc(sizeof(uint8_t), sizeof(struct GlobalObjectId) + 0x0c + [uidAsASCII length]);
+
   prefix = @"040000008200e00074c5b7101a82e008";
 
   // dataPrefix is "vCal-Uid %x01 %x00 %x00 %x00"
   uint8_t dataPrefix[] = { 0x76, 0x43, 0x61, 0x6c, 0x2d, 0x55, 0x69, 0x64, 0x01, 0x00, 0x00, 0x00 };
-  uid = [event uid];
 
   binPrefix = [prefix convertHexStringToBytes];
-  [binPrefix getBytes: &newGlobalId.ByteArrayID];
-  [self _setInstanceDate: &newGlobalId
+  [binPrefix getBytes: &newGlobalId->ByteArrayID];
+  [self _setInstanceDate: newGlobalId
                 fromDate: [event recurrenceId]];
-  uidAsASCII = [uid dataUsingEncoding: NSASCIIStringEncoding];
   bytes = [uidAsASCII bytes];
 
   // 0x0c is the size of our dataPrefix
-  newGlobalId.Size = 0x0c + [uidAsASCII length];
-  newGlobalId.Data = malloc(newGlobalId.Size * sizeof(uint8_t));
-  memcpy(newGlobalId.Data, dataPrefix, 0x0c);
-  memcpy(newGlobalId.Data + 0x0c, bytes, newGlobalId.Size - 0x0c);
+  newGlobalId->Size = 0x0c + [uidAsASCII length];
+  memcpy(newGlobalId->Data, dataPrefix, 0x0c);
+  memcpy(newGlobalId->Data + 0x0c, bytes, newGlobalId->Size - 0x0c);
 
-  globalObjectId = [[NSData alloc] initWithBytes: &newGlobalId  length: 40 + newGlobalId.Size*sizeof(uint8_t)];
-  free(newGlobalId.Data);
-  
+  globalObjectId = [[NSData alloc] initWithBytes: newGlobalId  length: 40 + newGlobalId->Size*sizeof(uint8_t)];
+  free(newGlobalId);
   return [globalObjectId autorelease];
 }
 
+
 //
 // For debugging purposes...
 //
@@ -216,6 +256,8 @@ struct GlobalObjectId {
   NSEnumerator *e;
   NSData *d;
 
+  BOOL ignore;
+
   textParts = [self fetchPlainTextParts];
   e = [textParts keyEnumerator];
   plainKey = nil;
@@ -224,6 +266,27 @@ struct GlobalObjectId {
 
   while ((key = [e nextObject]))
     {
+      // Walk the hierarchy up and check whether parents are of type mulipart - i.e. ignore message/rfc822
+      if ([key countOccurrencesOfString: @"."] > 0)
+        {
+          NSMutableArray *a;
+
+          a = [[[key componentsSeparatedByString: @"."] mutableCopy] autorelease];
+          ignore = NO;
+
+          while ([a count] > 0)
+            {
+              [a removeLastObject];
+              part = [self lookupInfoForBodyPart: [a componentsJoinedByString: @"."]];
+
+              if (![[part valueForKey: @"type"] isEqualToString: @"multipart"])
+                ignore = YES;
+            }
+
+          if (ignore)
+            continue;
+        }
+
       part = [self lookupInfoForBodyPart: key];
       type = [part valueForKey: @"type"];
       subtype = [part valueForKey: @"subtype"];
@@ -355,18 +418,13 @@ struct GlobalObjectId {
 
           if (s)
             {
-              // We sanitize the content immediately, in case we have non-UNICODE safe
-              // characters that would be re-encoded later in HTML entities and thus,
-              // ignore afterwards.
-              s = [s safeString];
-
               body = [s dataUsingEncoding: NSUTF8StringEncoding];
             }
 
           mimeType = [NGMimeType mimeType: [[thePart contentType] type]
                                   subType: [[thePart contentType] subType]
                                parameters: [NSDictionary dictionaryWithObject: @"utf-8"  forKey: @"charset"]];
-          [thePart setHeader: mimeType  forKey: @"content-type"];
+          [(NGMimeBodyPart *)thePart setHeader: mimeType  forKey: @"content-type"];
           
           fdata = [[NGMimeFileData alloc] initWithBytes: [body bytes]
                                                  length: [body length]];
@@ -415,18 +473,131 @@ struct GlobalObjectId {
   return d;
 }
 
+//
+//
+//
+- (BOOL) _sanitinizeNeeded: (NSArray *) theParts
+{
+  NSString *encoding, *charset;
+  int i;
+
+  for (i = 0; i < [theParts count]; i++)
+    {
+      encoding = [[theParts objectAtIndex: i ] objectForKey: @"encoding"];
+      charset = [[[theParts objectAtIndex: i ] objectForKey: @"parameterList"] objectForKey: @"charset"];
+      if (encoding && ([encoding caseInsensitiveCompare: @"7bit"] == NSOrderedSame || [encoding caseInsensitiveCompare: @"8bit"] == NSOrderedSame) &&
+          charset  && ![charset caseInsensitiveCompare: @"utf-8"] == NSOrderedSame
+                   && ![charset caseInsensitiveCompare: @"us-ascii"] == NSOrderedSame)
+        {
+          return YES;
+        }
+
+      if ([self _sanitinizeNeeded: [[theParts objectAtIndex: i ] objectForKey: @"parts"]])
+          return YES;
+   }
+
+   return NO;
+}
+
+
+- (BOOL) _isSigned: (NSDictionary *) thePart
+{
+  NSMutableDictionary *currentPart;
+  NSArray *subparts;
+  NSString *type, *subtype;
+  NSUInteger i;
+
+  type = [[thePart objectForKey: @"type"] lowercaseString];
+  subtype = [[thePart objectForKey: @"subtype"] lowercaseString];
+
+  if ([type isEqualToString: @"multipart"])
+    {
+      if ([subtype isEqualToString: @"signed"])
+        return YES;
+
+      subparts = [thePart objectForKey: @"parts"];
+      for (i = 0; i < [subparts count]; i++)
+        {
+          currentPart = [subparts objectAtIndex: i];
+          if ([self _isSigned: currentPart])
+            return YES;
+        }
+    }
+
+  return NO;
+}
+
+- (BOOL) _isSmimeEncrypted: (NSDictionary *) thePart
+{
+  NSMutableDictionary *currentPart;
+  NSArray *subparts;
+  NSString *type, *subtype;
+  NSUInteger i;
+
+  type = [[thePart objectForKey: @"type"] lowercaseString];
+  subtype = [[thePart objectForKey: @"subtype"] lowercaseString];
+
+  if ([type isEqualToString: @"multipart"])
+    {
+      subparts = [thePart objectForKey: @"parts"];
+      for (i = 0; i < [subparts count]; i++)
+        {
+          currentPart = [subparts objectAtIndex: i];
+          if ([self _isSmimeEncrypted: currentPart])
+            return YES;
+        }
+    }
+  else if ([type isEqualToString: @"application"] && ([subtype isEqualToString: @"pkcs7-mime"] ||
+           [subtype isEqualToString: @"x-pkcs7-mime"]))
+    return YES;
+
+  return NO;
+}
+
+- (BOOL) _isPGP: (NSDictionary *) thePart
+{
+  NSMutableDictionary *currentPart;
+  NSArray *subparts;
+  NSString *type, *subtype, *protocol;
+  NSUInteger i;
+
+  type = [[thePart objectForKey: @"type"] lowercaseString];
+  subtype = [[thePart objectForKey: @"subtype"] lowercaseString];
+  protocol = [[[thePart objectForKey: @"parameterList"] objectForKey: @"protocol"] lowercaseString];
+
+  if ([type isEqualToString: @"multipart"])
+    {
+      if (([protocol isEqualToString: @"application/pgp-signature"] || [protocol isEqualToString: @"application/pgp-encrypted"]))
+        return YES;
+
+      subparts = [thePart objectForKey: @"parts"];
+      for (i = 0; i < [subparts count]; i++)
+        {
+          currentPart = [subparts objectAtIndex: i];
+          if ([self _isPGP: currentPart])
+            return YES;
+        }
+    }
+  else if ([type isEqualToString: @"application"] && [subtype isEqualToString: @"pgp-encrypted"])
+    return YES;
+
+  return NO;
+}
+
 //
 //
 //
 - (NSData *) _preferredBodyDataUsingType: (int) theType
+                              mimeSupport: (int) theMimeSupport
                               nativeType: (int *) theNativeType
 {
   NSString *type, *subtype, *encoding;
   NSData *d;
+  BOOL isSMIME, sanitinizeNeeded;
   
   type = [[[self bodyStructure] valueForKey: @"type"] lowercaseString];
   subtype = [[[self bodyStructure] valueForKey: @"subtype"] lowercaseString];
-
+  isSMIME = NO;
   d = nil;
   
   // We determine the native type
@@ -437,10 +608,18 @@ struct GlobalObjectId {
   else if ([type isEqualToString: @"multipart"])
     *theNativeType = 4;
 
-  // We get the right part based on the preference
-  if (theType == 1 || theType == 2)
+  if (([self _isSigned: [self bodyStructure]] ||
+       [self _isSmimeEncrypted: [self bodyStructure]] ||
+       [self _isPGP: [self bodyStructure]]) && theMimeSupport > 0)
     {
-      if ([type isEqualToString: @"text"])
+      *theNativeType = 4;
+      isSMIME = YES;
+    }
+
+  // We get the right part based on the preference
+  if ((theType == 1 || theType == 2) && !isSMIME)
+    {
+      if ([type isEqualToString: @"text"] && ![subtype isEqualToString: @"calendar"])
         {
           NSString *s, *charset;
           
@@ -479,12 +658,12 @@ struct GlobalObjectId {
           d = [self _preferredBodyDataInMultipartUsingType: theType nativeTypeFound: theNativeType];
         }
     }
-  else if (theType == 4)
+  else if (theType == 4 || isSMIME)
     {
-      // We sanitize the content *ONLY* for Outlook clients and if the content-transfer-encoding is 8bit. Outlook has strange issues
-      // with quoted-printable/base64 encoded text parts. It just doesn't decode them.
-      encoding = [[self lookupInfoForBodyPart: @""] objectForKey: @"encoding"];
-      if ([[context objectForKey: @"DeviceType"] isEqualToString: @"WindowsOutlook15"] || ([encoding caseInsensitiveCompare: @"8bit"] == NSOrderedSame))
+      // We sanitize the content if the content-transfer-encoding is 8bit and charset is not utf-8 or us-ascii.
+      sanitinizeNeeded = [self _sanitinizeNeeded: [NSArray arrayWithObject: [self bodyStructure]]];
+
+      if (sanitinizeNeeded && !isSMIME)
         d = [self _sanitizedMIMEMessage];
       else
         d = [self content];
@@ -493,51 +672,140 @@ struct GlobalObjectId {
   return d;
 }
 
+
+- (NSString *) _getNormalizedSubject
+{
+  NSString *subject;
+  NSUInteger colIdx;
+  NSString *stringValue;
+
+  subject = [[self subject] decodedHeader];
+
+  colIdx = [subject rangeOfString: @":" options:NSBackwardsSearch].location;
+  if (colIdx != NSNotFound && colIdx + 1 < [subject length])
+    stringValue = [[subject substringFromIndex: colIdx + 1] stringByTrimmingLeadSpaces];
+  else
+    stringValue = subject;
+
+  if (!stringValue)
+    stringValue = @"";
+
+  return stringValue;
+}
+
+
+- (NSString *) _truncateContent: (NSString *) theContent
+                          limit: (int) theLimit
+                      truncated: (int *) wasTruncated
+{
+  if ([theContent length] > theLimit)
+    {
+      int i, len;
+
+      theContent = [theContent substringToIndex: theLimit];
+      *wasTruncated = 1;
+
+      // We search for the first "space" character starting from the
+      // end and we truncate the string once more. We do this to avoid
+      // truncating the content in the middle of a XML entity
+      len = theLimit-1;
+
+      for (i = len; i >= 0; i--)
+        {
+          if (isspace([theContent characterAtIndex: i]))
+            break;
+        }
+
+      return [theContent substringToIndex: i];
+    }
+
+  *wasTruncated = 0;
+  return theContent;
+}
+
+//
+//
+//
+- (iCalCalendar *) _calendarFromParts: (NSArray *) parts
+			       parent: (id) parent
+{
+  NSDictionary *part;
+  id bodyPart;
+  int i;
+
+  for (i = 0; i < [parts count]; i++)
+    {
+      part = [parts objectAtIndex: i];
+      bodyPart = [parent lookupImap4BodyPartKey: [NSString stringWithFormat: @"%d", i+1]
+				      inContext: self->context];
+
+      if ([[part objectForKey: @"type"] isEqualToString: @"text"] &&
+	  [[part objectForKey: @"subtype"] isEqualToString: @"calendar"])
+	{
+	  if (bodyPart)
+	    {
+	      iCalCalendar *calendar;
+	      NSData *calendarData;
+
+	      calendarData = [bodyPart fetchBLOBWithPeek: YES];
+	      calendar = nil;
+
+	      NS_DURING
+		calendar = [iCalCalendar parseSingleFromSource: calendarData];
+	      NS_HANDLER
+		calendar = nil;
+	      NS_ENDHANDLER
+
+	      return calendar;
+	    }
+	}
+      else if ([[part objectForKey: @"type"] isEqualToString: @"multipart"])
+	return [self _calendarFromParts: [part objectForKey: @"parts"]  parent: bodyPart];
+    }
+
+  return nil;
+}
+
 //
 //
 //
 - (iCalCalendar *) calendarFromIMIPMessage
 {
-  NSDictionary *part;
+  NSString *type, *subtype;
   NSArray *parts;
-  int i;
+
+  type = [[[self bodyStructure] valueForKey: @"type"] lowercaseString];
+  subtype = [[[self bodyStructure] valueForKey: @"subtype"] lowercaseString];
+
+  // process mail of type text/calendar
+  if ([type isEqualToString: @"text"] && [subtype isEqualToString: @"calendar"])
+    {
+      iCalCalendar *calendar;
+      NSString *encoding;
+      NSData *calendarData;
+
+      encoding = [[[self bodyStructure] valueForKey: @"encoding"] lowercaseString];
+      calendarData = [[self fetchPlainTextParts] objectForKey: @""];
+
+      if ([encoding caseInsensitiveCompare: @"base64"] == NSOrderedSame)
+        calendarData = [calendarData dataByDecodingBase64];
+      else if ([encoding caseInsensitiveCompare: @"quoted-printable"] == NSOrderedSame)
+        calendarData = [calendarData dataByDecodingQuotedPrintableTransferEncoding];
+
+      NS_DURING
+        calendar = [iCalCalendar parseSingleFromSource: calendarData];
+      NS_HANDLER
+        calendar = nil;
+      NS_ENDHANDLER
+
+      return calendar;
+    }
 
   // We check if we have at least 2 parts and if one of them is a text/calendar
   parts = [[self bodyStructure] objectForKey: @"parts"];
 
   if ([parts count] > 1)
-    {
-      for (i = 0; i < [parts count]; i++)
-        {
-          part = [parts objectAtIndex: i];
-          
-          if ([[part objectForKey: @"type"] isEqualToString: @"text"] &&
-              [[part objectForKey: @"subtype"] isEqualToString: @"calendar"])
-            {
-              id bodyPart;
-
-              bodyPart = [self lookupImap4BodyPartKey: [NSString stringWithFormat: @"%d", i+1]
-                                            inContext: self->context];
-
-              if (bodyPart)
-                {
-                  iCalCalendar *calendar;
-                  NSData *calendarData;
-                  
-                  calendarData = [bodyPart fetchBLOB];
-                  calendar = nil;
-
-                  NS_DURING
-                    calendar = [iCalCalendar parseSingleFromSource: calendarData];
-                  NS_HANDLER
-                    calendar = nil;
-                  NS_ENDHANDLER
-
-                  return calendar;
-                }
-            }
-        }
-    }
+    return [self _calendarFromParts: parts  parent: self];
 
   return nil;
 }
@@ -549,16 +817,17 @@ struct GlobalObjectId {
 {
   NSData *d, *globalObjId;
   NSArray *attachmentKeys;
-  NSMutableString *s;
-
-  uint32_t v;
-  NSString *p;
-  
-  id value;
-
   iCalCalendar *calendar;
+  NSString *p;
+  NSMutableString *s;
+  id value;
       
-  int preferredBodyType, nativeBodyType;
+  int preferredBodyType, mimeSupport, mimeTruncation, nativeBodyType;
+  uint32_t v;
+
+  preferredBodyType = [[context objectForKey: @"BodyPreferenceType"] intValue];
+  mimeSupport = [[context objectForKey: @"MIMESupport"] intValue];
+  mimeTruncation = [[context objectForKey: @"MIMETruncation"] intValue];
 
   s = [NSMutableString string];
 
@@ -578,7 +847,7 @@ struct GlobalObjectId {
   if (value)
     {
       [s appendFormat: @"<Subject xmlns=\"Email:\">%@</Subject>", [value activeSyncRepresentationInContext: context]];
-      [s appendFormat: @"<ThreadTopic xmlns=\"Email:\">%@</ThreadTopic>", [value activeSyncRepresentationInContext: context]];
+      [s appendFormat: @"<ThreadTopic xmlns=\"Email:\">%@</ThreadTopic>", [[self _getNormalizedSubject] activeSyncRepresentationInContext: context]];
     }
 
   // DateReceived
@@ -596,7 +865,11 @@ struct GlobalObjectId {
     
   // Importance
   v = 0x1;
-  p = [[self mailHeaders] objectForKey: @"x-priority"];
+  value = [[self mailHeaders] objectForKey: @"x-priority"];
+  if ([value isKindOfClass: [NSArray class]])
+    p = [value lastObject];
+  else
+    p = value;
 
   if (p) 
     {
@@ -607,7 +880,12 @@ struct GlobalObjectId {
     }
   else
     {
-      p = [[self mailHeaders] objectForKey: @"importance"];
+      value = [[self mailHeaders] objectForKey: @"importance"];
+      if ([value isKindOfClass: [NSArray class]])
+        p = [value lastObject];
+      else
+        p = value;
+
       if ([p hasPrefix: @"High"]) v = 0x2;
       else if ([p hasPrefix: @"Low"]) v = 0x0;
     }
@@ -676,18 +954,18 @@ struct GlobalObjectId {
 
       [s appendFormat: @"<AllDayEvent xmlns=\"Email:\">%d</AllDayEvent>", ([event isAllDay] ? 1 : 0)];
 
-      // StartTime -- http://msdn.microsoft.com/en-us/library/ee157132(v=exchg.80).aspx
+      // StartTime -- https://msdn.microsoft.com/en-us/library/ee203365%28v=exchg.80%29.aspx
       if ([event startDate])
-        [s appendFormat: @"<StartTime xmlns=\"Email:\">%@</StartTime>", [[event startDate] activeSyncRepresentationWithoutSeparatorsInContext: context]];
+        [s appendFormat: @"<StartTime xmlns=\"Email:\">%@</StartTime>", [[event startDate] activeSyncRepresentationInContext: context]];
       
       if ([event timeStampAsDate])
-        [s appendFormat: @"<DTStamp xmlns=\"Email:\">%@</DTStamp>", [[event timeStampAsDate] activeSyncRepresentationWithoutSeparatorsInContext: context]];
+        [s appendFormat: @"<DTStamp xmlns=\"Email:\">%@</DTStamp>", [[event timeStampAsDate] activeSyncRepresentationInContext: context]];
       else if ([event created])
-        [s appendFormat: @"<DTStamp xmlns=\"Email:\">%@</DTStamp>", [[event created] activeSyncRepresentationWithoutSeparatorsInContext: context]];
+        [s appendFormat: @"<DTStamp xmlns=\"Email:\">%@</DTStamp>", [[event created] activeSyncRepresentationInContext: context]];
       
-      // EndTime -- http://msdn.microsoft.com/en-us/library/ee157945(v=exchg.80).aspx
+      // EndTime -- https://msdn.microsoft.com/en-us/library/ee158628(v=exchg.80).aspx
       if ([event endDate])
-        [s appendFormat: @"<EndTime xmlns=\"Email:\">%@</EndTime>", [[event endDate] activeSyncRepresentationWithoutSeparatorsInContext: context]];
+        [s appendFormat: @"<EndTime xmlns=\"Email:\">%@</EndTime>", [[event endDate] activeSyncRepresentationInContext: context]];
       
       // FIXME: Single appointment - others are not supported right now
       [s appendFormat: @"<InstanceType xmlns=\"Email:\">%d</InstanceType>", 0];
@@ -735,7 +1013,9 @@ struct GlobalObjectId {
       [s appendFormat: @"<GlobalObjId xmlns=\"Email:\">%@</GlobalObjId>", [globalObjId activeSyncRepresentationInContext: context]];
 
       // We set the right message type - we must set AS version to 14.1 for this
-      [s appendFormat: @"<MeetingMessageType xmlns=\"Email2:\">%d</MeetingMessageType>", 1];
+      if ([[context objectForKey: @"ASProtocolVersion"] floatValue] >= 14.1)
+        [s appendFormat: @"<MeetingMessageType xmlns=\"Email2:\">%d</MeetingMessageType>", 1];
+
       [s appendString: @"</MeetingRequest>"];
 
       // ContentClass
@@ -744,7 +1024,13 @@ struct GlobalObjectId {
   else
     {
       // MesssageClass and ContentClass
-      [s appendFormat: @"<MessageClass xmlns=\"Email:\">%@</MessageClass>", @"IPM.Note"];
+      if ([self _isSigned: [self bodyStructure]])
+        [s appendFormat: @"<MessageClass xmlns=\"Email:\">%@</MessageClass>", @"IPM.Note.SMIME.MultipartSigned"];
+      else if ([self _isSmimeEncrypted: [self bodyStructure]])
+        [s appendFormat: @"<MessageClass xmlns=\"Email:\">%@</MessageClass>", @"IPM.Note.SMIME"];
+      else
+        [s appendFormat: @"<MessageClass xmlns=\"Email:\">%@</MessageClass>", @"IPM.Note"];
+
       [s appendFormat: @"<ContentClass xmlns=\"Email:\">%@</ContentClass>", @"urn:content-classes:message"];
     }
 
@@ -758,17 +1044,82 @@ struct GlobalObjectId {
   [s appendFormat: @"<InternetCPID xmlns=\"Email:\">%@</InternetCPID>", @"65001"];
           
   // Body - namespace 17
-  preferredBodyType = [[context objectForKey: @"BodyPreferenceType"] intValue];
-
   nativeBodyType = 1;
-  d = [self _preferredBodyDataUsingType: preferredBodyType  nativeType: &nativeBodyType];
+  d = [self _preferredBodyDataUsingType: preferredBodyType  mimeSupport: mimeSupport  nativeType: &nativeBodyType];
+
+  if (calendar && !d)
+    {
+      WOApplication *app;
+      SOGoAptMailNotification *p;
+      NSString *pageName;
+
+      nativeBodyType = 2;
+
+      /* get WOApplication instance */
+      app = [WOApplication application];
+
+      /* create page name */
+      pageName = [NSString stringWithFormat: @"SOGoAptMail%@", @"Invitation"];
+      /* construct message content */
+      p = [app pageWithName: pageName inContext: context];
+      [p setApt: (iCalEvent *)  [[calendar events] lastObject]];
+
+      if ([[ [[calendar events] lastObject] organizer] cn] && [[[ [[calendar events] lastObject] organizer] cn] length])
+        {
+          [p setOrganizerName: [[ [[calendar events] lastObject] organizer] cn]];
+        }
+      else
+        {
+          [p setOrganizerName: [[SOGoUser userWithLogin: owner] cn]];
+        }
+
+      if (preferredBodyType == 1 && nativeBodyType == 2)
+        d = [[[p getBody] htmlToText] dataUsingEncoding: NSUTF8StringEncoding];
+      else
+        {
+          preferredBodyType = 2;
+          d = [[p getBody] dataUsingEncoding: NSUTF8StringEncoding];
+        }
+
+    }
   
   if (d)
     {
+      NSMutableData *sanitizedData;
       NSString *content;
       int len, truncated;
-      
-      content = [[NSString alloc] initWithData: d  encoding: NSUTF8StringEncoding];
+
+      // Outlook fails to decode quoted-printable (see #3082) if lines are not termined by CRLF.
+      const char *bytes;
+      char *mbytes;
+      int mlen;
+
+      len = [d length];
+      mlen = 0;
+
+      sanitizedData = [NSMutableData dataWithLength: len*2];
+
+      bytes = [d bytes];
+      mbytes = [sanitizedData mutableBytes];
+
+      while (len > 0)
+        {
+          if (*bytes == '\n' && *(bytes-1) != '\r' && mlen > 0)
+            {
+              *mbytes = '\r';
+              mbytes++;
+              mlen++;
+            }
+
+          *mbytes = *bytes;
+          mbytes++; bytes++;
+          len--;
+          mlen++;
+        }
+
+      [sanitizedData setLength: mlen];
+
+      content = [[NSString alloc] initWithData: sanitizedData  encoding: NSUTF8StringEncoding];
 
       // FIXME: This is a hack. We should normally avoid doing this as we might get
       // broken encodings. We should rather tell that the data was truncated and expect
@@ -778,49 +1129,92 @@ struct GlobalObjectId {
       // for an "interesting" discussion around this.
       //
       if (!content)
-        content = [[NSString alloc] initWithData: d  encoding: NSISOLatin1StringEncoding];
+        content = [[NSString alloc] initWithData: sanitizedData  encoding: NSISOLatin1StringEncoding];
       
       AUTORELEASE(content);
       
       content = [content activeSyncRepresentationInContext: context];
-      truncated = 0;
-    
       len = [content length];
-      
-      if ([[[context request] headerForKey: @"MS-ASProtocolVersion"] isEqualToString: @"2.5"])
+      truncated = 0;
+
+      // We handle MIMETruncation
+      switch (mimeTruncation)
+        {
+        case 0:
+          {
+            content = @"";
+            len = 0; truncated = 1;
+          }
+          break;
+        case 1:
+          content = [self _truncateContent: content  limit: 4096  truncated: &truncated];
+          len = [content length];
+          break;
+        case 2:
+          content = [self _truncateContent: content  limit: 5120  truncated: &truncated];
+          len = [content length];
+          break;
+        case 3:
+          content = [self _truncateContent: content  limit: 7168  truncated: &truncated];
+          len = [content length];
+          break;
+        case 4:
+          content = [self _truncateContent: content  limit: 10240  truncated: &truncated];
+          len = [content length];
+          break;
+        case 5:
+          content = [self _truncateContent: content  limit: 20480  truncated: &truncated];
+          len = [content length];
+          break;
+        case 6:
+          content = [self _truncateContent: content  limit: 51200  truncated: &truncated];
+          len = [content length];
+          break;
+        case 7:
+          content = [self _truncateContent: content  limit: 102400  truncated: &truncated];
+          len = [content length];
+          break;
+        case 8:
+        default:
+          truncated = 0;
+        }
+
+      if ([[context objectForKey: @"ASProtocolVersion"] isEqualToString: @"2.5"])
         {
           [s appendFormat: @"<Body xmlns=\"Email:\">%@</Body>", content];
           [s appendFormat: @"<BodyTruncated xmlns=\"Email:\">%d</BodyTruncated>", truncated];
         }
       else
-       {
+        {
           [s appendString: @"<Body xmlns=\"AirSyncBase:\">"];
 
-          // Set the correct type if client requested text/html but we got text/plain
+          // Set the correct type if client requested text/html but we got text/plain.
+          // For s/mime mails type is always 4 if mimeSupport is 1 or 2.
           if (preferredBodyType == 2 && nativeBodyType == 1)
              [s appendString: @"<Type>1</Type>"];
+          else if (([self _isSigned: [self bodyStructure]] ||
+                    [self _isSmimeEncrypted: [self bodyStructure]] ||
+                    [self _isPGP: [self bodyStructure]]) && mimeSupport > 0)
+             [s appendString: @"<Type>4</Type>"];
           else
              [s appendFormat: @"<Type>%d</Type>", preferredBodyType];
 
           [s appendFormat: @"<Truncated>%d</Truncated>", truncated];
           [s appendFormat: @"<Preview></Preview>"];
-
-          if (!truncated)
-            {
-              [s appendFormat: @"<Data>%@</Data>", content];
-              [s appendFormat: @"<EstimatedDataSize>%d</EstimatedDataSize>", len];
-            }
+          [s appendFormat: @"<Data>%@</Data>", content];
+          [s appendFormat: @"<EstimatedDataSize>%d</EstimatedDataSize>", len];
           [s appendString: @"</Body>"];
        }
     }
-  
+
   // Attachments -namespace 16
   attachmentKeys = [self fetchFileAttachmentKeys];
-  if ([attachmentKeys count])
+
+  if ([attachmentKeys count] && !([self _isSigned: [self bodyStructure]]))
     {
       int i;
 
-      if ([[[context request] headerForKey: @"MS-ASProtocolVersion"] isEqualToString: @"2.5"])
+      if ([[context objectForKey: @"ASProtocolVersion"] isEqualToString: @"2.5"])
         [s appendString: @"<Attachments xmlns=\"Email:\">"];
       else
         [s appendString: @"<Attachments xmlns=\"AirSyncBase:\">"];
@@ -835,18 +1229,24 @@ struct GlobalObjectId {
           // FileReference must be a unique identifier across the whole store. We use the following structure:
           // mail/<foldername>/<message UID/<pathofpart>
           // mail/INBOX/2          
-          if ([[[context request] headerForKey: @"MS-ASProtocolVersion"] isEqualToString: @"2.5"])
+          if ([[context objectForKey: @"ASProtocolVersion"] isEqualToString: @"2.5"])
             [s appendFormat: @"<AttName>mail/%@/%@/%@</AttName>", [[[self container] relativeImap4Name] stringByEscapingURL], [self nameInContainer], [value objectForKey: @"path"]];
           else
             [s appendFormat: @"<FileReference>mail/%@/%@/%@</FileReference>", [[[self container] relativeImap4Name] stringByEscapingURL], [self nameInContainer], [value objectForKey: @"path"]];
 
-          if ([[[context request] headerForKey: @"MS-ASProtocolVersion"] isEqualToString: @"2.5"])
+          if ([[context objectForKey: @"ASProtocolVersion"] isEqualToString: @"2.5"])
             {
               [s appendFormat: @"<AttMethod>%d</AttMethod>", 1];
               [s appendFormat: @"<AttSize>%d</AttSize>", [[value objectForKey: @"size"] intValue]];
             }
           else
             {
+              if ([[value objectForKey: @"bodyId"] length])
+                {
+                  [s appendFormat: @"<ContentId>%@</ContentId>", [[value objectForKey: @"bodyId"] activeSyncRepresentationInContext: context]];
+                  [s appendFormat: @"<IsInline>%d</IsInline>", 1];
+                }
+
               [s appendFormat: @"<Method>%d</Method>", 1]; // See: http://msdn.microsoft.com/en-us/library/ee160322(v=exchg.80).aspx
               [s appendFormat: @"<EstimatedDataSize>%d</EstimatedDataSize>", [[value objectForKey: @"size"] intValue]];
               //[s appendFormat: @"<IsInline>%d</IsInline>", 1];
@@ -883,9 +1283,36 @@ struct GlobalObjectId {
       [s appendFormat: @"</Categories>"];
     }
   
+    if ([[context objectForKey: @"ASProtocolVersion"] floatValue] >= 14.0)
+    {
+      id value;
+      NSString *reference;
+
+      value = [[self mailHeaders] objectForKey: @"references"];
+
+      if ([value isKindOfClass: [NSArray class]])
+         reference = [[[value objectAtIndex: 0] componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] objectAtIndex: 0];
+      else
+         reference = [[value componentsSeparatedByCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] objectAtIndex: 0];
+
+      if ([reference length] > 0)
+        [s appendFormat: @"<ConversationId xmlns=\"Email2:\">%@</ConversationId>", [[reference dataUsingEncoding: NSUTF8StringEncoding] activeSyncRepresentationInContext: context]];
+      else if ([[self inReplyTo] length] > 0)
+        [s appendFormat: @"<ConversationId xmlns=\"Email2:\">%@</ConversationId>", [[[self inReplyTo] dataUsingEncoding: NSUTF8StringEncoding] activeSyncRepresentationInContext: context]];
+      else if ([[self messageId] length] > 0)
+        [s appendFormat: @"<ConversationId xmlns=\"Email2:\">%@</ConversationId>", [[[self messageId] dataUsingEncoding: NSUTF8StringEncoding] activeSyncRepresentationInContext: context]];
+
+      if ([self replied])
+        [s appendFormat: @"<LastVerbExecuted xmlns=\"Email2:\">%d</LastVerbExecuted>", 1];
+      else if ([self forwarded])
+        [s appendFormat: @"<LastVerbExecuted xmlns=\"Email2:\">%d</LastVerbExecuted>", 3];
+      else
+        [s appendFormat: @"<LastVerbExecuted xmlns=\"Email2:\">%d</LastVerbExecuted>", 0];
+    }
+
   // FIXME - support these in the future
-  //[s appendString: @"<ConversationId xmlns=\"Email2:\">foobar</ConversationId>"];
   //[s appendString: @"<ConversationIndex xmlns=\"Email2:\">zot=</ConversationIndex>"];
+
   
   // NativeBodyType -- http://msdn.microsoft.com/en-us/library/ee218276(v=exchg.80).aspx
   // This is a required child element.
