@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2005-2014 Inverse inc.
-  Copyright (C) 2004-2005 SKYRIX Software AG
+  Copyright (C) 2005-2016 Inverse inc.
 
   This file is part of SOGo
 
@@ -20,21 +19,15 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSDebug.h>
-#import <Foundation/NSData.h>
-#import <Foundation/NSDate.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSProcessInfo.h>
-#import <Foundation/NSRunLoop.h>
-#import <Foundation/NSURL.h>
 
 #import <GDLAccess/EOAdaptorChannel.h>
 #import <GDLContentStore/GCSChannelManager.h>
 #import <GDLContentStore/GCSFolderManager.h>
+#import <GDLContentStore/GCSFolderType.h>
 #import <GDLContentStore/GCSAlarmsFolder.h>
 #import <GDLContentStore/GCSSessionsFolder.h>
 
+#import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/SoClassSecurityInfo.h>
 #import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest+So.h>
@@ -58,7 +51,6 @@
 #import <SOGo/SOGoPublicBaseFolder.h>
 #import <SOGo/SOGoProductLoader.h>
 #import <SOGo/SOGoProxyAuthenticator.h>
-#import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUser.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoWebAuthenticator.h>
@@ -123,7 +115,7 @@ static BOOL debugLeaks;
   vMemSizeLimit = [defaults vmemLimit];
   if (vMemSizeLimit > 0)
     [self logWithFormat: @"vmem size check enabled: shutting down app when "
-          @"vmem > %d MB", vMemSizeLimit];
+          @"vmem > %d MB. Currently at %d MB", vMemSizeLimit, [[NSProcessInfo processInfo] virtualMemorySize]/1048576];
 
   /* SoClass security declarations */
   sInfo = [self soClassSecurityInfo];
@@ -145,7 +137,9 @@ static BOOL debugLeaks;
   [$(@"SOGoFolder") soClass];
 
   /* load products */
-  [[SOGoProductLoader productLoader] loadAllProducts];
+  [[SOGoProductLoader productLoader] loadAllProducts: YES];
+  if (vMemSizeLimit > 0)
+    [self logWithFormat: @"All products loaded - current memory usage at %d MB", [[NSProcessInfo processInfo] virtualMemorySize]/1048576];
 }
 
 - (id) init
@@ -220,36 +214,94 @@ static BOOL debugLeaks;
   [cm releaseChannel: tc];
 }
 
+- (void) _checkQuickTableWithTypeName: typeName
+                               withCm: (GCSChannelManager *) cm
+                             tableURL: (NSString *) url
+{
+  GCSFolderType *type;
+  NSString *sql;
+  NSString *tableName;
+  EOAdaptorChannel *channel;
+
+  channel = [cm acquireOpenChannelForURL: [NSURL URLWithString: url]];
+
+  tableName = [NSString stringWithFormat: @"sogo_quick_%@", typeName];
+  sql = [NSString stringWithFormat: @"SELECT count(*) FROM %@",
+                  tableName];
+  if ([channel evaluateExpressionX: sql])
+    {
+      type = [GCSFolderType folderTypeWithName: typeName];
+      if (type)
+        {
+          sql = [type sqlQuickCreateWithTableName: tableName];
+          if (![channel evaluateExpressionX: sql])
+            [self logWithFormat: @"sogo quick table %@ successfully created!",
+                  tableName];
+        }
+    }
+  else
+    [channel cancelFetch];
+
+  [cm releaseChannel:channel];
+}
+
+
+//
+// If OCSStoreURL is defined, we also check for OCSAclURL, OCSCacheFolderURL
+// and we create the combined quick tables.
+//
 - (BOOL) _checkMandatoryTables
 {
   GCSChannelManager *cm;
   GCSFolderManager *fm;
-  NSString *urlStrings[] = {@"SOGoProfileURL", @"OCSFolderInfoURL", nil};
-  NSString **urlString;
-  NSString *value;
+  NSArray *urlStrings;
+  NSArray *quickTypeStrings;
+  NSString *tmp, *value;
   SOGoSystemDefaults *defaults;
-  BOOL ok;
+  NSEnumerator *e;
+  BOOL ok, combined;
 
   defaults = [SOGoSystemDefaults sharedSystemDefaults];
   ok = YES;
+
+  if ([GCSFolderManager singleStoreMode])
+    {
+      urlStrings = [NSArray arrayWithObjects: @"SOGoProfileURL", @"OCSFolderInfoURL", @"OCSStoreURL", @"OCSAclURL", @"OCSCacheFolderURL", nil];
+      quickTypeStrings = [NSArray arrayWithObjects: @"contact", @"appointment", nil];
+      combined = YES;
+    }
+  else
+    {
+      urlStrings = [NSArray arrayWithObjects: @"SOGoProfileURL", @"OCSFolderInfoURL", nil];
+      combined = NO;
+    }
+
   cm = [GCSChannelManager defaultChannelManager];
 
-  urlString = urlStrings;
-  while (ok && *urlString)
+  e = [urlStrings objectEnumerator];
+  while (ok && (tmp = [e nextObject]))
     {
-      value = [defaults stringForKey: *urlString];
+      value = [defaults stringForKey: tmp];
       if (value)
-	{
-	  [self _checkTableWithCM: cm tableURL: value andType: *urlString];
-	  urlString++;
-	}
+	  [self _checkTableWithCM: cm tableURL: value andType: tmp];
       else
 	{
-	  [self errorWithFormat: @"No value specified for '%@'", *urlString];
+	  [self errorWithFormat: @"No value specified for '%@'", tmp];
 	  ok = NO;
 	}
     }
 
+  if (combined)
+    {
+      e = [quickTypeStrings objectEnumerator];
+      while ((tmp = [e nextObject]))
+        {
+          [self _checkQuickTableWithTypeName: tmp
+                                      withCm: cm
+                                    tableURL: [defaults stringForKey: @"OCSFolderInfoURL"]];
+        }
+    }
+
   if (ok)
     {
       fm = [GCSFolderManager defaultFolderManager];
@@ -369,8 +421,12 @@ static BOOL debugLeaks;
              "GET" if no method was provided in the query path.
           */
           if ([_key length] > 0 && ![_key isEqualToString:@"favicon.ico"])
-            obj = [self lookupUser: _key inContext: _ctx];
-        }
+            {
+              obj = [self lookupUser: _key inContext: _ctx];
+              if (!obj && ![_key isEqualToString: @"public"])
+		obj = [self lookupUser: @"anonymous" inContext: _ctx];
+            }
+       }
     }
   else
     obj = nil;
@@ -425,8 +481,12 @@ static BOOL debugLeaks;
 {
   static NSArray *runLoopModes = nil;
   static BOOL debugOn = NO;
+
+  SOGoSystemDefaults *sd;
   WOResponse *resp;
   NSDate *startDate;
+  NSString *path;
+
   NSTimeInterval timeDelta;
 
   if (debugRequests)
@@ -436,6 +496,7 @@ static BOOL debugLeaks;
       startDate = [NSDate date];
     }
 
+  sd = [SOGoSystemDefaults sharedSystemDefaults];
   cache = [SOGoCache sharedCache];
 #ifdef GNUSTEP_BASE_LIBRARY
   if (debugLeaks)
@@ -450,6 +511,64 @@ static BOOL debugLeaks;
     }
 #endif
 
+  // We check for rate-limiting settings - ignore anything actually
+  // sent to /SOGo/ (so unauthenticated requests).
+  path = [_request requestHandlerPath];
+  if ([path length] && [sd maximumRequestCount] > 0)
+    {
+      NSDictionary *requestCount;
+      NSString *username;
+      NSRange r;
+
+      r = [path rangeOfString: @"/"];
+
+      // We handle /sogo1/Calendar/.../ and "sogo1" as paths
+      if (r.length)
+        username = [path substringWithRange: NSMakeRange(0, r.location)];
+      else
+        username = path;
+
+      requestCount = [cache requestCountForLogin: username];
+
+      if (requestCount)
+        {
+          unsigned int  current_time, start_time, delta, block_time, request_count;
+
+          current_time = [[NSCalendarDate date] timeIntervalSince1970];
+          start_time = [[requestCount objectForKey: @"InitialDate"] unsignedIntValue];
+          delta = current_time - start_time;
+
+          block_time = [sd requestBlockInterval];
+          request_count = [[requestCount objectForKey: @"RequestCount"] intValue];
+
+          if ( request_count >= [sd maximumRequestCount] &&
+               delta < [sd maximumRequestInterval] &&
+               delta <= block_time )
+            {
+              resp = [WOResponse responseWithRequest: _request];
+              [resp setStatus: 429];
+              return resp;
+            }
+
+          if (delta > block_time)
+            {
+              [cache setRequestCount: 1
+                            forLogin: username
+                            interval: current_time];
+            }
+          else
+            [cache setRequestCount: (request_count+1)
+                          forLogin: username
+                          interval: start_time];
+        }
+      else
+        {
+          [cache setRequestCount: 1
+                        forLogin: username
+                        interval: 0];
+        }
+    }
+
   resp = [super dispatchRequest: _request];
   [cache killCache];
 
