@@ -30,6 +30,7 @@
 #import <SOGo/NSArray+Utilities.h>
 #import <Mailer/SOGoMailBodyPart.h>
 #import <Mailer/SOGoMailObject.h>
+#import <Mailer/NSDictionary+Mail.h>
 
 #import "MAPIStoreTypes.h"
 #import "MAPIStoreMailMessage.h"
@@ -75,24 +76,24 @@
   ASSIGN (bodyPart, newBodyPart);
 }
 
-- (int) getPidTagAttachMethod: (void **) data
-                     inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachMethod: (void **) data
+                                      inMemCtx: (TALLOC_CTX *) memCtx
 {
   *data = MAPILongValue (memCtx, 0x00000001); // afByValue
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachTag: (void **) data
-                  inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachTag: (void **) data
+                                   inMemCtx: (TALLOC_CTX *) memCtx
 {
   *data = [[self mimeAttachTag] asBinaryInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachSize: (void **) data
-                   inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachSize: (void **) data
+                                    inMemCtx: (TALLOC_CTX *) memCtx
 {
   uint32_t longValue;
 
@@ -102,13 +103,13 @@
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagRecordKey: (void **) data
-                  inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagRecordKey: (void **) data
+                                   inMemCtx: (TALLOC_CTX *) memCtx
 {
   static char recordBytes[] = {0xd9, 0xd8, 0x11, 0xa3, 0xe2, 0x90, 0x18, 0x41,
                                0x9e, 0x04, 0x58, 0x46, 0x9d, 0x6d, 0x1b,
                                0x68};
-  
+
   *data = [[NSData dataWithBytes: recordBytes length: 16]
             asBinaryInMemCtx: memCtx];
 
@@ -117,31 +118,19 @@
 
 - (NSString *) _fileName
 {
-  NSString *fileName;
-  NSDictionary *parameters;
-
-  fileName = [[bodyInfo objectForKey: @"parameterList"]
-               objectForKey: @"name"];
-  if (!fileName)
-    {
-      parameters = [[bodyInfo objectForKey: @"disposition"]
-                     objectForKey: @"parameterList"];
-      fileName = [parameters objectForKey: @"filename"];
-    }
-
-  return fileName;
+  return [bodyInfo filename];
 }
 
-- (int) getPidTagAttachLongFilename: (void **) data
-                           inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachLongFilename: (void **) data
+                                            inMemCtx: (TALLOC_CTX *) memCtx
 {
   *data = [[self _fileName] asUnicodeInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachFilename: (void **) data
-                       inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachFilename: (void **) data
+                                        inMemCtx: (TALLOC_CTX *) memCtx
 {
   NSString *fileName, *baseName, *ext;
 
@@ -160,39 +149,52 @@
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagDisplayName: (void **) data
-                    inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagDisplayName: (void **) data
+                                     inMemCtx: (TALLOC_CTX *) memCtx
 {
-  *data = [[bodyInfo objectForKey: @"description"]
-            asUnicodeInMemCtx: memCtx];
+  NSString *fileName;
+
+  fileName = [self _fileName];
+  if ([fileName isEqualToString: @"sharing_metadata.xml"])
+    /* Required to disallow user from seeing the attachment by default */
+    *data = [@"sharing_metadata.xml" asUnicodeInMemCtx: memCtx];
+  else
+    *data = [[bodyInfo objectForKey: @"description"]
+              asUnicodeInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachContentId: (void **) data
-                        inMemCtx: (TALLOC_CTX *) memCtx
-{ 
+- (enum mapistore_error) getPidTagAttachContentId: (void **) data
+                                         inMemCtx: (TALLOC_CTX *) memCtx
+{
   *data = [[bodyInfo objectForKey: @"bodyId"]
             asUnicodeInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachMimeTag: (void **) data
-                      inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachMimeTag: (void **) data
+                                       inMemCtx: (TALLOC_CTX *) memCtx
 {
-  NSString *mimeTag;
+  NSString *mimeTag, *fileName;
+
+  fileName = [self _fileName];
+  if ([fileName isEqualToString: @"sharing_metadata.xml"])
+    /* Required by [MS-OXWSMSHR] Section 3.1.1 */
+    mimeTag = [NSString stringWithFormat: @"application/x-sharing-metadata-xml"];
+  else
+    mimeTag = [NSString stringWithFormat: @"%@/%@",
+                  [bodyInfo objectForKey: @"type"],
+                  [bodyInfo objectForKey: @"subtype"]];
 
-  mimeTag = [NSString stringWithFormat: @"%@/%@",
-                [bodyInfo objectForKey: @"type"],
-                [bodyInfo objectForKey: @"subtype"]];
   *data = [[mimeTag lowercaseString] asUnicodeInMemCtx: memCtx];
 
   return MAPISTORE_SUCCESS;
 }
 
-- (int) getPidTagAttachDataBinary: (void **) data
-                         inMemCtx: (TALLOC_CTX *) memCtx
+- (enum mapistore_error) getPidTagAttachDataBinary: (void **) data
+                                          inMemCtx: (TALLOC_CTX *) memCtx
 {
   *data = [[bodyPart fetchBLOBWithPeek: YES] asBinaryInMemCtx: memCtx];
 
