@@ -68,7 +68,7 @@ static GCSStringFormatter *stringFormatter = nil;
 + (void) initialize
 {
   NSUserDefaults *ud = [NSUserDefaults standardUserDefaults];
-  
+
   debugOn         = [ud boolForKey:@"GCSFolderDebugEnabled"];
   doLogStore      = [ud boolForKey:@"GCSFolderStoreDebugEnabled"];
 
@@ -92,13 +92,6 @@ static GCSStringFormatter *stringFormatter = nil;
   GCSFieldInfo *field;
   NSString *fieldName;
 
-  if (![_loc isNotNull])
-    {
-      [self errorWithFormat:@"missing quicktable parameter!"];
-      [self release];
-      return nil;
-    }
-  
   if ((self = [super init])) {
     folderManager  = [_fm    retain];
     folderInfo     = [_ftype retain];
@@ -107,6 +100,7 @@ static GCSStringFormatter *stringFormatter = nil;
     while ((field = [fields nextObject]))
       {
 	fieldName = [field columnName];
+
 	if (![fieldName isEqualToString: @"c_name"])
 	  [quickFieldNames addObject: fieldName];
       }
@@ -116,9 +110,11 @@ static GCSStringFormatter *stringFormatter = nil;
     while ((field = [fields nextObject]))
       {
 	fieldName = [field columnName];
-        [contentFieldNames addObject: fieldName];
+
+	if (![fieldName isEqualToString: @"c_name"])
+          [contentFieldNames addObject: fieldName];
       }
-    
+
     folderId       = [_folderId copy];
     folderName     = [[_path lastPathComponent] copy];
     path           = [_path   copy];
@@ -128,7 +124,7 @@ static GCSStringFormatter *stringFormatter = nil;
     folderTypeName = [_ftname copy];
 
     ofFlags.requiresFolderSelect = 0;
-    ofFlags.sameTableForQuick = 
+    ofFlags.sameTableForQuick =
       [location isEqualTo:quickLocation] ? 1 : 0;
   }
   return self;
@@ -137,7 +133,7 @@ static GCSStringFormatter *stringFormatter = nil;
 - (id) init
 {
   return [self initWithPath:nil primaryKey:nil
-	       folderTypeName:nil folderType:nil 
+	       folderTypeName:nil folderType:nil
 	       location:nil quickLocation:nil
                aclLocation:nil
 	       folderManager:nil];
@@ -187,12 +183,31 @@ static GCSStringFormatter *stringFormatter = nil;
 }
 
 - (NSURL *)location {
+  if ([GCSFolderManager singleStoreMode])
+    return [folderManager storeLocation];
+
   return location;
 }
 - (NSURL *)quickLocation {
+  if ([GCSFolderManager singleStoreMode])
+    {
+      NSString *baseURL;
+      NSRange range;
+
+      baseURL = [[folderManager folderInfoLocation] absoluteString];
+      range = [baseURL rangeOfString: @"/" options: NSBackwardsSearch];
+      if (range.location != NSNotFound)
+        baseURL = [baseURL substringToIndex: range.location];
+
+      return [NSURL URLWithString: [NSString stringWithFormat: @"%@/%@", baseURL, [self quickTableName]]];
+    }
+
   return quickLocation;
 }
 - (NSURL *)aclLocation {
+  if ([GCSFolderManager singleStoreMode])
+    return [folderManager aclLocation];
+
   return aclLocation;
 }
 
@@ -208,6 +223,9 @@ static GCSStringFormatter *stringFormatter = nil;
   return [[self location] gcsTableName];
 }
 - (NSString *)quickTableName {
+  if ([GCSFolderManager singleStoreMode])
+    return [NSString stringWithFormat: @"sogo_quick_%@", [folderTypeName lowercaseString]];
+
   return [[self quickLocation] gcsTableName];
 }
 - (NSString *)aclTableName {
@@ -253,13 +271,13 @@ static GCSStringFormatter *stringFormatter = nil;
   NSDictionary *ui;
 
   ui = [NSDictionary dictionaryWithObjectsAndKeys:
-		       [NSNumber numberWithUnsignedInt:_base],  
+		       [NSNumber numberWithUnsignedInt:_base],
 		       @"GCSExpectedVersion",
 		       [NSNumber numberWithUnsignedInt:_store],
 		       @"GCSStoredVersion",
   		       self, @"GCSFolder",
 		       nil];
-  
+
   return [NSException exceptionWithName:@"GCSVersionMismatch"
 		      reason:@"Transaction conflict during a GCS modification."
 		      userInfo:ui];
@@ -351,7 +369,7 @@ static GCSStringFormatter *stringFormatter = nil;
 		   requirement: (GCSTableRequirement) requirement
 {
   NSString *selectedFields;
-  
+
   if (requirement == bothTableRequired && [fields containsObject: @"c_name"])
     selectedFields = [self _dottedFields: fields];
   else
@@ -363,7 +381,7 @@ static GCSStringFormatter *stringFormatter = nil;
 - (NSString *) _sqlForQualifier: (EOQualifier *) qualifier
 {
   NSMutableString *ms;
-  
+
   if (qualifier)
     {
       ms = [NSMutableString stringWithCapacity:32];
@@ -381,7 +399,7 @@ static GCSStringFormatter *stringFormatter = nil;
 
   if ((count = [_so count]) == 0)
     return nil;
-  
+
   sql = [NSMutableString stringWithCapacity:(count * 16)];
   for (i = 0; i < count; i++) {
     EOSortOrdering *so;
@@ -391,9 +409,9 @@ static GCSStringFormatter *stringFormatter = nil;
     so     = [_so objectAtIndex:i];
     sel    = [so selector];
     column = [so key];
-    
+
     if (i > 0) [sql appendString:@", "];
-    
+
     if (sel_eq(sel, EOCompareAscending)) {
       [sql appendString:column];
       [sql appendString:@" ASC"];
@@ -425,20 +443,36 @@ static GCSStringFormatter *stringFormatter = nil;
 		 ignoreDeleted: (BOOL) ignoreDeleted
 {
   EOQualifier      *qualifier;
+  EOSortOrdering   *ordering;
   NSArray          *sortOrderings;
+  NSEnumerator     *sortOrderingsList;
   NSMutableString  *sql;
-  NSMutableArray   *whereSql;
+  NSMutableArray   *allFields, *whereSql;
   GCSTableRequirement requirement;
   NSString *whereString;
 
-//   NSLog(@"queryForFields...");
   qualifier = [spec qualifier];
-  requirement = [self _tableRequirementForFields: fields
-		      andOrQualifier: qualifier];
+  sortOrderings = [spec sortOrderings];
+
+  if (fields)
+    allFields = [NSMutableArray arrayWithArray: fields];
+  else
+    allFields = [NSMutableArray arrayWithCapacity: [sortOrderings count]];
+
+  if ([sortOrderings count] > 0)
+    {
+      sortOrderingsList = [sortOrderings objectEnumerator];
+      while ((ordering = [sortOrderingsList nextObject]))
+        {
+          [allFields addObject: [ordering key]];
+        }
+    }
+  requirement = [self _tableRequirementForFields: allFields
+                                  andOrQualifier: qualifier];
   sql = [NSMutableString stringWithCapacity: 256];
   [sql appendString: @"SELECT "];
-  if (fields)
-    [sql appendString: [self _selectedFields: fields requirement: requirement]];
+  if ([allFields count])
+    [sql appendString: [self _selectedFields: allFields requirement: requirement]];
   else
     [sql appendString: @"*"];
   [sql appendString:@" FROM "];
@@ -454,6 +488,16 @@ static GCSStringFormatter *stringFormatter = nil;
     }
 
   whereSql = [NSMutableArray array];
+  if ([GCSFolderManager singleStoreMode])
+    {
+      if (requirement == bothTableRequired)
+        [whereSql addObject: [NSString stringWithFormat:
+                                         @"(a.c_folder_id = %@ AND b.c_folder_id = %@)",
+                                       folderId, folderId]];
+      else
+        [whereSql addObject: [NSString stringWithFormat: @"c_folder_id = %@",
+                                       folderId]];
+    }
   if (qualifier)
     {
       whereString = [NSString stringWithFormat: @"(%@)",
@@ -474,7 +518,6 @@ static GCSStringFormatter *stringFormatter = nil;
     [sql appendFormat: @" WHERE %@",
 	 [whereSql componentsJoinedByString: @" AND "]];
 
-  sortOrderings = [spec sortOrderings];
   if ([sortOrderings count] > 0)
     {
       [sql appendString:@" ORDER BY "];
@@ -515,21 +558,21 @@ static GCSStringFormatter *stringFormatter = nil;
       error = [channel evaluateExpressionX:sql];
       if (error)
 	{
-	  [self errorWithFormat:@"%s: cannot execute quick-fetch SQL '%@': %@", 
+	  [self errorWithFormat:@"%s: cannot execute quick-fetch SQL '%@': %@",
 		__PRETTY_FUNCTION__, sql, error];
 	  results = nil;
 	}
       else
 	{
 	  /* fetch results */
-  
+
 	  results = [NSMutableArray arrayWithCapacity: 64];
 	  attrs = [channel describeResults: NO /* do not beautify names */];
 	  while ((row = [channel fetchAttributes: attrs withZone: NULL]))
 	    [results addObject: row];
 
 	  /* release channels */
-  
+
 	}
       [self releaseChannel: channel];
 //         NSLog(@"/running query");
@@ -539,7 +582,7 @@ static GCSStringFormatter *stringFormatter = nil;
       [self errorWithFormat:@" could not open storage channel!"];
       results = nil;
     }
-  
+
   return results;
 }
 
@@ -675,6 +718,8 @@ andAttribute: (EOAttribute *)_attribute
   [sql appendString:@"INSERT INTO "];
   [sql appendString:_table];
   [sql appendString:@" ("];
+  if ([GCSFolderManager singleStoreMode])
+    [sql appendString:@"c_folder_id, "];
 
   for (i = 0, count = [keys count]; i < count; i++) {
     if (i != 0) [sql appendString:@", "];
@@ -682,6 +727,8 @@ andAttribute: (EOAttribute *)_attribute
   }
 
   [sql appendString:@") VALUES ("];
+  if ([GCSFolderManager singleStoreMode])
+    [sql appendFormat:@"%@, ", folderId];
 
   for (i = 0, count = [keys count]; i < count; i++) {
     fieldName = [keys objectAtIndex:i];
@@ -717,7 +764,7 @@ andAttribute: (EOAttribute *)_attribute
   EOAttribute *attribute;
   id value;
   unsigned i, count;
-  
+
   if (_row == nil || _table == nil)
     return nil;
 
@@ -727,7 +774,7 @@ andAttribute: (EOAttribute *)_attribute
   [sql appendString:@"UPDATE "];
   [sql appendString:_table];
   [sql appendString:@" SET "];
-  
+
   for (i = 0, count = [keys count]; i < count; i++) {
     fieldName = [keys objectAtIndex:i];
 
@@ -749,6 +796,8 @@ andAttribute: (EOAttribute *)_attribute
   }
 
   [sql appendString:@" WHERE "];
+  if ([GCSFolderManager singleStoreMode])
+    [sql appendString: [NSString stringWithFormat: @"c_folder_id = %@ AND ", folderId]];
   [sql appendString:_colname];
   [sql appendString:@" = "];
   attribute = [self _attributeForColumn: _colname];
@@ -774,7 +823,7 @@ andAttribute: (EOAttribute *)_attribute
 {
   EOAttribute *attribute;
   EOEntity *entity;
-  
+
   entity = AUTORELEASE([[EOEntity alloc] init]);
   [entity setName: _name];
   [entity setExternalName: _name];
@@ -792,7 +841,7 @@ andAttribute: (EOAttribute *)_attribute
   EOAttribute *attribute;
   EOEntity *entity;
 
-  entity = [self _entityWithName: [self storeTableName]];  
+  entity = [self _entityWithName: [self storeTableName]];
 
   attribute = AUTORELEASE([[EOAttribute alloc] init]);
   [attribute setName: @"c_version"];
@@ -824,7 +873,7 @@ andAttribute: (EOAttribute *)_attribute
 
 - (EOSQLQualifier *) _qualifierUsingWhereColumn:(NSString *)_colname
 				      isEqualTo:(id)_value
-				      andColumn:(NSString *)_colname2 
+				      andColumn:(NSString *)_colname2
 				      isEqualTo:(id)_value2
 					 entity: (EOEntity *)_entity
                                     withAdaptor: (EOAdaptor *)_adaptor
@@ -835,22 +884,39 @@ andAttribute: (EOAttribute *)_attribute
   attribute1 = [self _attributeForColumn: _colname];
   if (_colname2 == nil)
     {
-      qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
-					  qualifierFormat: @"%A = %@", _colname,
-                                          [self _formatRowValue:_value
-                                                withAdaptor: _adaptor andAttribute: attribute1]];
+      if ([GCSFolderManager singleStoreMode])
+        qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
+                                           qualifierFormat: @"%A = %@ AND c_folder_id = %@", _colname,
+                                     [self _formatRowValue:_value
+                                               withAdaptor: _adaptor andAttribute: attribute1], folderId];
+      else
+        qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
+                                           qualifierFormat: @"%A = %@", _colname,
+                                     [self _formatRowValue:_value
+                                               withAdaptor: _adaptor andAttribute: attribute1]];
     }
   else
     {
       attribute2 = [self _attributeForColumn: _colname2];
-      qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
-					  qualifierFormat: @"%A = %@ AND %A = %@",
-                                          _colname,
-                                          [self _formatRowValue:_value
-                                                withAdaptor: _adaptor andAttribute: attribute1],
-                                          _colname2,
-                                          [self _formatRowValue:_value2
-                                                withAdaptor: _adaptor andAttribute: attribute2]];
+      if ([GCSFolderManager singleStoreMode])
+        qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
+                                           qualifierFormat: @"%A = %@ AND %A = %@ AND c_folder_id = %@",
+                                            _colname,
+                                     [self _formatRowValue:_value
+                                               withAdaptor: _adaptor andAttribute: attribute1],
+                                            _colname2,
+                                     [self _formatRowValue:_value2
+                                               withAdaptor: _adaptor andAttribute: attribute2]];
+      else
+        qualifier = [[EOSQLQualifier alloc] initWithEntity: _entity
+                                           qualifierFormat: @"%A = %@ AND %A = %@",
+                                            _colname,
+                                     [self _formatRowValue:_value
+                                               withAdaptor: _adaptor andAttribute: attribute1],
+                                            _colname2,
+                                     [self _formatRowValue:_value2
+                                               withAdaptor: _adaptor andAttribute: attribute2],
+                                            folderId];
     }
 
   return AUTORELEASE(qualifier);
@@ -869,10 +935,17 @@ andAttribute: (EOAttribute *)_attribute
 
   table = [self storeTableName];
   attribute = [self _attributeForColumn: @"c_name"];
-  delSql = [NSString stringWithFormat: @"DELETE FROM %@"
-		     @" WHERE c_name = %@", table,
-                     [self _formatRowValue: recordName
-                           withAdaptor: [adaptorCtx adaptor]
+  if ([GCSFolderManager singleStoreMode])
+    delSql = [NSString stringWithFormat: @"DELETE FROM %@"
+                       @" WHERE c_name = %@ AND c_folder_id = %@", table,
+                  [self _formatRowValue: recordName
+                            withAdaptor: [adaptorCtx adaptor]
+                           andAttribute: attribute], folderId];
+  else
+    delSql = [NSString stringWithFormat: @"DELETE FROM %@"
+                       @" WHERE c_name = %@", table,
+                  [self _formatRowValue: recordName
+                            withAdaptor: [adaptorCtx adaptor]
                            andAttribute: attribute]];
   [channel evaluateExpressionX: delSql];
 
@@ -897,7 +970,7 @@ andAttribute: (EOAttribute *)_attribute
   NSArray *rows;
   NSException         *error;
 
-  /* check preconditions */  
+  /* check preconditions */
   if (_name)
     {
       if (_content)
@@ -906,10 +979,10 @@ andAttribute: (EOAttribute *)_attribute
 	  error   = nil;
 	  nowDate = [NSCalendarDate date];
 	  now     = [NSNumber numberWithUnsignedInt:[nowDate timeIntervalSince1970]];
-  
+
 	  if (doLogStore)
 	    [self logWithFormat:@"should store content: '%@'\n%@", _name, _content];
-  
+
 	  rows = [self fetchFields: [NSArray arrayWithObjects:
 					       @"c_version",
 					     @"c_deleted",
@@ -944,20 +1017,34 @@ andAttribute: (EOAttribute *)_attribute
 	      || *_baseVersion == [storedVersion unsignedIntValue])
 	    {
 	      /* extract quick info */
-              quickRow = [theComponent performSelector: @selector(quickRecordFromContent:container:)  withObject: _content  withObject: theContainer];
+	      NSMethodSignature *aSignature;
+	      NSInvocation *anInvocation;
+	      SEL aSelector;
+
+	      aSelector = @selector(quickRecordFromContent:container:nameInContainer:);
+	      aSignature = [[theComponent class] instanceMethodSignatureForSelector: aSelector];
+	      anInvocation = [NSInvocation invocationWithMethodSignature:aSignature];
+	      [anInvocation setSelector: aSelector];
+	      [anInvocation setTarget: theComponent];
+	      [anInvocation setArgument:&_content  atIndex: 2];
+	      [anInvocation setArgument:&theContainer  atIndex: 3];
+	      [anInvocation setArgument:&_name atIndex: 4];
+	      [anInvocation invoke];
+	      [anInvocation getReturnValue: &quickRow];
+
 	      if (quickRow)
 		{
 		  [quickRow setObject:_name forKey:@"c_name"];
-  
+
 		  if (doLogStore)
 		    [self logWithFormat:@"  store quick: %@", quickRow];
-  
+
 		  /* make content row */
 		  contentRow = [NSMutableDictionary dictionaryWithCapacity:16];
-  
+
 		  if (ofFlags.sameTableForQuick)
 		    [contentRow addEntriesFromDictionary:quickRow];
-  
+
 		  [contentRow setObject:_name forKey:@"c_name"];
 		  [contentRow setObject:now forKey:@"c_lastmodified"];
 		  if (isNewRecord)
@@ -971,7 +1058,7 @@ andAttribute: (EOAttribute *)_attribute
 				  [NSNumber numberWithInt:([storedVersion intValue] + 1)]
 				forKey:@"c_version"];
 		  [contentRow setObject:_content forKey:@"c_content"];
-  
+
 		  /* open channels */
 		  storeChannel = [self acquireStoreChannel];
 		  if (storeChannel)
@@ -993,14 +1080,14 @@ andAttribute: (EOAttribute *)_attribute
 		      /* we check if we can call directly methods on our adaptor
 			 channel delegate. If not, we generate SQL ourself since it'll
 			 be a little bit faster and less complex than using GDL to do so */
-		      hasInsertDelegate = [[storeChannel delegate] 
+		      hasInsertDelegate = [[storeChannel delegate]
 					    respondsToSelector: @selector(adaptorChannel:willInsertRow:forEntity:)];
 		      hasUpdateDelegate = [[storeChannel delegate]
 					    respondsToSelector: @selector(adaptorChannel:willUpdateRow:describedByQualifier:)];
 
 		      [[quickChannel adaptorContext] beginTransaction];
 		      [[storeChannel adaptorContext] beginTransaction];
-  
+
 		      quickTableEntity = [self _quickTableEntity];
 		      storeTableEntity = [self _storeTableEntity];
 
@@ -1010,10 +1097,10 @@ andAttribute: (EOAttribute *)_attribute
 			    error = (hasInsertDelegate
 				     ? [quickChannel insertRowX: quickRow forEntity: quickTableEntity]
 				     : [quickChannel
-					 evaluateExpressionX: [self _generateInsertStatementForRow: quickRow 
+					 evaluateExpressionX: [self _generateInsertStatementForRow: quickRow
                                                                     adaptor: [[quickChannel adaptorContext] adaptor]
 								    tableName: [self quickTableName]]]);
-			  
+
 			  if (!error)
 			    error = (hasInsertDelegate
 				     ? [storeChannel insertRowX: contentRow forEntity: storeTableEntity]
@@ -1051,7 +1138,7 @@ andAttribute: (EOAttribute *)_attribute
 										andColumn: (*_baseVersion != 0 ? (id)@"c_version" : (id)nil)
 										isEqualTo: (*_baseVersion != 0 ? [NSNumber numberWithUnsignedInt: *_baseVersion] : (NSNumber *)nil)]]);
 		      }
-  
+
 		      if (error)
 			{
 			  [[storeChannel adaptorContext] rollbackTransaction];
@@ -1100,26 +1187,25 @@ andAttribute: (EOAttribute *)_attribute
   return error;
 }
 
-- (NSException *)deleteContentWithName:(NSString *)_name {
+- (NSException *) deleteContentWithName:(NSString *)_name {
   EOAdaptorChannel *storeChannel, *quickChannel;
   EOAdaptorContext *adaptorCtx;
   NSException *error;
   NSString *delsql;
   NSCalendarDate *nowDate;
-  
+
   /* check preconditions */
-  
   if (_name == nil) {
     return [NSException exceptionWithName:@"GCSDeleteException"
 			reason:@"no content filename was provided"
 			userInfo:nil];
   }
-  
+
   if (doLogStore)
     [self logWithFormat:@"should delete content: '%@'", _name];
-  
+
   /* open channels */
-  
+
   if ((storeChannel = [self acquireStoreChannel]) == nil) {
     [self errorWithFormat:@"could not open storage channel!"];
     return nil;
@@ -1152,9 +1238,11 @@ andAttribute: (EOAttribute *)_attribute
   delsql = [delsql stringByAppendingString: [self _formatRowValue:_name
                                                   withAdaptor: [adaptorCtx adaptor]
                                                   andAttribute: [self _attributeForColumn: @"c_name"]]];
+  if ([GCSFolderManager singleStoreMode])
+    delsql = [delsql stringByAppendingFormat:@" AND c_folder_id = %@", folderId];
   if ((error = [storeChannel evaluateExpressionX:delsql]) != nil) {
     [self errorWithFormat:
-	    @"%s: cannot delete content '%@': %@", 
+	    @"%s: cannot delete content '%@': %@",
 	  __PRETTY_FUNCTION__, delsql, error];
   }
   else if (!ofFlags.sameTableForQuick) {
@@ -1164,21 +1252,23 @@ andAttribute: (EOAttribute *)_attribute
     delsql = [delsql stringByAppendingString: [self _formatRowValue:_name
                                                     withAdaptor: [adaptorCtx adaptor]
                                                     andAttribute: [self _attributeForColumn: @"c_name"]]];
+    if ([GCSFolderManager singleStoreMode])
+      delsql = [delsql stringByAppendingFormat:@" AND c_folder_id = %@", folderId];
     if ((error = [quickChannel evaluateExpressionX:delsql]) != nil) {
       [self errorWithFormat:
-	      @"%s: cannot delete quick row '%@': %@", 
+	      @"%s: cannot delete quick row '%@': %@",
 	    __PRETTY_FUNCTION__, delsql, error];
-      /* 
+      /*
 	 Note: we now have a "broken" record, needs to be periodically GCed by
 	       a script!
       */
     }
   }
-  
+
   /* release channels and return */
   [adaptorCtx commitTransaction];
   [self releaseChannel:storeChannel];
-  
+
   if (!ofFlags.sameTableForQuick) {
     [[quickChannel adaptorContext] commitTransaction];
     [self releaseChannel:quickChannel];
@@ -1211,18 +1301,24 @@ andAttribute: (EOAttribute *)_attribute
     if (!ofFlags.sameTableForQuick) [[quickChannel adaptorContext] beginTransaction];
     [[storeChannel adaptorContext] beginTransaction];
 
-    query = [NSString stringWithFormat: @"DELETE FROM %@", [self storeTableName]];
+    if ([GCSFolderManager singleStoreMode])
+      query = [NSString stringWithFormat: @"DELETE FROM %@ WHERE c_folder_id = %@", [self storeTableName], folderId];
+    else
+      query = [NSString stringWithFormat: @"DELETE FROM %@", [self storeTableName]];
     error = [storeChannel evaluateExpressionX:query];
     if (error)
-      [self errorWithFormat: @"%s: cannot delete content '%@': %@", 
+      [self errorWithFormat: @"%s: cannot delete content '%@': %@",
         __PRETTY_FUNCTION__, query, error];
     else if (!ofFlags.sameTableForQuick) {
-        /* content row deleted, now delete the quick row */
+      /* content row deleted, now delete the quick row */
+      if ([GCSFolderManager singleStoreMode])
+        query = [NSString stringWithFormat: @"DELETE FROM %@ WHERE c_folder_id = %@", [self quickTableName], folderId];
+      else
         query = [NSString stringWithFormat: @"DELETE FROM %@", [self quickTableName]];
-        error = [quickChannel evaluateExpressionX: query];
-        if (error)
-          [self errorWithFormat: @"%s: cannot delete quick row '%@': %@", 
-            __PRETTY_FUNCTION__, query, error];
+      error = [quickChannel evaluateExpressionX: query];
+      if (error)
+        [self errorWithFormat: @"%s: cannot delete quick row '%@': %@",
+              __PRETTY_FUNCTION__, query, error];
     }
 
     /* release channels and return */
@@ -1237,36 +1333,140 @@ andAttribute: (EOAttribute *)_attribute
     return error;
 }
 
+- (NSException *) purgeDeletedRecordsBefore: (unsigned int) days
+{
+  NSString *delSql, *table;
+  EOAdaptorContext *adaptorCtx;
+  EOAdaptorChannel *channel;
+  NSCalendarDate *nowDate;
+  unsigned int delta;
+
+  channel = [self acquireStoreChannel];
+
+  if ((channel = [self acquireStoreChannel]) == nil)
+    return [NSException exceptionWithName: @"GCSChannelException"
+                                   reason: @"could not open storage channel!"
+                                 userInfo: nil];
+
+  adaptorCtx = [channel adaptorContext];
+  [adaptorCtx beginTransaction];
+
+  table = [self storeTableName];
+  nowDate = [NSCalendarDate date];
+  delta = days * 24 * 60 * 60;
+  if (delta > [nowDate timeIntervalSince1970])
+    return [NSException exceptionWithName: @"GCSArgumentException"
+                                   reason: @"number of days is too high!"
+                                 userInfo: nil];
+  delta = [nowDate timeIntervalSince1970] - delta;
+
+  if ([GCSFolderManager singleStoreMode])
+    delSql = [NSString stringWithFormat: @"DELETE FROM %@"
+                       @" WHERE c_folder_id = %@ AND c_deleted = 1 AND c_lastmodified < %u",
+                       table, folderId, delta];
+  else
+    delSql = [NSString stringWithFormat: @"DELETE FROM %@"
+                       @" WHERE c_deleted = 1 AND c_lastmodified < %u",
+                       table, delta];
+  [channel evaluateExpressionX: delSql];
+
+  [[channel adaptorContext] commitTransaction];
+  [self releaseChannel: channel];
+
+  return nil;
+}
+
+- (NSException *) touchContentWithName: (NSString *) _name
+{
+  NSString *touchSql, *table;
+  EOAdaptorContext *adaptorCtx;
+  EOAdaptorChannel *channel;
+  EOAttribute *attribute;
+  NSCalendarDate *nowDate;
+
+  if (_name == nil)
+      return [NSException exceptionWithName: @"GCSDeleteException"
+				     reason: @"no content filename was provided"
+				   userInfo: nil];
+
+  channel = [self acquireStoreChannel];
+
+  if ((channel = [self acquireStoreChannel]) == nil)
+    {
+      [self errorWithFormat:@"could not open storage channel!"];
+      return nil;
+    }
+
+  adaptorCtx = [channel adaptorContext];
+  [adaptorCtx beginTransaction];
+
+  table = [self storeTableName];
+  attribute = [self _attributeForColumn: @"c_name"];
+  nowDate = [NSCalendarDate date];
+
+  if ([GCSFolderManager singleStoreMode])
+    touchSql = [NSString stringWithFormat: @"UPDATE %@ SET c_lastmodified = %u WHERE c_name = %@ AND c_folder_id = %@",
+			 table,
+			 (unsigned int) [nowDate timeIntervalSince1970],
+		    [self _formatRowValue: _name
+			      withAdaptor: [adaptorCtx adaptor]
+			     andAttribute: attribute],
+			 folderId];
+  else
+    touchSql = [NSString stringWithFormat: @"UPDATE %@ SET c_lastmodified = %u WHERE c_name = %@",
+			 table,
+			 (unsigned int) [nowDate timeIntervalSince1970],
+		    [self _formatRowValue: _name
+			      withAdaptor: [adaptorCtx adaptor]
+			     andAttribute: attribute]];
+
+  [channel evaluateExpressionX: touchSql];
+
+  [[channel adaptorContext] commitTransaction];
+  [self releaseChannel: channel];
+
+  return nil;
+}
+
 - (NSException *)deleteFolder {
   EOAdaptorChannel *channel;
   NSString *delsql;
   NSString *table;
-  
+
   /* open channels */
-  
+
   if ((channel = [self acquireStoreChannel]) == nil) {
     [self errorWithFormat:@"could not open channel!"];
     return nil;
   }
-  
+
   /* delete rows */
   [[channel adaptorContext] beginTransaction];
   table = [self storeTableName];
   if ([table length] > 0) {
-    delsql = [@"DROP TABLE " stringByAppendingString: table];
+    if ([GCSFolderManager singleStoreMode])
+      delsql = [NSString stringWithFormat: @"DELETE FROM %@ WHERE c_folder_id = %@", table, folderId];
+    else
+      delsql = [@"DROP TABLE " stringByAppendingString: table];
     [channel evaluateExpressionX:delsql];
   }
   table = [self quickTableName];
   if ([table length] > 0) {
-    delsql = [@"DROP TABLE " stringByAppendingString: table];
+    if ([GCSFolderManager singleStoreMode])
+      delsql = [NSString stringWithFormat: @"DELETE FROM %@ WHERE c_folder_id = %@", table, folderId];
+    else
+      delsql = [@"DROP TABLE " stringByAppendingString: table];
     [channel evaluateExpressionX:delsql];
   }
   table = [self aclTableName];
   if ([table length] > 0) {
-    delsql = [@"DROP TABLE  " stringByAppendingString: table];
+    if ([GCSFolderManager singleStoreMode])
+      delsql = [NSString stringWithFormat: @"DELETE FROM %@ WHERE c_folder_id = %@", table, folderId];
+    else
+      delsql = [@"DROP TABLE  " stringByAppendingString: table];
     [channel evaluateExpressionX:delsql];
   }
-  
+
   [[channel adaptorContext] commitTransaction];
   [self releaseChannel:channel];
 
@@ -1280,7 +1480,7 @@ andAttribute: (EOAttribute *)_attribute
   EOAdaptorChannel *quickChannel;
   EOAdaptorContext *adaptorCtx;
   NSException *error;
-  
+
   quickChannel = [self acquireQuickChannel];
   adaptorCtx = [quickChannel adaptorContext];
   [adaptorCtx beginTransaction];
@@ -1343,25 +1543,32 @@ andAttribute: (EOAttribute *)_attribute
   NSArray          *attrs;
   NSMutableArray   *results;
   NSDictionary     *row;
-  
+
   qualifier     = [_fs qualifier];
   sortOrderings = [_fs sortOrderings];
-  
+
 #if 0
   [self logWithFormat:@"FETCH: %@", _flds];
   [self logWithFormat:@"  MATCH: %@", _q];
 #endif
-  
+
   /* generate SQL */
 
   sql = [NSMutableString stringWithCapacity:256];
   [sql appendString:@"SELECT c_uid, c_object, c_role"];
   [sql appendString:@" FROM "];
   [sql appendString:[self aclTableName]];
-  
+  if ([GCSFolderManager singleStoreMode])
+    [sql appendFormat:@" WHERE c_folder_id = %@", folderId];
+
   if (qualifier != nil) {
-    [sql appendString:@" WHERE "];
-    [sql appendString:[self _sqlForQualifier:qualifier]];
+    if ([GCSFolderManager singleStoreMode])
+      [sql appendFormat:@" AND (%@)", [self _sqlForQualifier:qualifier]];
+    else
+      {
+        [sql appendString:@" WHERE "];
+        [sql appendString:[self _sqlForQualifier:qualifier]];
+      }
   }
   if ([sortOrderings count] > 0) {
     [sql appendString:@" ORDER BY "];
@@ -1372,34 +1579,34 @@ andAttribute: (EOAttribute *)_attribute
   [sql appendString:@" LIMIT "]; // count
   [sql appendString:@" OFFSET "]; // index from 0
 #endif
-  
+
   /* open channel */
 
   if ((channel = [self acquireAclChannel]) == nil) {
     [self errorWithFormat:@"could not open acl channel!"];
     return nil;
   }
-  
+
   /* run SQL */
 
   if ((error = [channel evaluateExpressionX:sql]) != nil) {
-    [self errorWithFormat:@"%s: cannot execute acl-fetch SQL '%@': %@", 
+    [self errorWithFormat:@"%s: cannot execute acl-fetch SQL '%@': %@",
 	    __PRETTY_FUNCTION__, sql, error];
     [self releaseChannel:channel];
     return nil;
   }
-  
+
   /* fetch results */
-  
+
   results = [NSMutableArray arrayWithCapacity:64];
   attrs   = [channel describeResults:NO /* do not beautify names */];
   while ((row = [channel fetchAttributes:attrs withZone:NULL]) != nil)
     [results addObject:row];
-  
+
   /* release channels */
-  
+
   [self releaseChannel: channel];
-  
+
   return results;
 }
 
@@ -1438,30 +1645,34 @@ andAttribute: (EOAttribute *)_attribute
   NSException      *error;
   NSMutableString  *sql;
   NSString *qSql;
-  
+
   sql = [NSMutableString stringWithCapacity:256];
   [sql appendString:@"DELETE FROM "];
   [sql appendString:[self aclTableName]];
   qSql = [self _sqlForQualifier: [_fs qualifier]];
   if (qSql)
-    [sql appendFormat:@" WHERE %@", qSql];
-  
-  /* open channel */
+    {
+      if ([GCSFolderManager singleStoreMode])
+        [sql appendFormat:@" WHERE c_folder_id = %@ AND (%@)", folderId, qSql];
+      else
+        [sql appendFormat:@" WHERE %@", qSql];
+    }
 
+  /* open channel */
   if ((channel = [self acquireAclChannel]) == nil) {
     [self errorWithFormat:@"could not open acl channel!"];
     return;
   }
-  
+
   /* run SQL */
   [[channel adaptorContext] beginTransaction];
   if ((error = [channel evaluateExpressionX:sql]) != nil) {
-    [self errorWithFormat:@"%s: cannot execute acl-fetch SQL '%@': %@", 
+    [self errorWithFormat:@"%s: cannot execute acl-fetch SQL '%@': %@",
           __PRETTY_FUNCTION__, sql, error];
     [self releaseChannel:channel];
     return;
   }
-  
+
   [[channel adaptorContext] commitTransaction];
   [self releaseChannel:channel];
 }
@@ -1477,18 +1688,77 @@ andAttribute: (EOAttribute *)_attribute
 
   count = 0;
 
-  sqlString = [NSMutableString stringWithFormat:
-				 @"SELECT COUNT(*) AS CNT FROM %@",
-			       [self storeTableName]];
+  if ([GCSFolderManager singleStoreMode])
+    sqlString = [NSMutableString stringWithFormat:
+                                   @"SELECT COUNT(*) AS cnt FROM %@ WHERE c_folder_id = %@",
+                                 [self storeTableName], folderId];
+  else
+    sqlString = [NSMutableString stringWithFormat:
+                                   @"SELECT COUNT(*) AS cnt FROM %@",
+                                 [self storeTableName]];
   if (excludeDeleted)
-    [sqlString appendString: @" WHERE (c_deleted != 1 OR c_deleted IS NULL)"];
+    {
+      if ([GCSFolderManager singleStoreMode])
+        [sqlString appendString: @" AND (c_deleted != 1 OR c_deleted IS NULL)"];
+      else
+        [sqlString appendString: @" WHERE (c_deleted != 1 OR c_deleted IS NULL)"];
+    }
 
   channel = [self acquireStoreChannel];
   if (channel)
     {
       error = [channel evaluateExpressionX: sqlString];
       if (error)
-	[self errorWithFormat: @"%s: cannot execute SQL '%@': %@", 
+	[self errorWithFormat: @"%s: cannot execute SQL '%@': %@",
+	      __PRETTY_FUNCTION__, sqlString, error];
+      else
+	{
+	  attrs = [channel describeResults: NO];
+	  row = [channel fetchAttributes: attrs withZone: NULL];
+	  count = [[row objectForKey: @"cnt"] unsignedIntValue];
+	  [channel cancelFetch];
+	}
+      [self releaseChannel: channel];
+    }
+
+  return count;
+}
+
+- (unsigned int) recordsCountDeletedBefore: (unsigned int) days
+{
+  EOAdaptorChannel *channel;
+  NSArray *attrs;
+  NSCalendarDate *nowDate;
+  NSDictionary *row;
+  NSException *error;
+  NSMutableString  *sqlString;
+  unsigned int delta, count;
+
+  count = 0;
+  nowDate = [NSCalendarDate date];
+  delta = days * 24 * 60 * 60;
+  if (delta < [nowDate timeIntervalSince1970])
+    delta = [nowDate timeIntervalSince1970] - delta;
+  else
+    delta = 0;
+
+  if ([GCSFolderManager singleStoreMode])
+    sqlString = [NSMutableString stringWithFormat:
+                                   @"SELECT COUNT(*) AS cnt FROM %@"
+                                 @" WHERE c_folder_id = %@ AND c_lastmodified < %u AND c_deleted = 1",
+                                 [self storeTableName], folderId, delta];
+  else
+    sqlString = [NSMutableString stringWithFormat:
+                                   @"SELECT COUNT(*) AS cnt FROM %@"
+                                 @" WHERE c_lastmodified < %u AND c_deleted = 1",
+                                 [self storeTableName], delta];
+
+  channel = [self acquireStoreChannel];
+  if (channel)
+    {
+      error = [channel evaluateExpressionX: sqlString];
+      if (error)
+	[self errorWithFormat: @"%s: cannot execute SQL '%@': %@",
 	      __PRETTY_FUNCTION__, sqlString, error];
       else
 	{
@@ -1538,7 +1808,7 @@ andAttribute: (EOAttribute *)_attribute
 - (NSString *)description {
   NSMutableString *ms;
   id tmp;
-  
+
   ms = [NSMutableString stringWithCapacity:256];
   [ms appendFormat:@"<0x%p[%@]:", self, NSStringFromClass([self class])];
 
@@ -1551,7 +1821,7 @@ andAttribute: (EOAttribute *)_attribute
   if ((tmp = [self folderTypeName])) [ms appendFormat:@" type=%@", tmp];
   if ((tmp = [self location]))
     [ms appendFormat:@" loc=%@", [tmp absoluteString]];
-  
+
   [ms appendString:@">"];
   return ms;
 }
