@@ -19,10 +19,6 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
 
 #import "NSArray+NGCards.h"
 #import "NSString+NGCards.h"
