@@ -20,16 +20,12 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSCharacterSet.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSTimeZone.h>
 
-#import <NGObjWeb/WOActionResults.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGCards/iCalEvent.h>
-#import <NGCards/iCalPerson.h>
 
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSObject+Utilities.h>
