@@ -1,6 +1,6 @@
 /* SOGoCalendarComponent.m - this file is part of SOGo
  *
- * Copyright (C) 2006-2014 Inverse inc.
+ * Copyright (C) 2006-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,6 @@
  */
 
 #import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSProcessInfo.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
@@ -36,8 +32,7 @@
 #import <NGCards/iCalCalendar.h>
 #import <NGCards/iCalDateTime.h>
 #import <NGCards/iCalEvent.h>
-#import <NGCards/iCalPerson.h>
-#import <NGCards/iCalRepeatableEntityObject.h>
+#import <NGCards/iCalToDo.h>
 #import <NGMime/NGMimeBodyPart.h>
 #import <NGMime/NGMimeMultipartBody.h>
 #import <NGMail/NGMimeMessage.h>
@@ -47,14 +42,14 @@
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSObject+DAV.h>
 #import <SOGo/NSObject+Utilities.h>
+#import <SOGo/NSString+Crypto.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoBuild.h>
-#import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoGroup.h>
 #import <SOGo/SOGoPermissions.h>
 #import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserDefaults.h>
+#import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUserManager.h>
 #import <SOGo/SOGoWebDAVAclManager.h>
@@ -63,14 +58,12 @@
 #import <Mailer/NSString+Mail.h>
 
 #import "SOGoAptMailICalReply.h"
-#import "SOGoAptMailNotification.h"
 #import "SOGoAptMailReceipt.h"
 #import "SOGoEMailAlarmsManager.h"
 #import "iCalEntityObject+SOGo.h"
 #import "iCalPerson+SOGo.h"
 #import "iCalRepeatableEntityObject+SOGo.h"
 #import "SOGoCalendarComponent.h"
-#import "SOGoComponentOccurence.h"
 
 @implementation SOGoCalendarComponent
 
@@ -140,6 +133,24 @@
   return aclManager;
 }
 
+- (void) setIsNew: (BOOL) newIsNew
+{
+  [super setIsNew: newIsNew];
+}
+
+- (BOOL) isNew
+{
+  return [super isNew];
+}
+
+- (NSException *) changeParticipationStatus: (NSString *) newPartStat
+                               withDelegate: (iCalPerson *) delegate
+                                      alarm: (iCalAlarm *) alarm
+{
+  // Required for protocol <SOGoComponentOccurence>
+  return nil;
+}
+
 - (id) init
 {
   if ((self = [super init]))
@@ -171,7 +182,7 @@
 
 - (Class *) parsingClass
 {
-  return [iCalCalendar class];
+  return (Class *)[iCalCalendar class];
 }
 
 - (NSString *) davContentType
@@ -194,12 +205,21 @@
 
 - (void) _filterComponent: (iCalEntityObject *) component
 {
-  NSString *type, *summary;
+  NSString *type, *summary, *tag, *uid;
+  SOGoUserSettings *settings;
+  SOGoUser *calendarOwner;
+  NSEnumerator *children;
+  CardElement *element;
+  NSArray *tags;
+
   int classification;
 
   type = @"vtodo";
   classification = 0;
 
+  calendarOwner = [SOGoUser userWithLogin: [self ownerInContext: context]];
+  settings = [calendarOwner userSettings];
+
   if ([component isKindOfClass: [iCalEvent class]])
     type = @"vevent";
   
@@ -211,14 +231,23 @@
   summary = [self labelForKey: [NSString stringWithFormat: @"%@_class%d",
                                          type, classification]
                     inContext: context];
+
+  tags = [NSArray arrayWithObjects: @"DTSTAMP", @"DTSTART", @"DTEND", @"DUE", @"EXDATE", @"EXRULE", @"RRULE", @"RECURRENCE-ID", nil];
+  uid = [[component uid] asCryptedPassUsingScheme: @"ssha256"
+                                         withSalt: [[settings userSalt] dataUsingEncoding: NSASCIIStringEncoding]
+                                      andEncoding: encHex];
+
+  children = [[[[component children] copy] autorelease] objectEnumerator];
+
+  while ((element = [children nextObject]))
+    {
+      tag = [element tag];
+      if (![tags containsObject: [tag uppercaseString]])
+        [component removeChild: element];
+    }
+
   [component setSummary: summary];
-  [component setComment: @""];
-  [component setUserComment: @""];
-  [component setLocation: @""];
-  [component setCategories: [NSArray array]];
-  [component setUrl: @""];
-  [component removeAllAttendees];
-  [component removeAllAlarms];
+  [component setUid: uid];
 }
 
 - (NSString *) secureContentAsString
@@ -590,7 +619,10 @@
       if (content)
 	ASSIGN (originalCalendar, [iCalCalendar parseSingleFromSource: content]);
       else
-	[self warnWithFormat: @"content not available, we will crash"];
+	{
+	  [self warnWithFormat: @"content not available, we don't update the event"];
+	  return;
+	}
     }
 
   oldMaster = (iCalRepeatableEntityObject *)
@@ -654,15 +686,6 @@
 	newUid = [newUid substringToIndex: [newUid length]-4];
       [newObject setUid: newUid];
     }
-
-  if ([[SOGoSystemDefaults sharedSystemDefaults] enableEMailAlarms])
-    {
-      SOGoEMailAlarmsManager *eaMgr;
-      
-      eaMgr = [SOGoEMailAlarmsManager sharedEMailAlarmsManager];
-      [eaMgr handleAlarmsInCalendar: [newObject parent]
-	     fromComponent: self];
-    }
 }
 
 - (NSException *) saveCalendar: (iCalCalendar *) newCalendar
@@ -677,6 +700,12 @@
   return [self saveCalendar: [newObject parent]];
 }
 
+- (NSException *) saveComponent: (iCalRepeatableEntityObject *) newEvent
+                          force: (BOOL) forceSave
+{
+  return [self saveComponent: newEvent];
+}
+
 /* raw saving */
 
 /* EMail Notifications */
@@ -723,16 +752,8 @@
 
   parent = [object parent];
   iCalString = [NSString stringWithFormat: @"%@\r\n", [parent versitString]];
-  if ([iCalString canBeConvertedToEncoding: NSISOLatin1StringEncoding])
-    {
-      objectData = [iCalString dataUsingEncoding: NSISOLatin1StringEncoding];
-      charset = @"ISO-8859-1";
-    }
-  else
-    {
-      objectData = [iCalString dataUsingEncoding: NSUTF8StringEncoding];
-      charset = @"UTF-8";
-    }
+  objectData = [iCalString dataUsingEncoding: NSUTF8StringEncoding];
+  charset = @"UTF-8";
 
   header = [NSString stringWithFormat: @"text/calendar; method=%@;"
                      @" charset=\"%@\"",
@@ -776,6 +797,14 @@
   if ([object firstChildWithTag: @"X-SOGo-Send-Appointment-Notifications"])
     return;
 
+  // We never send IMIP inivitaton/deletion/update when the "initiator" is Outlook 2013/2016 over
+  // the EAS protocol. That is because Outlook will always issue a SendMail command
+  // with the meeting details (ie., IMIP message with METHOD:REQUEST) so there's
+  // no need to send it twice. Moreover, Outlook users can also choose to NOT send
+  // the IMIP messsage at all, so SOGo won't send one without user's consent
+  if ([[context objectForKey: @"DeviceType"] isEqualToString: @"WindowsOutlook15"])
+    return;
+
   ownerUser = [SOGoUser userWithLogin: owner];
   dd = [ownerUser domainDefaults];
   if ([dd appointmentSendEMailNotifications] && [object isStillRelevant])
@@ -891,12 +920,15 @@
     }
 }
 
-#warning fix this when sendEmailUsing blabla has been cleaned up
-- (void) sendIMIPReplyForEvent: (iCalRepeatableEntityObject *) event
-			  from: (SOGoUser *) from
-			    to: (iCalPerson *) recipient
+//
+//
+//
+- (void) _sendIMIPReplyForEvent: (iCalRepeatableEntityObject *) event
+                           from: (SOGoUser *) from
+                             to: (iCalPerson *) recipient
+                       textOnly: (BOOL) textOnly
 {
-  NSString *pageName, *mailDate, *email;
+  NSString *mailDate, *email;
   WOApplication *app;
   iCalPerson *attendee;
   SOGoAptMailICalReply *p;
@@ -908,50 +940,61 @@
   SOGoDomainDefaults *dd;
 
   dd = [from domainDefaults];
-  if ([dd appointmentSendEMailNotifications] && [event isStillRelevant])
-    {
-      /* get WOApplication instance */
-      app = [WOApplication application];
 
-      /* create page name */
-      pageName = @"SOGoAptMailICalReply";
-      /* construct message content */
-      p = [app pageWithName: pageName inContext: context];
-      [p setApt: (iCalEvent *) event];
+  /* get WOApplication instance */
+  app = [WOApplication application];
 
-      attendee = [event userAsAttendee: from];
-      [p setAttendee: attendee];
+  /* remove all alarms to avoid bug #3925 */
+  [event removeAllAlarms];
 
-      /* construct message */
-      headerMap = [NGMutableHashMap hashMapWithCapacity: 5];
+  /* construct message content */
+  p = [app pageWithName: @"SOGoAptMailICalReply"  inContext: context];
+  [p setApt: (iCalEvent *) event];
 
-      /* NOTE: multipart/alternative seems like the correct choice but
-       * unfortunately Thunderbird doesn't offer the rich content alternative
-       * at all. Mail.app shows the rich content alternative _only_
-       * so we'll stick with multipart/mixed for the time being.
-       */
+  attendee = [event userAsAttendee: from];
+  [p setAttendee: attendee];
+
+  /* construct message */
+  headerMap = [NGMutableHashMap hashMapWithCapacity: 5];
+
+  /* NOTE: multipart/alternative seems like the correct choice but
+   * unfortunately Thunderbird doesn't offer the rich content alternative
+   * at all. Mail.app shows the rich content alternative _only_
+   * so we'll stick with multipart/mixed for the time being.
+   */
 #warning SOPE is just plain stupid here - if you change the case of keys, it will break the encoding of fields
-      [headerMap setObject: [attendee mailAddress] forKey: @"from"];
-      [headerMap setObject: [recipient mailAddress] forKey: @"to"];
-      mailDate = [[NSCalendarDate date] rfc822DateString];
-      [headerMap setObject: mailDate forKey: @"date"];
-      [headerMap setObject: [[p getSubject] asQPSubjectString: @"UTF-8"]
-                    forKey: @"subject"];
-      [headerMap setObject: [NSString generateMessageID] forKey: @"message-id"];
-      [headerMap setObject: @"1.0" forKey: @"MIME-Version"];
-      [headerMap setObject: @"multipart/mixed" forKey: @"content-type"];
-      [headerMap setObject: @"calendar:invitation-reply" forKey: @"x-sogo-message-type"];
-      msg = [NGMimeMessage messageWithHeader: headerMap];
+  [headerMap setObject: [attendee mailAddress] forKey: @"from"];
+  [headerMap setObject: [recipient mailAddress] forKey: @"to"];
+  mailDate = [[NSCalendarDate date] rfc822DateString];
+  [headerMap setObject: mailDate forKey: @"date"];
+  [headerMap setObject: [[p getSubject] asQPSubjectString: @"UTF-8"]
+                forKey: @"subject"];
+  [headerMap setObject: [NSString generateMessageID] forKey: @"message-id"];
+  [headerMap setObject: @"1.0" forKey: @"MIME-Version"];
 
+  if (textOnly)
+    [headerMap setObject: @"text/html" forKey: @"content-type"];
+  else
+    [headerMap setObject: @"multipart/mixed" forKey: @"content-type"];
+
+  [headerMap setObject: @"calendar:invitation-reply" forKey: @"x-sogo-message-type"];
+  msg = [NGMimeMessage messageWithHeader: headerMap];
+
+  /* text part */
+  headerMap = [NGMutableHashMap hashMapWithCapacity: 1];
+  [headerMap setObject: @"text/html; charset=utf-8"
+                forKey: @"content-type"];
+  bodyPart = [NGMimeBodyPart bodyPartWithHeader: headerMap];
+  bodyData = [[p getBody] dataUsingEncoding: NSUTF8StringEncoding];
+
+  if (textOnly)
+    {
+      [msg setBody: bodyData];
+    }
+  else
+    {
       /* multipart body */
       body = [[NGMimeMultipartBody alloc] initWithPart: msg];
-
-      /* text part */
-      headerMap = [NGMutableHashMap hashMapWithCapacity: 1];
-      [headerMap setObject: @"text/html; charset=utf-8"
-		    forKey: @"content-type"];
-      bodyPart = [NGMimeBodyPart bodyPartWithHeader: headerMap];
-      bodyData = [[p getBody] dataUsingEncoding: NSUTF8StringEncoding];
       [bodyPart setBody: bodyData];
 
       /* attach text part to multipart body */
@@ -963,15 +1006,59 @@
       /* attach multipart body to message */
       [msg setBody: body];
       [body release];
+    }
 
-      /* send the damn thing */
-      email = [recipient rfc822Email];
-      [[SOGoMailer mailerWithDomainDefaults: dd]
+  /* send the damn thing */
+  email = [recipient rfc822Email];
+  [[SOGoMailer mailerWithDomainDefaults: dd]
 	       sendMimePart: msg
                toRecipients: [NSArray arrayWithObject: email]
                      sender: [attendee rfc822Email]
           withAuthenticator: [self authenticatorInContext: context]
                   inContext: context];
+}
+
+
+//
+//
+//
+- (void) sendIMIPReplyForEvent: (iCalRepeatableEntityObject *) event
+			  from: (SOGoUser *) from
+			    to: (iCalPerson *) recipient
+{
+  SOGoDomainDefaults *dd;
+
+  // We never send IMIP reply when the "initiator" is Outlook 2013/2016 over
+  // the EAS protocol. That is because Outlook will always issue a SendMail command
+  // with the meeting's response (ie., IMIP message with METHOD:REPLY) so there's
+  // no need to send it twice. Moreover, Outlook users can also choose to NOT send
+  // the IMIP messsage at all, so SOGo won't send one without user's consent
+  if ([[context objectForKey: @"DeviceType"] isEqualToString: @"WindowsOutlook15"])
+    return;
+
+  dd = [from domainDefaults];
+  if ([dd appointmentSendEMailNotifications] && [event isStillRelevant])
+    {
+      // We first send to the real recipient (organizer)
+      [self _sendIMIPReplyForEvent: event
+                              from: from
+                                to: recipient
+                          textOnly: NO];
+
+      // If we have a sent-by, we send it to it also. This is useful since
+      // if Alice is Bob's asssitant and invites Tony, when Tony accepts/declines
+      // the event, Alice will also be informed about this. See #3195 for details.
+      if ([recipient hasSentBy])
+        {
+          iCalPerson *sentBy;
+
+          sentBy = [[[iCalPerson alloc] init] autorelease];
+          [sentBy setEmail: [recipient sentBy]];
+          [self _sendIMIPReplyForEvent: event
+                                  from: from
+                                    to: sentBy
+                              textOnly: YES];
+        }
     }
 }
 
@@ -1174,6 +1261,36 @@
   return uids;
 }
 
+- (NSException *) _copyComponent: (iCalCalendar *) calendar
+                        toFolder: (SOGoGCSFolder *) newFolder
+                       updateUID: (BOOL) updateUID
+{
+  NSString *newUID;
+  SOGoCalendarComponent *newComponent;
+
+  if (updateUID)
+    {
+      NSArray *elements;
+      unsigned int count, max;
+
+      newUID = [self globallyUniqueObjectId];
+      elements = [calendar allObjects];
+      max = [elements count];
+      for (count = 0; count < max; count++)
+        [[elements objectAtIndex: count] setUid: newUID];
+    }
+  else
+    {
+      newUID = [[[calendar allObjects] objectAtIndex: 0] uid];
+    }
+
+  newComponent = [[self class] objectWithName:
+				 [NSString stringWithFormat: @"%@.ics", newUID]
+			       inContainer: newFolder];
+
+  return [newComponent saveCalendar: calendar];
+}
+
 - (NSException *) copyToFolder: (SOGoGCSFolder *) newFolder
 {
   return [self copyComponent: [self calendar: NO secure: NO]
@@ -1183,29 +1300,18 @@
 - (NSException *) copyComponent: (iCalCalendar *) calendar
 		       toFolder: (SOGoGCSFolder *) newFolder
 {
-  NSArray *elements;
-  NSString *newUID;
-  unsigned int count, max;
-  SOGoCalendarComponent *newComponent;
-
-  newUID = [self globallyUniqueObjectId];
-  elements = [calendar allObjects];
-  max = [elements count];
-  for (count = 0; count < max; count++)
-    [[elements objectAtIndex: count] setUid: newUID];
-
-  newComponent = [[self class] objectWithName:
-				 [NSString stringWithFormat: @"%@.ics", newUID]
-			       inContainer: newFolder];
-
-  return [newComponent saveCalendar: calendar];
+  return [self _copyComponent: calendar
+                     toFolder: newFolder
+                    updateUID: YES];
 }
 
 - (NSException *) moveToFolder: (SOGoGCSFolder *) newFolder
 {
   NSException *ex;
 
-  ex = [self copyToFolder: newFolder];
+  ex = [self _copyComponent: [self calendar: NO secure: NO]
+                   toFolder: newFolder
+                  updateUID: NO];
 
   if (!ex)
     ex = [self delete];
@@ -1350,25 +1456,6 @@
   return nil;
 }
 
-- (id) PUTAction: (WOContext *) localContext
-{
-  WORequest *rq;
-  iCalCalendar *putCalendar;
-
-  rq = [localContext request];
-  putCalendar = [iCalCalendar parseSingleFromSource: [rq contentAsString]];
-
-  if ([[SOGoSystemDefaults sharedSystemDefaults] enableEMailAlarms])
-    {
-      SOGoEMailAlarmsManager *eaMgr;
-
-      eaMgr = [SOGoEMailAlarmsManager sharedEMailAlarmsManager];
-      [eaMgr handleAlarmsInCalendar: putCalendar
-	     fromComponent: self];
-    }
-
-  return [super PUTAction: localContext];
-}
 
 // /* Overriding this method dramatically speeds up PROPFIND request, but may
 //    otherwise be a bad idea... Wait and see. */
@@ -1386,4 +1473,36 @@
 //   return values;
 // }
 
+- (void) adjustClassificationInRequestCalendar: (iCalCalendar *) rqCalendar
+{
+  SOGoUserDefaults *userDefaults;
+  NSString *accessClass;
+  NSArray *allObjects;
+  id entity;
+
+  int i;
+
+  userDefaults = [[context activeUser] userDefaults];
+  allObjects = [rqCalendar allObjects];
+
+  for (i = 0; i < [allObjects count]; i++)
+    {
+      entity = [allObjects objectAtIndex: i];
+
+      if ([entity respondsToSelector: @selector(accessClass)])
+        {
+          accessClass = [entity accessClass];
+
+          if (!accessClass || [accessClass length] == 0)
+            {
+              if ([entity isKindOfClass: [iCalEvent class]])
+                [entity setAccessClass: [userDefaults calendarEventsDefaultClassification]];
+              else if ([entity isKindOfClass: [iCalToDo class]])
+                [entity setAccessClass: [userDefaults calendarTasksDefaultClassification]];
+            }
+        }
+    }
+}
+
+
 @end
