@@ -1,6 +1,6 @@
 /* iCalAlarm+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2014 Inverse inc.
+ * Copyright (C) 2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,11 @@
 
 #import <NGCards/iCalAlarm.h>
 
-@class iCalRepeatableEntityObject;
+@class iCalEntityObject;
 
 @interface iCalAlarm (SOGoExtensions)
 
-+ (id) alarmForEvent: (iCalRepeatableEntityObject *) theEntity
++ (id) alarmForEvent: (iCalEntityObject *) theEntity
                owner: (NSString *) theOwner
               action: (NSString *) reminderAction
                 unit: (NSString *) reminderUnit
