@@ -1,8 +1,6 @@
 /* NGVCard+SOGo.h - this file is part of SOGo
  *
- * Copyright (C) 2009 Inverse inc.
- *
- * Author: Cyril Robert <crobert@inverse.ca>
+ * Copyright (C) 2009-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,9 +28,31 @@
 
 @interface NGVCard (SOGoExtensions)
 
+- (CardElement *) elementWithTag: (NSString *) elementTag
+                          ofType: (NSString *) type;
+
+- (void) addElementWithTag: (NSString *) elementTag
+                    ofType: (NSString *) type
+                 withValue: (id) value;
+
 - (void) updateFromLDIFRecord: (NSDictionary *) ldifRecord;
 - (NSMutableDictionary *) asLDIFRecord;
 
+- (NSString *) workCompany;
+- (NSString *) fullName;
+- (NSArray *) emails;
+- (NSArray *) secondaryEmails;
+
+- (NSString *) workPhone;
+- (NSString *) homePhone;
+- (NSString *) fax;
+- (NSString *) mobile;
+- (NSString *) pager;
+- (NSCalendarDate *) birthday;
+
+- (void) setNotes: (NSArray *) newNotes;
+- (NSArray *) notes;
+
 @end
 
 #endif /* NGVCARD_SOGO_H */
