@@ -1,6 +1,6 @@
 /* NGVCard+SOGo.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2014 Inverse inc.
+ * Copyright (C) 2009-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,12 +18,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
 
+#import <Foundation/NSTimeZone.h>
+
+#import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NSNull+misc.h>
 
 #import <NGCards/NSArray+NGCards.h>
@@ -48,7 +46,7 @@ objectclass ( 2.5.6.7 NAME 'organizationalPerson'
 	SUP person STRUCTURAL
 	MAY ( title $ x121Address $ registeredAddress $ destinationIndicator $
 		preferredDeliveryMethod $ telexNumber $ teletexTerminalIdentifier $
-		telephoneNumber $ internationaliSDNNumber $ 
+		telephoneNumber $ internationaliSDNNumber $
 		facsimileTelephoneNumber $ street $ postOfficeBox $ postalCode $
 		postalAddress $ physicalDeliveryOfficeName $ ou $ st $ l ) )
 
@@ -67,10 +65,10 @@ objectclass	( 2.16.840.1.113730.3.2.2
 		userSMIMECertificate $ userPKCS12 )
 	)
 
-objectclass ( 1.3.6.1.4.1.13769.9.1 NAME 'mozillaAbPersonAlpha' 
-         SUP top AUXILIARY 
+objectclass ( 1.3.6.1.4.1.13769.9.1 NAME 'mozillaAbPersonAlpha'
+         SUP top AUXILIARY
          MUST ( cn )
-         MAY( c $ 
+         MAY( c $
               description $
               displayName $
               facsimileTelephoneNumber $
@@ -187,29 +185,70 @@ convention:
   return element;
 }
 
+- (void) addElementWithTag: (NSString *) elementTag
+                    ofType: (NSString *) type
+                 withValue: (id) value
+{
+  NSArray *allValues;
+  NSEnumerator *list;
+  CardElement *element;
+
+  // value is either an array or a string
+  if ([value isKindOfClass: [NSString class]])
+    allValues = [NSArray arrayWithObject: value];
+  else
+    allValues = value;
+
+  // Add all values as separate elements
+  list = [allValues objectEnumerator];
+  while ((value = [list nextObject]))
+    {
+      if ([type length])
+        element = [CardElement simpleElementWithTag: elementTag
+                                         singleType: type
+                                              value: value];
+      else
+        element = [CardElement simpleElementWithTag: elementTag
+                                              value: value];
+      [self addChild: element];
+    }
+}
+
 - (void) _setPhoneValues: (NSDictionary *) ldifRecord
 {
-  CardElement *phone;
-
-  phone = [self elementWithTag: @"tel" ofType: @"work"];
-  [phone setSingleValue: [ldifRecord objectForKey: @"telephonenumber"] forKey: @""];
-  phone = [self elementWithTag: @"tel" ofType: @"home"];
-  [phone setSingleValue: [ldifRecord objectForKey: @"homephone"] forKey: @""];
-  phone = [self elementWithTag: @"tel" ofType: @"cell"];
-  [phone setSingleValue: [ldifRecord objectForKey: @"mobile"] forKey: @""];
-  phone = [self elementWithTag: @"tel" ofType: @"fax"];
-  [phone setSingleValue: [ldifRecord objectForKey: @"facsimiletelephonenumber"]
-                 forKey: @""];
-  phone = [self elementWithTag: @"tel" ofType: @"pager"];
-  [phone setSingleValue: [ldifRecord objectForKey: @"pager"] forKey: @""];
+  [self addElementWithTag: @"tel"
+                   ofType: @"work"
+                withValue: [ldifRecord objectForKey: @"telephonenumber"]];
+  [self addElementWithTag: @"tel"
+                   ofType: @"home"
+                withValue: [ldifRecord objectForKey: @"homephone"]];
+  [self addElementWithTag: @"tel"
+                   ofType: @"cell"
+                withValue: [ldifRecord objectForKey: @"mobile"]];
+  [self addElementWithTag: @"tel"
+                   ofType: @"fax"
+                withValue: [ldifRecord objectForKey: @"facsimiletelephonenumber"]];
+  [self addElementWithTag: @"tel"
+                   ofType: @"pager"
+                withValue: [ldifRecord objectForKey: @"pager"]];
 }
 
 - (void) _setEmails: (NSDictionary *) ldifRecord
 {
-  CardElement *mail, *homeMail;
+  CardElement *homeMail;
+  NSString* mail;
 
-  mail = [self elementWithTag: @"email" ofType: @"work"];
-  [mail setSingleValue: [ldifRecord objectForKey: @"mail"] forKey: @""];
+  // Emails from the configured mail fields of the source have already been extracted in
+  // [LDAPSource _fillEmailsOfEntry:intoLDIFRecord:]
+  [self addElementWithTag: @"email"
+                   ofType: @"work"
+                withValue: [ldifRecord objectForKey: @"c_emails"]];
+  // When importing an LDIF file, add the default mail attribute
+  mail = [ldifRecord objectForKey: @"mail"];
+  if ([mail length] && ![[self emails] containsObject: mail])
+    [self addElementWithTag: @"email"
+                     ofType: @"work"
+                  withValue: [ldifRecord objectForKey: @"mail"]];
   homeMail = [self elementWithTag: @"email" ofType: @"home"];
   [homeMail setSingleValue: [ldifRecord objectForKey: @"mozillasecondemail"] forKey: @""];
   [[self uniqueChildWithTag: @"x-mozilla-html"]
@@ -223,15 +262,19 @@ convention:
   CardElement *element;
   NSCalendarDate *now;
   NSArray *units;
-  NSString *ou;
+  NSString *fn, *ou;
   id o;
 
   [self setNWithFamily: [ldifRecord objectForKey: @"sn"]
                  given: [ldifRecord objectForKey: @"givenname"]
             additional: nil prefixes: nil suffixes: nil];
   [self setNickname: [ldifRecord objectForKey: @"mozillanickname"]];
-  [self setFn: [ldifRecord objectForKey: @"displayname"]];
-  [self setTitle: [ldifRecord objectForKey: @"title"]];  
+  [self setTitle: [ldifRecord objectForKey: @"title"]];
+
+  fn = [ldifRecord objectForKey: @"displayname"];
+  if (!fn)
+    fn = [ldifRecord objectForKey: @"cn"];
+  [self setFn: fn];
 
   element = [self elementWithTag: @"adr" ofType: @"home"];
   [element setSingleValue: [ldifRecord objectForKey: @"mozillahomestreet2"]
@@ -262,7 +305,9 @@ convention:
                   atIndex: 6 forKey: @""];
 
   ou = [ldifRecord objectForKey: @"ou"];
-  if (ou)
+  if ([ou isKindOfClass: [NSArray class]])
+    units = [NSArray arrayWithArray: (NSArray *)ou];
+  else if (ou)
     units = [NSArray arrayWithObject: ou];
   else
     units = nil;
@@ -275,7 +320,7 @@ convention:
     setSingleValue: [ldifRecord objectForKey: @"mozillahomeurl"] forKey: @""];
   [[self elementWithTag: @"url" ofType: @"work"]
     setSingleValue: [ldifRecord objectForKey: @"mozillaworkurl"] forKey: @""];
-  
+
   [[self uniqueChildWithTag: @"x-aim"]
     setSingleValue: [ldifRecord objectForKey: @"nsaimid"]
             forKey: @""];
@@ -297,7 +342,7 @@ convention:
 
   if (year && month && day)
     [self setBday: [NSString stringWithFormat: @"%.4d-%.2d-%.2d",
-                             year, month, day]];
+                             (int)year, (int)month, (int)day]];
   else
     [self setBday: @""];
 
@@ -306,24 +351,32 @@ convention:
     setSingleValue: [ldifRecord objectForKey: @"c_info"]
             forKey: @""];
 
-  [self setNote: [ldifRecord objectForKey: @"description"]];
+  o = [ldifRecord objectForKey: @"description"];
+  if ([o isKindOfClass: [NSArray class]])
+    [self setNotes: o];
+  else
+    [self setNote: o];
 
   o = [ldifRecord objectForKey: @"vcardcategories"];
 
-  // We can either have an array (from SOGo's web gui) or a 
+  // We can either have an array (from SOGo's web gui) or a
   // string (from a LDIF import) as the value here.
   if ([o isKindOfClass: [NSArray class]])
     [self setCategories: o];
   else
     [self setCategories: [o componentsSeparatedByString: @","]];
 
+  // Photo
+  if ([ldifRecord objectForKey: @"photo"])
+    [self setPhoto: [[ldifRecord objectForKey: @"photo"] stringByEncodingBase64]];
+
   [self cleanupEmptyChildren];
 }
 
 /* VCARD -> LDIF */
 - (NSString *) _simpleValueForType: (NSString *) aType
                            inArray: (NSArray *) anArray
-                         excluding: (NSString *) aTypeToExclude		  
+                         excluding: (NSString *) aTypeToExclude
 {
   NSArray *elements;
   NSString *value;
@@ -344,7 +397,7 @@ convention:
 
 	  if (!aTypeToExclude)
 	    break;
-	  
+
 	  if (![ce hasAttribute: @"type" havingValue: aTypeToExclude])
 	    break;
 
@@ -513,31 +566,6 @@ convention:
                      flattenedValuesForKey: @""]
       inLDIFRecord: ldifRecord];
 
-  elements = [self childrenWithTag: @"adr"
-                   andAttribute: @"type" havingValue: @"work"];
-  if (elements && [elements count] > 0)
-    {
-      element = [elements objectAtIndex: 0];
-      [self _setValue: @"mozillaworkstreet2"
-                   to: [element flattenedValueAtIndex: 1 forKey: @""]
-          inLDIFRecord: ldifRecord];
-      [self _setValue: @"street"
-                   to: [element flattenedValueAtIndex: 2 forKey: @""]
-          inLDIFRecord: ldifRecord];
-      [self _setValue: @"l"
-                   to: [element flattenedValueAtIndex: 3 forKey: @""]
-          inLDIFRecord: ldifRecord];
-      [self _setValue: @"st"
-                   to: [element flattenedValueAtIndex: 4 forKey: @""]
-          inLDIFRecord: ldifRecord];
-      [self _setValue: @"postalcode"
-                   to: [element flattenedValueAtIndex: 5 forKey: @""]
-          inLDIFRecord: ldifRecord];
-      [self _setValue: @"c"
-                   to: [element flattenedValueAtIndex: 6 forKey: @""]
-          inLDIFRecord: ldifRecord];
-    }
-
   elements = [self childrenWithTag: @"adr"
                    andAttribute: @"type" havingValue: @"home"];
   if (elements && [elements count] > 0)
@@ -563,6 +591,35 @@ convention:
           inLDIFRecord: ldifRecord];
     }
 
+  elements = [self childrenWithTag: @"adr"
+                   andAttribute: @"type" havingValue: @"work"];
+
+  if (!elements || [elements count] == 0)
+    elements = [self childrenWithTag: @"adr"];
+
+  if (elements && [elements count] > 0)
+    {
+      element = [elements objectAtIndex: 0];
+      [self _setValue: @"mozillaworkstreet2"
+                   to: [element flattenedValueAtIndex: 1 forKey: @""]
+          inLDIFRecord: ldifRecord];
+      [self _setValue: @"street"
+                   to: [element flattenedValueAtIndex: 2 forKey: @""]
+          inLDIFRecord: ldifRecord];
+      [self _setValue: @"l"
+                   to: [element flattenedValueAtIndex: 3 forKey: @""]
+          inLDIFRecord: ldifRecord];
+      [self _setValue: @"st"
+                   to: [element flattenedValueAtIndex: 4 forKey: @""]
+          inLDIFRecord: ldifRecord];
+      [self _setValue: @"postalcode"
+                   to: [element flattenedValueAtIndex: 5 forKey: @""]
+          inLDIFRecord: ldifRecord];
+      [self _setValue: @"c"
+                   to: [element flattenedValueAtIndex: 6 forKey: @""]
+          inLDIFRecord: ldifRecord];
+    }
+
   elements = [self childrenWithTag: @"url"];
   [self _setValue: @"mozillaworkurl"
                to: [self _simpleValueForType: @"work" inArray: elements
@@ -572,8 +629,8 @@ convention:
                to: [self _simpleValueForType: @"home" inArray: elements
                                    excluding: nil]
       inLDIFRecord: ldifRecord];
-  
-  // If we don't have a "work" or "home" URL but we still have 
+
+  // If we don't have a "work" or "home" URL but we still have
   // an URL field present, let's add it to the "home" value
   if ([[ldifRecord objectForKey: @"mozillaworkurl"] length] == 0 &&
       [[ldifRecord objectForKey: @"mozillahomeurl"] length] == 0 &&
@@ -603,7 +660,7 @@ convention:
 	    }
 	}
     }
-    
+
   [self _setValue: @"title" to: [self title] inLDIFRecord: ldifRecord];
   [self _setupOrgFieldsInLDIFRecord: ldifRecord];
 
@@ -614,11 +671,11 @@ convention:
   birthDay = [[self bday] asCalendarDate];
   if (birthDay)
     {
-      stringValue = [NSString stringWithFormat: @"%.4d", [birthDay yearOfCommonEra]];
+      stringValue = [NSString stringWithFormat: @"%.4d", (int)[birthDay yearOfCommonEra]];
       [self _setValue: @"birthyear" to: stringValue inLDIFRecord: ldifRecord];
-      stringValue = [NSString stringWithFormat: @"%.2d", [birthDay monthOfYear]];
+      stringValue = [NSString stringWithFormat: @"%.2d", (int)[birthDay monthOfYear]];
       [self _setValue: @"birthmonth" to: stringValue inLDIFRecord: ldifRecord];
-      stringValue = [NSString stringWithFormat: @"%.2d", [birthDay dayOfMonth]];
+      stringValue = [NSString stringWithFormat: @"%.2d", (int)[birthDay dayOfMonth]];
       [self _setValue: @"birthday" to: stringValue inLDIFRecord: ldifRecord];
     }
   [self _setValue: @"description" to: [self note] inLDIFRecord: ldifRecord];
@@ -646,7 +703,7 @@ convention:
 {
   CardElement *org;
   NSString *company;
-  
+
   org = [self org];
   company = [org flattenedValueAtIndex: 0 forKey: @""];
   if ([company length] == 0)
@@ -659,7 +716,7 @@ convention:
 {
   CardElement *n;
   NSString *fn, *firstName, *lastName, *org;
-  
+
   fn = [self fn];
   if ([fn length] == 0)
     {
@@ -686,6 +743,29 @@ convention:
   return fn;
 }
 
+- (NSArray *) emails
+{
+  NSArray *elements;
+  NSMutableArray *emails;
+  NSString *email;
+  int i;
+
+  emails = [NSMutableArray array];
+  elements = [self childrenWithTag: @"email"];
+
+  for (i = [elements count]-1; i >= 0; i--)
+    {
+      email = [[elements objectAtIndex: i] flattenedValuesForKey: @""];
+      if ([email caseInsensitiveCompare: [self preferredEMail]] == NSOrderedSame)
+        // First element of array is the preferred email address
+        [emails insertObject: email atIndex: 0];
+      else
+        [emails addObject: email];
+    }
+
+  return emails;
+}
+
 - (NSArray *) secondaryEmails
 {
   NSMutableArray *emails;
@@ -698,11 +778,11 @@ convention:
   [emails removeObjectsInArray: [self childrenWithTag: @"email"
                                          andAttribute: @"type"
                                           havingValue: @"pref"]];
-  
+
   for (i = [emails count]-1; i >= 0; i--)
     {
       email = [[emails objectAtIndex: i] flattenedValuesForKey: @""];
-      
+
       if ([email caseInsensitiveCompare: [self preferredEMail]] == NSOrderedSame)
         [emails removeObjectAtIndex: i];
     }
@@ -734,7 +814,7 @@ convention:
 
 	  if (!aTypeToExclude)
 	    break;
-	  
+
 	  if (![ce hasAttribute: @"type" havingValue: aTypeToExclude])
 	    break;
 
@@ -785,14 +865,52 @@ convention:
       value = [bday stringByReplacingString: @"-" withString: @""];
       date = [NSCalendarDate dateFromShortDateString: value
                                   andShortTimeString: nil
-                                          inTimeZone: nil];
+                                          inTimeZone: [NSTimeZone timeZoneWithName: @"GMT"]];
     }
-  
+
   return date;
 }
 
+- (void) setNotes: (NSArray *) newNotes
+{
+  NSArray *elements;
+  NSUInteger count, max;
+
+  elements = [self childrenWithTag: @"note"];
+  [self removeChildren: elements];
+  max = [newNotes count];
+  for (count = 0; count < max; count++)
+    {
+      [self addChildWithTag: @"note"
+                      types: nil
+                singleValue: [newNotes objectAtIndex: count]];
+    }
+}
+
+
+- (NSArray *) notes
+{
+  NSArray *notes;
+  NSMutableArray *flattenedNotes;
+  NSString *note;
+  NSUInteger count, max;
+
+  notes = [self childrenWithTag: @"note"];
+  max = [notes count];
+  flattenedNotes = [NSMutableArray arrayWithCapacity: max];
+
+  for (count = 0; count < max; count++)
+    {
+      note = [[notes objectAtIndex: count] flattenedValuesForKey: @""];
+      [flattenedNotes addObject: note];
+    }
+
+  return flattenedNotes;
+}
+
 - (NSMutableDictionary *) quickRecordFromContent: (NSString *) theContent
                                        container: (id) theContainer
+				 nameInContainer: (NSString *) nameInContainer
 {
   NSMutableDictionary *fields;
   CardElement *element;
@@ -812,10 +930,12 @@ convention:
   value = [self preferredTel];
   if (value)
     [fields setObject: value forKey: @"c_telephonenumber"];
-  value = [self preferredEMail];
-  if (![value isNotNull])
-    value = @"";
-  [fields setObject: value forKey: @"c_mail"];
+  v = [self emails];
+  if ([v count] > 0)
+    [fields setObject: [v componentsJoinedByString: @","]
+               forKey: @"c_mail"];
+  else
+    [fields setObject: [NSNull null] forKey: @"c_mail"];
   element = [self org];
   [fields setObject: [element flattenedValueAtIndex: 0 forKey: @""]
              forKey: @"c_o"];
