@@ -18,20 +18,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
 #import <Foundation/NSException.h>
-#import <Foundation/NSObject.h>
 #import <Foundation/NSProcessInfo.h>
 #import <Foundation/NSValue.h>
 
-#import <SaxObjC/SaxAttributes.h>
-#import <SaxObjC/SaxContentHandler.h>
 #import <SaxObjC/SaxLexicalHandler.h>
-#import <SaxObjC/SaxXMLReader.h>
 #import <SaxObjC/SaxXMLReaderFactory.h>
 #import <NGExtensions/NGHashMap.h>
-#import <NGExtensions/NGQuotedPrintableCoding.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGMime/NGMimeBodyPart.h>
@@ -55,11 +48,11 @@
 @interface _SOGoHTMLContentHandler : NSObject <SaxContentHandler, SaxLexicalHandler>
 {
   NSMutableArray *images;
-  
+
   NSArray *ignoreContentTags;
   NSArray *specialTreatmentTags;
   NSArray *voidTags;
-  
+
   BOOL ignoreContent;
   BOOL orderedList;
   BOOL unorderedList;
@@ -296,7 +289,7 @@
           //
           if ([value length] > 5 && [[value substringToIndex: 5] caseInsensitiveCompare: @"data:"] == NSOrderedSame)
             {
-              NSString *uniqueId, *mimeType, *encoding;
+              NSString *uniqueId, *mimeType, *encoding, *attrName;
               NGMimeBodyPart *bodyPart;
               NGMutableHashMap *map;
               NSData *data;
@@ -307,7 +300,7 @@
               i = [value indexOf: ';'];
               j = [value indexOf: ';' fromIndex: i+1];
               k = [value indexOf: ','];
-          
+
               // We try to get the MIME type
               mimeType = nil;
 
@@ -321,7 +314,7 @@
               // We might get a stupid value. We discard anything that doesn't have a / in it
               if ([mimeType indexOf: '/'] < 0)
                 mimeType = @"image/jpeg";
-          
+
               // We check and skip the charset
               if (j < i)
                 j = i;
@@ -342,17 +335,30 @@
               [map setObject: [NSString stringWithFormat: @"inline; filename=\"%@\"", uniqueId]  forKey: @"content-disposition"];
               [map setObject: [NSString stringWithFormat: @"%@; name=\"%@\"", mimeType, uniqueId]  forKey: @"content-type"];
               [map setObject: [NSString stringWithFormat: @"<%@>", uniqueId]  forKey: @"content-id"];
-                    
-                    
+
+
               body = [[NGMimeFileData alloc] initWithBytes: [data bytes]  length: [data length]];
 
               bodyPart = [[[NGMimeBodyPart alloc] initWithHeader:map] autorelease];
               [bodyPart setBody: body];
               [body release];
-          
+
               [images addObject: bodyPart];
-          
-              [result appendFormat: @"<img src=\"cid:%@\" type=\"%@\"/>", uniqueId, mimeType];
+
+              [result appendFormat: @"<img src=\"cid:%@\" type=\"%@\"", uniqueId, mimeType];
+
+              // Restore img attributes
+              for (i = 0; i < [attributes count]; i++)
+                {
+                  attrName = [[attributes rawNameAtIndex: i] lowercaseString];
+                  if (![attrName isEqualToString: @"src"] && ![attrName isEqualToString: @"type"])
+                    {
+                      value = [attributes valueAtIndex: i];
+                      [result appendFormat: @" %@=\"%@\"", attrName, value];
+                    }
+                }
+
+              [result appendString: @"/>"];
             }
         }
       else if (voidTags)
@@ -533,7 +539,7 @@
   messageID = [NSMutableString string];
   [messageID appendFormat: @"<%@", [SOGoObject globallyUniqueObjectId]];
   pGUID = [[NSProcessInfo processInfo] globallyUniqueString];
-  [messageID appendFormat: @"@%u>", [pGUID hash]];
+  [messageID appendFormat: @"@%u>", (unsigned int)[pGUID hash]];
 
   return [messageID lowercaseString];
 }
@@ -650,19 +656,19 @@ convertChars (const char *oldString, unsigned int oldLength,
       fromIndex: (int) start
 {
   int i, len;
-  
+
   len = [self length];
-  
+
   if (start < 0 || start >= len)
     start = 0;
-  
+
   for (i = start; i < len; i++)
     {
       if ([self characterAtIndex: i] == _c) return i;
     }
 
   return -1;
-  
+
 }
 
 - (int) indexOf: (unichar) _c
@@ -678,8 +684,30 @@ convertChars (const char *oldString, unsigned int oldLength,
                      decodedHeader];
   if (!decodedHeader)
     decodedHeader = self;
-  
+
   return decodedHeader;
 }
 
+- (NSString *) asSafeFilename
+{
+  NSRange r;
+  NSMutableString *safeName;
+
+  r = [self rangeOfString: @"\\"
+                  options: NSBackwardsSearch];
+  if (r.length > 0)
+    safeName = [NSMutableString stringWithString: [self substringFromIndex: r.location + 1]];
+  else
+    safeName = [NSMutableString stringWithString: self];
+  [safeName replaceString: @"/" withString: @"_"];
+
+  if ([self isEqualToString: @"."])
+    return @"_";
+
+  if ([self isEqualToString: @".."])
+    return @"__";
+
+  return safeName;
+}
+
 @end
