@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2014 Inverse inc.
+  Copyright (C) 2007-2016 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -20,12 +20,6 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSData.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSKeyValueCoding.h>
-#import <Foundation/NSProcessInfo.h>
 #import <Foundation/NSURL.h>
 #import <Foundation/NSValue.h>
 
@@ -33,13 +27,11 @@
 #import <NGObjWeb/SoObject+SoDAV.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGObjWeb/WORequest+So.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NSFileManager+Extensions.h>
 #import <NGExtensions/NGHashMap.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
-#import <NGExtensions/NGQuotedPrintableCoding.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGImap4/NGImap4Connection.h>
 #import <NGImap4/NGImap4Client.h>
@@ -53,7 +45,6 @@
 #import <NGMime/NGMimeFileData.h>
 #import <NGMime/NGMimeMultipartBody.h>
 #import <NGMime/NGMimeType.h>
-#import <NGMime/NGMimeHeaderFieldGenerator.h>
 #import <NGMime/NGMimeHeaderFields.h>
 
 #import <SOGo/NSArray+Utilities.h>
@@ -63,26 +54,29 @@
 #import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoUser.h>
+#import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUserDefaults.h>
+#import <SOGo/SOGoSystemDefaults.h>
 
 #import <NGCards/NGVCard.h>
 
+#import <Contacts/SOGoContactFolder.h>
 #import <Contacts/SOGoContactFolders.h>
 #import <Contacts/SOGoContactGCSEntry.h>
 
 #import "NSData+Mail.h"
 #import "NSString+Mail.h"
+#import "SOGoDraftsFolder.h"
 #import "SOGoMailAccount.h"
-#import "SOGoMailFolder.h"
-#import "SOGoMailObject.h"
 #import "SOGoMailObject+Draft.h"
 #import "SOGoSentFolder.h"
 
 #import "SOGoDraftObject.h"
 
+
 static NSString *contentTypeValue = @"text/plain; charset=utf-8";
 static NSString *htmlContentTypeValue = @"text/html; charset=utf-8";
-static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc", 
+static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 				 @"from", @"replyTo", @"message-id",
 				 nil};
 
@@ -102,6 +96,7 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 //
 @interface NSMutableData (DataCleanupExtension)
 
+- (unichar) characterAtIndex: (int) theIndex;
 - (NSRange) rangeOfCString: (const char *) theCString;
 - (NSRange) rangeOfCString: (const char *) theCString
 		  options: (unsigned int) theOptions
@@ -110,6 +105,32 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 
 @implementation NSMutableData (DataCleanupExtension)
 
+- (unichar) characterAtIndex: (int) theIndex
+{
+  const char *bytes;
+  int i, len;
+
+  len = [self length];
+
+  if (len == 0 || theIndex >= len)
+    {
+      [[NSException exceptionWithName: NSRangeException
+                    reason: @"Index out of range."
+                    userInfo: nil] raise];
+
+      return (unichar)0;
+    }
+
+  bytes = [self bytes];
+
+  for (i = 0; i < theIndex; i++)
+    {
+      bytes++;
+    }
+
+  return (unichar)*bytes;
+}
+
 - (NSRange) rangeOfCString: (const char *) theCString
 {
   return [self rangeOfCString: theCString
@@ -123,18 +144,18 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 {
   const char *b, *bytes;
   int i, len, slen;
-  
+
   if (!theCString)
     {
       return NSMakeRange(NSNotFound,0);
     }
-  
+
   bytes = [self bytes];
   len = [self length];
   slen = strlen(theCString);
-  
+
   b = bytes;
-  
+
   if (len > theRange.location + theRange.length)
     {
       len = theRange.location + theRange.length;
@@ -144,7 +165,7 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
     {
       i = theRange.location;
       b += i;
-      
+
       for (; i <= len-slen; i++, b++)
 	{
 	  if (!strncasecmp(theCString,b,slen))
@@ -157,7 +178,7 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
     {
       i = theRange.location;
       b += i;
-      
+
       for (; i <= len-slen; i++, b++)
 	{
 	  if (!memcmp(theCString,b,slen))
@@ -166,7 +187,7 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 	    }
 	}
     }
-  
+
   return NSMakeRange(NSNotFound,0);
 }
 
@@ -179,6 +200,7 @@ static NSString *headerKeys[] = {@"subject", @"to", @"cc", @"bcc",
 
 static NGMimeType  *MultiMixedType = nil;
 static NGMimeType  *MultiAlternativeType = nil;
+static NGMimeType  *MultiRelatedType = nil;
 static NSString    *userAgent      = nil;
 
 + (void) initialize
@@ -189,6 +211,9 @@ static NSString    *userAgent      = nil;
   MultiAlternativeType = [NGMimeType mimeType: @"multipart" subType: @"alternative"];
   [MultiAlternativeType retain];
 
+  MultiRelatedType = [NGMimeType mimeType: @"multipart" subType: @"related"];
+  [MultiRelatedType retain];
+
   userAgent      = [NSString stringWithFormat: @"SOGoMail %@",
 			     SOGoVersion];
   [userAgent retain];
@@ -248,7 +273,7 @@ static NSString    *userAgent      = nil;
   NSFileManager *fm;
 
   fm = [NSFileManager defaultManager];
-  
+
   return ([fm createDirectoriesAtPath: [container userSpoolFolderPath]
 	      attributes: nil]
 	  && [fm createDirectoriesAtPath: [self draftFolderPath]
@@ -285,7 +310,7 @@ static NSString    *userAgent      = nil;
       messageID = [NSString generateMessageID];
       [headers setObject: messageID forKey: @"message-id"];
     }
-  
+
   priority = [newHeaders objectForKey: @"X-Priority"];
   if (priority)
     {
@@ -313,26 +338,26 @@ static NSString    *userAgent      = nil;
     {
       // newHeaders come from Web form; convert priority to MIME header representation
       priority = [newHeaders objectForKey: @"priority"];
-      if (!priority || [priority isEqualToString: @"NORMAL"])
-        {
-          [headers removeObjectForKey: @"X-Priority"];
-        }
-      else if ([priority isEqualToString: @"HIGHEST"])
+      if ([priority intValue] == 1)
         {
           [headers setObject: @"1 (Highest)"  forKey: @"X-Priority"];
         }
-      else if ([priority isEqualToString: @"HIGH"])
+      else if ([priority intValue] == 2)
         {
           [headers setObject: @"2 (High)"  forKey: @"X-Priority"];
         }
-      else if ([priority isEqualToString: @"LOW"])
+      else if ([priority intValue] == 4)
         {
           [headers setObject: @"4 (Low)"  forKey: @"X-Priority"];
         }
-      else
+      else if ([priority intValue] == 5)
         {
           [headers setObject: @"5 (Lowest)"  forKey: @"X-Priority"];
         }
+      else
+        {
+          [headers removeObjectForKey: @"X-Priority"];
+        }
       if (priority)
         {
           [headers setObject: priority  forKey: @"priority"];
@@ -355,7 +380,7 @@ static NSString    *userAgent      = nil;
   else
     {
       receipt = [newHeaders objectForKey: @"receipt"];
-      if ([receipt isEqualToString: @"true"])
+      if ([receipt boolValue])
         {
           [headers setObject: receipt  forKey: @"receipt"];
           pureSender = [[newHeaders objectForKey: @"from"] pureEMailAddress];
@@ -618,6 +643,13 @@ static NSString    *userAgent      = nil;
   error = nil;
   message = [self mimeMessageAsData];
 
+  if (!message)
+    {
+      error = [NSException exceptionWithHTTPStatus: 500 /* Server Error */
+					    reason: @"Message is too big"];
+      return error;
+    }
+
   client = [[self imap4Connection] client];
 
   if (![imap4 doesMailboxExistAtURL: [container imap4URL]])
@@ -626,7 +658,7 @@ static NSString    *userAgent      = nil;
 				      atURL: [[self mailAccountFolder] imap4URL]];
       [imap4 flushFolderHierarchyCache];
     }
-  
+
   folder = [imap4 imap4FolderNameForURL: [container imap4URL]];
   result = [client append: message toFolder: folder
                 withFlags: [NSArray arrayWithObjects: @"draft", nil]];
@@ -641,7 +673,6 @@ static NSString    *userAgent      = nil;
           [imap4URL release];
           imap4URL = nil;
         }
-      [self storeInfo];
     }
   else
     error = [NSException exceptionWithHTTPStatus: 500 /* Server Error */
@@ -699,7 +730,8 @@ static NSString    *userAgent      = nil;
     for (count = max - 1; count >= 0; count--)
       {
 	currentAddress = [addresses objectAtIndex: count];
-	if ([currentRecipient
+	if (![currentAddress baseEMail] ||
+            [currentRecipient
               caseInsensitiveCompare: [currentAddress baseEMail]]
             == NSOrderedSame)
 	  {
@@ -714,6 +746,7 @@ static NSString    *userAgent      = nil;
 //
 - (void) _fillInReplyAddresses: (NSMutableDictionary *) _info
 		    replyToAll: (BOOL) _replyToAll
+               fromSentMailbox: (BOOL) _fromSentMailbox
 		      envelope: (NGImap4Envelope *) _envelope
 {
   /*
@@ -721,12 +754,10 @@ static NSString    *userAgent      = nil;
     - if there is a 'reply-to' header, only include that (as TO)
     - if we reply to all, all non-from addresses are added as CC
     - the from is always the lone TO (except for reply-to)
-    
+
     Note: we cannot check reply-to, because Cyrus even sets a reply-to in the
           envelope if none is contained in the message itself! (bug or
           feature?)
-    
-    TODO: what about sender (RFC 822 3.6.2)
   */
   NSMutableArray *to, *addrs, *allRecipients;
   NSArray *envelopeAddresses;
@@ -754,11 +785,11 @@ static NSString    *userAgent      = nil;
       int i;
 
       identities = [[[self container] mailAccountFolder] identities];
-      
+
       for (i = 0; i < [identities count]; i++)
         {
           email = [[identities objectAtIndex: i] objectForKey: @"email"];
-          
+
           if (email)
             [allRecipients addObject: email];
         }
@@ -768,7 +799,9 @@ static NSString    *userAgent      = nil;
 
   addrs = [NSMutableArray array];
   envelopeAddresses = [_envelope replyTo];
-  if ([envelopeAddresses count])
+  if (_fromSentMailbox)
+    [addrs setArray: [_envelope to]];
+  else if ([envelopeAddresses count])
     [addrs setArray: envelopeAddresses];
   else
     [addrs setArray: [_envelope from]];
@@ -814,63 +847,29 @@ static NSString    *userAgent      = nil;
       [self _purgeRecipients: allRecipients
 	    fromAddresses: addrs];
       [self _addEMailsOfAddresses: addrs toArray: to];
-    
+
       [_info setObject: to forKey: @"cc"];
 
       [to release];
     }
 }
 
-//
-//
-//
-- (NSArray *) _attachmentBodiesFromPaths: (NSArray *) paths
-		       fromResponseFetch: (NSDictionary *) fetch;
-{
-  NSEnumerator *attachmentKeys;
-  NSMutableArray *bodies;
-  NSString *currentKey;
-  NSDictionary *body;
-
-  bodies = [NSMutableArray array];
-
-  attachmentKeys = [paths objectEnumerator];
-  while ((currentKey = [attachmentKeys nextObject]))
-    {
-      body = [fetch objectForKey: [currentKey lowercaseString]];
-      [bodies addObject: [body objectForKey: @"data"]];
-    }
-
-  return bodies;
-}
-
 //
 //
 //
 - (void) _fetchAttachmentsFromMail: (SOGoMailObject *) sourceMail
 {
-  unsigned int count, max;
-  NSArray *parts, *paths, *bodies;
-  NSData *body;
+  unsigned int max, count;
+  NSArray *attachments;
   NSDictionary *currentInfo;
-  NGHashMap *response;
 
-  parts = [sourceMail fetchFileAttachmentKeys];
-  max = [parts count];
-  if (max > 0)
+  attachments = [sourceMail fetchFileAttachments];
+  max = [attachments count];
+  for (count = 0; count < max; count++)
     {
-      paths = [parts keysWithFormat: @"BODY[%{path}]"];
-      response = [[sourceMail fetchParts: paths] objectForKey: @"RawResponse"];
-      bodies = [self _attachmentBodiesFromPaths: paths
-		     fromResponseFetch: [response objectForKey: @"fetch"]];
-      for (count = 0; count < max; count++)
-	{
-	  currentInfo = [parts objectAtIndex: count];
-	  body = [[bodies objectAtIndex: count]
-		   bodyDataFromEncoding: [currentInfo
-					   objectForKey: @"encoding"]];
-	  [self saveAttachment: body withMetadata: currentInfo];
-	}
+      currentInfo = [attachments objectAtIndex: count];
+      [self saveAttachment: [currentInfo objectForKey: @"body"]
+              withMetadata: currentInfo];
     }
 }
 
@@ -900,6 +899,10 @@ static NSString    *userAgent      = nil;
     [info setObject: msgid forKey: @"message-id"];
 
   addresses = [NSMutableArray array];
+  [self _addEMailsOfAddresses: [sourceEnvelope from] toArray: addresses];
+  if ([addresses count])
+    [info setObject: [addresses objectAtIndex: 0] forKey: @"from"];
+  addresses = [NSMutableArray array];
   [self _addEMailsOfAddresses: [sourceEnvelope to] toArray: addresses];
   [info setObject: addresses forKey: @"to"];
   addresses = [NSMutableArray array];
@@ -927,8 +930,6 @@ static NSString    *userAgent      = nil;
 
   [self setText: [sourceMail contentForEditing]];
   [self setIMAP4ID: [[sourceMail nameInContainer] intValue]];
-
-  [self storeInfo];
 }
 
 //
@@ -937,17 +938,21 @@ static NSString    *userAgent      = nil;
 - (void) fetchMailForReplying: (SOGoMailObject *) sourceMail
 			toAll: (BOOL) toAll
 {
+  BOOL fromSentMailbox;
   NSString *msgID;
   NSMutableDictionary *info;
   NGImap4Envelope *sourceEnvelope;
 
+  fromSentMailbox = [[sourceMail container] isKindOfClass: [SOGoSentFolder class]];
   [sourceMail fetchCoreInfos];
 
   info = [NSMutableDictionary dictionaryWithCapacity: 16];
   [info setObject: [sourceMail subjectForReply] forKey: @"subject"];
 
   sourceEnvelope = [sourceMail envelope];
-  [self _fillInReplyAddresses: info replyToAll: toAll
+  [self _fillInReplyAddresses: info
+                   replyToAll: toAll
+              fromSentMailbox: fromSentMailbox
                      envelope: sourceEnvelope];
   msgID = [sourceEnvelope messageID];
   if ([msgID length] > 0)
@@ -969,14 +974,14 @@ static NSString    *userAgent      = nil;
   SOGoUserDefaults *ud;
 
   [sourceMail fetchCoreInfos];
-  
+
   if ([sourceMail subjectForForward])
     {
-      info = [NSDictionary dictionaryWithObject: [sourceMail subjectForForward] 
+      info = [NSDictionary dictionaryWithObject: [sourceMail subjectForForward]
 			   forKey: @"subject"];
       [self setHeaders: info];
     }
-  
+
   [self setSourceURL: [sourceMail imap4URLString]];
   [self setSourceFlag: @"$Forwarded"];
   [self setSourceIMAP4ID: [[sourceMail nameInContainer] intValue]];
@@ -1007,11 +1012,11 @@ static NSString    *userAgent      = nil;
               withMetadata: attachment];
     }
 
-  [self storeInfo];
-
   // Save the message to the IMAP store so the user can eventually view the attached file(s)
   // from the Web interface
   [self save];
+
+  [self storeInfo];
 }
 
 /* accessors */
@@ -1019,7 +1024,7 @@ static NSString    *userAgent      = nil;
 - (NSString *) sender
 {
   id tmp;
-  
+
   if ((tmp = [headers objectForKey: @"from"]) == nil)
     return nil;
   if ([tmp isKindOfClass:[NSArray class]])
@@ -1065,52 +1070,36 @@ static NSString    *userAgent      = nil;
   return ma;
 }
 
-- (BOOL) isValidAttachmentName: (NSString *) filename
-{
-  return (!([filename rangeOfString: @"/"].length
-	    || [filename isEqualToString: @"."]
-	    || [filename isEqualToString: @".."]));
-}
 
 - (NSString *) pathToAttachmentWithName: (NSString *) _name
 {
   if ([_name length] == 0)
     return nil;
-  
+
   return [[self draftFolderPath] stringByAppendingPathComponent:_name];
 }
 
-- (NSException *) invalidAttachmentNameError: (NSString *) _name
-{
-  return [NSException exceptionWithHTTPStatus:400 /* Bad Request */
-		      reason: @"Invalid attachment name!"];
-}
 
+/**
+ * Write attachment file to the spool directory of the draft and write a dot
+ * file with its mime type.
+ */
 - (NSException *) saveAttachment: (NSData *) _attach
 		    withMetadata: (NSDictionary *) metadata
 {
-  NSString *p, *name, *mimeType;
-  NSRange r;
+  NSString *p, *pmime, *name, *mimeType;
 
   if (![_attach isNotNull]) {
     return [NSException exceptionWithHTTPStatus:400 /* Bad Request */
 			reason: @"Missing attachment content!"];
   }
-  
+
   if (![self _ensureDraftFolderPath]) {
     return [NSException exceptionWithHTTPStatus:500 /* Server Error */
 			reason: @"Could not create folder for draft!"];
   }
 
-  name = [metadata objectForKey: @"filename"];
-  r = [name rangeOfString: @"\\"
-	    options: NSBackwardsSearch];
-  if (r.length > 0)
-    name = [name substringFromIndex: r.location + 1];
-
-  if (![self isValidAttachmentName: name])
-    return [self invalidAttachmentNameError: name];
-  
+  name = [[metadata objectForKey: @"filename"] asSafeFilename];
   p = [self pathToAttachmentWithName: name];
   if (![_attach writeToFile: p atomically: YES])
     {
@@ -1121,16 +1110,15 @@ static NSString    *userAgent      = nil;
   mimeType = [metadata objectForKey: @"mimetype"];
   if ([mimeType length] > 0)
     {
-      p = [self pathToAttachmentWithName:
-		  [NSString stringWithFormat: @".%@.mime", name]];
-      if (![[mimeType dataUsingEncoding: NSUTF8StringEncoding]
-	     writeToFile: p atomically: YES])
-	{
-	  return [NSException exceptionWithHTTPStatus:500 /* Server Error */
-			      reason: @"Could not write attachment to draft!"];
-	}
+      pmime = [self pathToAttachmentWithName: [NSString stringWithFormat: @".%@.mime", name]];
+      if (![[mimeType dataUsingEncoding: NSUTF8StringEncoding] writeToFile: pmime atomically: YES])
+        {
+          [[NSFileManager defaultManager] removeFileAtPath: p  handler: nil];
+          return [NSException exceptionWithHTTPStatus: 500 /* Server Error */
+                                               reason: @"Could not write attachment to draft!"];
+        }
     }
-  
+
   return nil; /* everything OK */
 }
 
@@ -1141,21 +1129,14 @@ static NSString    *userAgent      = nil;
   NSException *error;
 
   error = nil;
+  fm = [NSFileManager defaultManager];
+  p = [self pathToAttachmentWithName: [_name asSafeFilename]];
+  if ([fm fileExistsAtPath: p])
+    if (![fm removeFileAtPath: p handler: nil])
+      error = [NSException exceptionWithHTTPStatus: 500 /* Server Error */
+                                            reason: @"Could not delete attachment from draft!"];
 
-  if ([self isValidAttachmentName:_name]) 
-    {
-      fm = [NSFileManager defaultManager];
-      p = [self pathToAttachmentWithName:_name];
-      if ([fm fileExistsAtPath: p])
-	if (![fm removeFileAtPath: p handler: nil])
-	  error
-	    = [NSException exceptionWithHTTPStatus: 500 /* Server Error */
-			   reason: @"Could not delete attachment from draft!"];
-    }
-  else
-    error = [self invalidAttachmentNameError:_name];
-
-  return error;  
+  return error;
 }
 
 //
@@ -1169,9 +1150,9 @@ static NSString    *userAgent      = nil;
 
   /* prepare header of body part */
   map = [[[NGMutableHashMap alloc] initWithCapacity: 1] autorelease];
-  
+
   [map setObject: contentTypeValue forKey: @"content-type"];
-  
+
    /* prepare body content */
   bodyPart = [[[NGMimeBodyPart alloc] initWithHeader:map] autorelease];
 
@@ -1192,7 +1173,7 @@ static NSString    *userAgent      = nil;
   */
   NGMutableHashMap *map;
   NGMimeBodyPart   *bodyPart;
-  
+
   /* prepare header of body part */
   map = [[[NGMutableHashMap alloc] initWithCapacity: 1] autorelease];
 
@@ -1200,7 +1181,7 @@ static NSString    *userAgent      = nil;
   if (text)
     [map setObject: (isHTML ? htmlContentTypeValue : contentTypeValue)
             forKey: @"content-type"];
-  
+
   /* prepare body content */
   bodyPart = [[[NGMimeBodyPart alloc] initWithHeader:map] autorelease];
   [bodyPart setBody: text];
@@ -1210,11 +1191,11 @@ static NSString    *userAgent      = nil;
 
 - (NGMimeMessage *) mimeMessageForContentWithHeaderMap: (NGMutableHashMap *) map
 {
-  NGMimeMessage *message;  
+  NGMimeMessage *message;
   id body;
 
   message = [[[NGMimeMessage alloc] initWithHeader:map] autorelease];
-  
+
   if (!isHTML)
     {
       [message setHeader: contentTypeValue  forKey: @"content-type"];
@@ -1231,9 +1212,9 @@ static NSString    *userAgent      = nil;
       // Add the HTML part
       [body addBodyPart: [self bodyPartForText]];
     }
-  
+
   [message setBody: body];
-  
+
   return message;
 }
 
@@ -1255,9 +1236,8 @@ static NSString    *userAgent      = nil;
 {
   NSString *s, *p;
   NSData *mimeData;
-  
-  p = [self pathToAttachmentWithName:
-	      [NSString stringWithFormat: @".%@.mime", _name]];
+
+  p = [self pathToAttachmentWithName: [NSString stringWithFormat: @".%@.mime", _name]];
   mimeData = [NSData dataWithContentsOfFile: p];
   if (mimeData)
     {
@@ -1269,33 +1249,31 @@ static NSString    *userAgent      = nil;
     {
       s = [self mimeTypeForExtension:[_name pathExtension]];
       if ([_name length] > 0)
-	s = [s stringByAppendingFormat: @"; name=\"%@\"", _name];
+	s = [s stringByAppendingFormat: @"; name=\"%@\"", [_name stringByReplacingOccurrencesOfString:@"\"" withString:@"\\\""]];
     }
 
   return s;
 }
 
 - (NSString *) contentDispositionForAttachmentWithName: (NSString *) _name
+                                        andContentType: (NSString *) _type
 {
-  NSString *type;
   NSString *cdtype;
   NSString *cd;
   SOGoDomainDefaults *dd;
-  
-  type = [self contentTypeForAttachmentWithName:_name];
 
-  if ([type hasPrefix: @"text/"])
+  if ([_type hasPrefix: @"text/"])
     {
       dd = [[context activeUser] domainDefaults];
       cdtype = [dd mailAttachTextDocumentsInline] ? @"inline" : @"attachment";
     }
-  else if ([type hasPrefix: @"image/"] || [type hasPrefix: @"message"])
+  else if ([_type hasPrefix: @"image/"] || [_type hasPrefix: @"message"])
     cdtype = @"inline";
   else
     cdtype = @"attachment";
 
   cd = [cdtype stringByAppendingString: @"; filename=\""];
-  cd = [cd stringByAppendingString: _name];
+  cd = [cd stringByAppendingString: [_name stringByReplacingOccurrencesOfString:@"\"" withString:@"\\\""]];
   cd = [cd stringByAppendingString: @"\""];
 
   // TODO: add size parameter (useful addition, RFC 2183)
@@ -1316,7 +1294,7 @@ static NSString    *userAgent      = nil;
   if (_name == nil) return nil;
 
   /* check attachment */
-  
+
   fm = [NSFileManager defaultManager];
   p  = [self pathToAttachmentWithName: _name];
   if (![fm isReadableFileAtPath: p]) {
@@ -1325,7 +1303,7 @@ static NSString    *userAgent      = nil;
   }
   attachAsString = NO;
   attachAsRFC822 = NO;
-  
+
   /* prepare header of body part */
 
   map = [[[NGMutableHashMap alloc] initWithCapacity: 4] autorelease];
@@ -1337,22 +1315,22 @@ static NSString    *userAgent      = nil;
     else if ([s hasPrefix: @"message/rfc822"])
       attachAsRFC822 = YES;
   }
-  if ((s = [self contentDispositionForAttachmentWithName: _name]))
+  if ((s = [self contentDispositionForAttachmentWithName: _name andContentType: s]))
     {
       NGMimeContentDispositionHeaderField *o;
-      
+
       o = [[NGMimeContentDispositionHeaderField alloc] initWithString: s];
       [map setObject: o forKey: @"content-disposition"];
       [o release];
     }
-  
+
   /* prepare body content */
-  
+
   if (attachAsString) { // TODO: is this really necessary?
     NSString *s;
-    
+
     content = [[NSData alloc] initWithContentsOfMappedFile:p];
-    
+
     s = [[NSString alloc] initWithData: content
                               encoding: [NSString defaultCStringEncoding]];
     if (s != nil) {
@@ -1367,7 +1345,7 @@ static NSString    *userAgent      = nil;
     }
   }
   else {
-    /* 
+    /*
        Note: in OGo this is done in LSWImapMailEditor.m:2477. Apparently
              NGMimeFileData objects are not processed by the MIME generator!
     */
@@ -1383,34 +1361,43 @@ static NSString    *userAgent      = nil;
 	content = [content dataByEncodingBase64];
         [map setObject: @"base64" forKey: @"content-transfer-encoding"];
       }
-    [map setObject:[NSNumber numberWithInt:[content length]] 
+    [map setObject:[NSNumber numberWithInt:[content length]]
 	 forKey: @"content-length"];
-    
+
     /* Note: the -init method will create a temporary file! */
     body = [[NGMimeFileData alloc] initWithBytes:[content bytes]
                                           length:[content length]];
   }
-  
+
   bodyPart = [[[NGMimeBodyPart alloc] initWithHeader:map] autorelease];
   [bodyPart setBody:body];
-  
+
   [body release]; body = nil;
   return bodyPart;
 }
 
 //
-//
+// returns nil on error
 //
 - (NSArray *) bodyPartsForAllAttachments
 {
-  /* returns nil on error */
-  NSArray  *attrs;
-  unsigned i, count;
   NGMimeBodyPart *bodyPart;
   NSMutableArray *bodyParts;
+  NSArray  *attrs;
+  unsigned i, count, size, limit;
 
   attrs = [self fetchAttachmentAttrs];
   count = [attrs count];
+  size = 0;
+
+  // We first check if we don't go over our message size limit
+  limit = [[SOGoSystemDefaults sharedSystemDefaults] maximumMessageSizeLimit] * 1024;
+  for (i = 0; i < count; i++)
+    size += [[[attrs objectAtIndex: i] objectForKey: @"size"] intValue];
+
+  if (limit && size > limit)
+    return nil;
+
   bodyParts = [NSMutableArray arrayWithCapacity: count];
 
   for (i = 0; i < count; i++)
@@ -1430,14 +1417,14 @@ static NSString    *userAgent      = nil;
   NGMimeMultipartBody *textParts;
   NGMutableHashMap *header;
   NGMimeBodyPart *part;
-  
+
   header = [NGMutableHashMap hashMap];
   [header addObject: MultiAlternativeType forKey: @"content-type"];
-  
+
   part = [NGMimeBodyPart bodyPartWithHeader: header];
-  
+
   textParts = [[NGMimeMultipartBody alloc] initWithPart: part];
-  
+
   // Get the text part from it and add it
   [textParts addBodyPart: [self plainTextBodyPartForText]];
 
@@ -1456,25 +1443,21 @@ static NSString    *userAgent      = nil;
 - (NGMimeMessage *) mimeMultiPartMessageWithHeaderMap: (NGMutableHashMap *) map
 					 andBodyParts: (NSArray *) _bodyParts
 {
-  NGMimeMessage       *message;  
+  NGMimeMessage       *message;
   NGMimeMultipartBody *mBody;
   NSEnumerator        *e;
   id                  part;
-  
+
   [map addObject: MultiMixedType forKey: @"content-type"];
 
   message = [[NGMimeMessage alloc] initWithHeader: map];
   [message autorelease];
   mBody = [[NGMimeMultipartBody alloc] initWithPart: message];
-  
+
   if (!isHTML)
-    {
-      part = [self bodyPartForText];
-    }
+    part = [self bodyPartForText];
   else
-    {
-      part = [self mimeMultipartAlternative];
-    }
+    part = [self mimeMultipartAlternative];
 
   [mBody addBodyPart: part];
 
@@ -1503,11 +1486,11 @@ static NSString    *userAgent      = nil;
 
   if ([_h count] == 0)
     return;
-    
+
   names = [_h keyEnumerator];
   while ((name = [names nextObject]) != nil) {
     id value;
-      
+
     value = [_h objectForKey:name];
     [_map addObject:value forKey:name];
   }
@@ -1517,10 +1500,10 @@ static NSString    *userAgent      = nil;
 {
   if (![_value isNotNull])
     return YES;
-  
+
   if ([_value isKindOfClass: [NSArray class]])
     return [_value count] == 0 ? YES : NO;
-  
+
   if ([_value isKindOfClass: [NSString class]])
     return [_value length] == 0 ? YES : NO;
 
@@ -1597,22 +1580,21 @@ static NSString    *userAgent      = nil;
 {
   NSString *s, *dateString;
   NGMutableHashMap *map;
-  NSArray *emails;
-  id from, replyTo;
-  
+  id emails, from, replyTo;
+
   map = [[[NGMutableHashMap alloc] initWithCapacity:16] autorelease];
-  
+
   /* add recipients */
-  if ((emails = [headers objectForKey: @"to"]) != nil)
+  if ((emails = [headers objectForKey: @"to"]) != nil && [emails isKindOfClass: [NSArray class]])
     [map setObjects: [self _quoteSpecialsInArray: emails] forKey: @"to"];
-  if ((emails = [headers objectForKey: @"cc"]) != nil)
+  if ((emails = [headers objectForKey: @"cc"]) != nil && [emails isKindOfClass: [NSArray class]])
     [map setObjects: [self _quoteSpecialsInArray: emails] forKey: @"cc"];
-  if ((emails = [headers objectForKey: @"bcc"]) != nil)
+  if ((emails = [headers objectForKey: @"bcc"]) != nil && [emails isKindOfClass: [NSArray class]])
     [map setObjects: [self _quoteSpecialsInArray: emails] forKey: @"bcc"];
 
   /* add senders */
   from = [headers objectForKey: @"from"];
-  
+
   if (![self isEmptyValue:from]) {
     if ([from isKindOfClass:[NSArray class]])
       [map setObjects: [self _quoteSpecialsInArray: from] forKey: @"from"];
@@ -1630,7 +1612,7 @@ static NSString    *userAgent      = nil;
   if ([(s = [headers objectForKey: @"subject"]) length] > 0)
     [map setObject: [s asQPSubjectString: @"utf-8"]
             forKey: @"subject"];
-  
+
   if ([(s = [headers objectForKey: @"message-id"]) length] > 0)
     [map setObject: s
             forKey: @"message-id"];
@@ -1654,16 +1636,16 @@ static NSString    *userAgent      = nil;
 	 forKey: @"Disposition-Notification-To"];
 
   [self _addHeaders: _headers toHeaderMap: map];
-  
+
   // We remove what we have to...
   if (_exclude)
     {
       int i;
-  
+
       for (i = 0; i < [_exclude count]; i++)
 	[map removeAllObjectsForKey: [_exclude objectAtIndex: i]];
     }
-  
+
   return map;
 }
 
@@ -1676,18 +1658,24 @@ static NSString    *userAgent      = nil;
 {
   NSMutableArray *bodyParts;
   NGMimeMessage *message;
+  NSArray *allBodyParts;
   NGMutableHashMap *map;
   NSString *newText;
 
-  message = nil;
+  BOOL has_inline_images;
 
+  message = nil;
+  has_inline_images = NO;
   bodyParts = [NSMutableArray array];
 
   if (_extractImages)
     {
       newText = [text htmlByExtractingImages: bodyParts];
       if ([bodyParts count])
-        [self setText: newText];
+        {
+          [self setText: newText];
+          has_inline_images = YES;
+        }
     }
 
   map = [self mimeHeaderMapWithHeaders: _headers
@@ -1695,21 +1683,35 @@ static NSString    *userAgent      = nil;
   if (map)
     {
       //[self debugWithFormat: @"MIME Envelope: %@", map];
-      
-      [bodyParts addObjectsFromArray: [self bodyPartsForAllAttachments]];
+      allBodyParts = [self bodyPartsForAllAttachments];
+
+      if (!allBodyParts)
+	return nil;
+
+      [bodyParts addObjectsFromArray: allBodyParts];
       
       //[self debugWithFormat: @"attachments: %@", bodyParts];
-      
+
       if ([bodyParts count] == 0)
         /* no attachments */
         message = [self mimeMessageForContentWithHeaderMap: map];
       else
-        /* attachments, create multipart/mixed */
-        message = [self mimeMultiPartMessageWithHeaderMap: map 
-                                             andBodyParts: bodyParts];
-      //[self debugWithFormat: @"message: %@", message];
+        {
+          // attachments, create multipart/mixed or multipart/related if
+          // we have inline image to avoid Thunderbird bug #61815 (https://bugzilla.mozilla.org/show_bug.cgi?id=61815)
+          if (has_inline_images)
+            {
+              [map removeAllObjectsForKey: @"content-type"];
+              [map addObject: MultiRelatedType forKey: @"content-type"];
+            }
+
+          message = [self mimeMultiPartMessageWithHeaderMap: map
+                                               andBodyParts: bodyParts];
+
+          //[self debugWithFormat: @"message: %@", message];
+        }
     }
-  
+
   return message;
 }
 
@@ -1727,10 +1729,19 @@ static NSString    *userAgent      = nil;
 - (NSData *) mimeMessageAsData
 {
   NGMimeMessageGenerator *generator;
+  NGMimeMessage *mimeMessage;
   NSData *message;
 
   generator = [NGMimeMessageGenerator new];
-  message = [generator generateMimeFromPart: [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: NO]];
+  mimeMessage = [self mimeMessageWithHeaders: nil  excluding: nil  extractingImages: NO];
+
+  if (!mimeMessage)
+    {
+      [generator release];
+      return nil;
+    }
+
+  message = [generator generateMimeFromPart: mimeMessage];
   [generator release];
 
   return message;
@@ -1782,9 +1793,9 @@ static NSString    *userAgent      = nil;
 - (NSException *) sendMail
 {
   SOGoUserDefaults *ud;
-  
+
   ud = [[context activeUser] userDefaults];
-  
+
   if ([ud mailAddOutgoingAddresses])
     {
       NSString *recipient, *emailAddress, *addressBook, *uid;
@@ -1793,11 +1804,11 @@ static NSString    *userAgent      = nil;
       SOGoContactGCSEntry *newContact;
       NGMailAddress *parsedRecipient;
       NGMailAddressParser *parser;
-      SOGoContactFolder *folder;
+      SOGoFolder <SOGoContactFolder> *folder;
       NGVCard *card;
- 
+
       int i;
-    
+
       // Get all the addressbooks
       contactFolders = [[[context activeUser] homeFolderInContext: context]
                          lookupName: @"Contacts"
@@ -1812,7 +1823,7 @@ static NSString    *userAgent      = nil;
           parser = [NGMailAddressParser mailAddressParserWithString: recipient];
           parsedRecipient = [parser parse];
           emailAddress = [parsedRecipient address];
-      
+
           matchingContacts = [contactFolders allContactsFromFilter: emailAddress
                                                      excludeGroups: YES
                                                       excludeLists: YES];
@@ -1824,13 +1835,13 @@ static NSString    *userAgent      = nil;
           addressBook = [ud selectedAddressBook];
           folder = [contactFolders lookupName: addressBook inContext: context  acquire: NO];
           uid = [folder globallyUniqueObjectId];
-      
+
           if (folder && uid)
             {
               card = [NGVCard cardWithUid: uid];
               [card addEmail: emailAddress types: nil];
               [card setFn: [parsedRecipient displayName]];
-              
+
               newContact = [SOGoContactGCSEntry objectWithName: uid
                                                    inContainer: folder];
               [newContact setIsNew: YES];
@@ -1852,9 +1863,9 @@ static NSString    *userAgent      = nil;
   NSURL *sourceIMAP4URL;
   NSException *error;
   NSData *message;
-  NSRange r1, r2;
-  
-  /* send mail */
+  NSRange r1;
+
+  unsigned int limit;
 
   // We strip the BCC fields prior sending any mails
   NGMimeMessageGenerator *generator;
@@ -1868,21 +1879,32 @@ static NSString    *userAgent      = nil;
   //
 #warning FIXME - we should fix the case issue when we switch to Pantomime
   cleaned_message = [NSMutableData dataWithData: message];
+
+  // We search only in the headers so we start at 0 until
+  // we find \r\n\r\n, which is the headers delimiter
   r1 = [cleaned_message rangeOfCString: "\r\n\r\n"];
+  limit = r1.location-1;
   r1 = [cleaned_message rangeOfCString: "\r\nbcc: "
                                options: 0
-                                 range: NSMakeRange(0,r1.location-1)];
-      
+                                 range: NSMakeRange(0,limit)];
+
   if (r1.location != NSNotFound)
     {
       // We search for the first \r\n AFTER the Bcc: header and
       // replace the whole thing with \r\n.
-      r2 = [cleaned_message rangeOfCString: "\r\n"
-                                   options: 0
-                                     range: NSMakeRange(NSMaxRange(r1)+1,[cleaned_message length]-NSMaxRange(r1)-1)];
-      [cleaned_message replaceBytesInRange: NSMakeRange(r1.location, NSMaxRange(r2)-r1.location)
-                                 withBytes: "\r\n"
-                                    length: 2];
+      unsigned int i;
+
+      for (i = r1.location+7; i < limit; i++)
+        {
+          if ([cleaned_message characterAtIndex: i] == '\r' &&
+              (i+1 < limit && [cleaned_message characterAtIndex: i+1] == '\n') &&
+              (i+2 < limit && !isspace([cleaned_message characterAtIndex: i+2])))
+            break;
+        }
+
+      [cleaned_message replaceBytesInRange: NSMakeRange(r1.location, i-r1.location)
+                                 withBytes: NULL
+                                    length: 0];
     }
 
   dd = [[context activeUser] domainDefaults];
@@ -1914,8 +1936,16 @@ static NSString    *userAgent      = nil;
         }
     }
 
-  if (!error && ![dd mailKeepDraftsAfterSend])
-    [self delete];
+  // Expunge Drafts mailbox if
+  //  - message was sent and saved to Sent mailbox if necessary;
+  //  - SOGoMailKeepDraftsAfterSend is not set;
+  //  - draft is successfully deleted;
+  //  - drafts mailbox exists.
+  if (!error &&
+      ![dd mailKeepDraftsAfterSend] &&
+      ![self delete] &&
+      [imap4 doesMailboxExistAtURL: [container imap4URL]])
+    [(SOGoDraftsFolder *) container expunge];
 
   return error;
 }
