@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2007-2014 Inverse inc.
+  Copyright (C) 2007-2016 Inverse inc.
 
   This file is part of SOGo.
 
@@ -20,44 +19,34 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSURL.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSValue.h>
 
-#import <DOM/DOMElement.h>
-#import <DOM/DOMProtocols.h>
-
 #import <NGObjWeb/NSException+HTTP.h>
-#import <NGObjWeb/SoHTTPAuthenticator.h>
-#import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGImap4/NGImap4Connection.h>
 #import <NGImap4/NGImap4Client.h>
-#import <NGImap4/NGImap4Context.h>
 #import <NGImap4/NSString+Imap4.h>
 
 #import <SOGo/NSArray+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoAuthenticator.h>
 #import <SOGo/SOGoDomainDefaults.h>
-#import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserDefaults.h>
+#import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/SOGoUserManager.h>
 #import <SOGo/SOGoSieveManager.h>
 
 #import "SOGoDraftsFolder.h"
-#import "SOGoMailFolder.h"
-#import "SOGoMailManager.h"
 #import "SOGoMailNamespace.h"
 #import "SOGoSentFolder.h"
 #import "SOGoTrashFolder.h"
+#import "SOGoJunkFolder.h"
 #import "SOGoUser+Mailer.h"
 
 #import "SOGoMailAccount.h"
@@ -74,14 +63,18 @@ static NSString *inboxFolderName = @"INBOX";
 {
   if ((self = [super init]))
     {
+      NSString *sieveFolderEncoding = [[SOGoSystemDefaults sharedSystemDefaults] sieveFolderEncoding];
       inboxFolder = nil;
       draftsFolder = nil;
       sentFolder = nil;
       trashFolder = nil;
+      junkFolder = nil;
       imapAclStyle = undefined;
       identities = nil;
       otherUsersFolderName = nil;
       sharedFoldersName = nil;
+      subscribedFolders = nil;
+      sieveFolderUTF8Encoding = [sieveFolderEncoding isEqualToString: @"UTF-8"];
     }
 
   return self;
@@ -93,14 +86,14 @@ static NSString *inboxFolderName = @"INBOX";
   [draftsFolder release];
   [sentFolder release];
   [trashFolder release];
+  [junkFolder release];
   [identities release];
   [otherUsersFolderName release];
   [sharedFoldersName release];
+  [subscribedFolders release];
   [super dealloc];  
 }
 
-/* listing the available folders */
-
 - (BOOL) isInDraftsFolder
 {
   return NO;
@@ -124,8 +117,6 @@ static NSString *inboxFolderName = @"INBOX";
     }
 }
 
-/* namespaces */
-
 - (void) _appendNamespaces: (NSMutableArray *) folders
 {
   NSDictionary *namespaceDict;
@@ -206,8 +197,8 @@ static NSString *inboxFolderName = @"INBOX";
         [folders removeObjectsInArray: nss];
     }
 
-  return [[folders stringsWithFormat: @"folder%@"]
-           resultsOfSelector: @selector (asCSSIdentifier)];
+  return [[folders resultsOfSelector: @selector (asCSSIdentifier)]
+           stringsWithFormat: @"folder%@"];
 }
 
 - (NSArray *) toManyRelationshipKeys
@@ -300,7 +291,7 @@ static NSString *inboxFolderName = @"INBOX";
 	  quota = quota * [(NSNumber*)[inboxQuota objectForKey: @"maxQuota"] intValue];
 	  inboxQuota = [NSDictionary dictionaryWithObjectsAndKeys:
 					[NSNumber numberWithLong: (long)(quota+0.5)], @"maxQuota",
-					[inboxQuota objectForKey: @"usedSpace"], @"usedSpace",
+                                         [NSNumber numberWithLong: [[inboxQuota objectForKey: @"usedSpace"] longLongValue]], @"usedSpace",
 				     nil];
 	}
     }
@@ -350,39 +341,64 @@ static NSString *inboxFolderName = @"INBOX";
 //
 //
 //
-- (NSArray *) allFolderPaths
+- (NSArray *) allFolderPaths: (SOGoMailListingMode) theListingMode
 {
   NSMutableArray *folderPaths, *namespaces;
   NSArray *folders, *mainFolders;
-  SOGoUserDefaults *ud;
+  NSString *namespace;
+
   BOOL subscribedOnly;
   int count, max;
 
-  ud = [[context activeUser] userDefaults];
-  subscribedOnly = [ud mailShowSubscribedFoldersOnly];
+  if (theListingMode == SOGoMailStandardListing)
+      subscribedOnly = [[[context activeUser] userDefaults] mailShowSubscribedFoldersOnly];
+  else
+    {
+      subscribedOnly = NO;
+      DESTROY(subscribedFolders);
+      subscribedFolders = [[NSMutableDictionary alloc] init];
+      folders = [[self imap4Connection] allFoldersForURL: [self imap4URL]
+				   onlySubscribedFolders: YES];
+      max = [folders count];
+      for (count = 0; count < max; count++)
+	{
+	  [subscribedFolders setObject: [NSNull null]
+				forKey: [folders objectAtIndex: count]];
+	}
+      [[self imap4Connection] flushFolderHierarchyCache];
+    }
 
   mainFolders = [[NSArray arrayWithObjects:
 			    [self inboxFolderNameInContext: context],
 			  [self draftsFolderNameInContext: context],
 			  [self sentFolderNameInContext: context],
 			  [self trashFolderNameInContext: context],
-			  nil] stringsWithFormat: @"/%@"];
+			  [self junkFolderNameInContext: context],
+		    nil] stringsWithFormat: @"/%@"];
   folders = [[self imap4Connection] allFoldersForURL: [self imap4URL]
-                               onlySubscribedFolders: subscribedOnly];
+			       onlySubscribedFolders: subscribedOnly];
   folderPaths = [folders mutableCopy];
   [folderPaths autorelease];
+
   [folderPaths removeObjectsInArray: mainFolders];
   namespaces = [NSMutableArray arrayWithCapacity: 10];
   [self _appendNamespaces: namespaces];
   max = [namespaces count];
   for (count = 0; count < max; count++)
     {
-      folders = [self _allFoldersFromNS: [namespaces objectAtIndex: count]
+      namespace = [namespaces objectAtIndex: count];
+      folders = [self _allFoldersFromNS: namespace
                          subscribedOnly: subscribedOnly];
       if ([folders count])
         {
           [folderPaths removeObjectsInArray: folders];
           [folderPaths addObjectsFromArray: folders];
+
+          // We make sure our "shared" / "public" namespace is always defined. Cyrus does NOT
+          // return them in LIST while Dovecot does.
+          namespace = [NSString stringWithFormat: @"/%@", namespace];
+          [folderPaths removeObject: namespace];
+          [folderPaths addObject: namespace];
         }
     }
   [folderPaths
@@ -397,59 +413,179 @@ static NSString *inboxFolderName = @"INBOX";
 //
 //
 - (NSString *) _folderType: (NSString *) folderName
+                     flags: (NSMutableArray *) flags
 {
-  NSString *folderType;
+  NSString *folderType, *key;
+  NSDictionary *metadata;
+  SOGoUserDefaults *ud;
 
-  if ([folderName isEqualToString: [NSString stringWithFormat: @"/%@", inboxFolderName]])
+  ud = [[context activeUser] userDefaults];
+
+  metadata = [[[self imap4Connection] allFoldersMetadataForURL: [self imap4URL]
+                                         onlySubscribedFolders: [ud mailShowSubscribedFoldersOnly]]
+               objectForKey: @"list"];
+
+  key = [NSString stringWithFormat: @"/%@", folderName];
+  [flags addObjectsFromArray: [metadata objectForKey: key]];
+
+  // RFC6154 (https://tools.ietf.org/html/rfc6154) describes special uses for IMAP mailboxes.
+  // We do honor them, as long as your SOGo{Drafts,Trash,Sent,Junk}FolderName are properly configured
+  // See http://wiki.dovecot.org/MailboxSettings for a Dovecot example.
+  if ([folderName isEqualToString: inboxFolderName])
     folderType = @"inbox";
-  else if ([folderName isEqualToString: [NSString stringWithFormat: @"/%@", [self draftsFolderNameInContext: context]]])
+  else if ([flags containsObject: [self draftsFolderNameInContext: context]] ||
+           [folderName isEqualToString: [self draftsFolderNameInContext: context]])
     folderType = @"draft";
-  else if ([folderName isEqualToString: [NSString stringWithFormat: @"/%@", [self sentFolderNameInContext: context]]])
+  else if ([flags containsObject: [self sentFolderNameInContext: context]] ||
+           [folderName isEqualToString: [self sentFolderNameInContext: context]])
     folderType = @"sent";
-  else if ([folderName isEqualToString: [NSString stringWithFormat: @"/%@", [self trashFolderNameInContext: context]]])
+  else if ([flags containsObject: [self trashFolderNameInContext: context]] ||
+           [folderName isEqualToString: [self trashFolderNameInContext: context]])
     folderType = @"trash";
+  else if ([flags containsObject: [self junkFolderNameInContext: context]] ||
+           [folderName isEqualToString: [self junkFolderNameInContext: context]])
+    folderType = @"junk";
+  else if ([folderName isEqualToString: otherUsersFolderName])
+    folderType = @"otherUsers";
+  else if ([folderName isEqualToString: sharedFoldersName])
+    folderType = @"shared";
   else
     folderType = @"folder";
 
   return folderType;
 }
 
-- (NSString *) _parentForFolder: (NSString *) folderName
-                    foldersList: (NSArray *) theFolders
+//
+//
+//
+- (NSMutableDictionary *) _insertFolder: (NSString *) folderPath
+                            foldersList: (NSMutableArray *) theFolders
 {
   NSArray *pathComponents;
-  NSString *s;
-  int i;
+  NSMutableArray *folders, *flags;
+  NSMutableDictionary *currentFolder, *parentFolder, *folder;
+  NSString *currentFolderName, *currentPath, *sievePath, *fullName, *folderType;
+  SOGoUserManager *userManager;
 
-  pathComponents = [folderName pathComponents];
-  s = [[[pathComponents subarrayWithRange: NSMakeRange(0,[pathComponents count]-1)] componentsJoinedByString: @"/"] substringFromIndex: 1];
+  BOOL last, isOtherUsersFolder, parentIsOtherUsersFolder, isSubscribed;
+  int i, j, count;
 
-  for (i = 0; i < [theFolders count]; i++)
+  parentFolder = nil;
+  parentIsOtherUsersFolder = NO;
+  pathComponents = [folderPath pathComponents];
+  count = [pathComponents count];
+
+  // Make sure all ancestors exist.
+  // The variable folderPath is something like '/INBOX/Junk' so pathComponents becomes ('/', 'INBOX', 'Junk').
+  // That's why we always ignore the first element
+  for (i = 1; i < count; i++)
     {
-      if ([s isEqualToString: [theFolders objectAtIndex: i]])
-        return s;
+      last = ((count - i) == 1);
+      folder = nil;
+      currentPath = [[[pathComponents subarrayWithRange: NSMakeRange(0,i+1)] componentsJoinedByString: @"/"] substringFromIndex: 2];
+
+      // Search for the current path in the children of the parent folder.
+      // For the first iteration, take the parent folder passed as argument.
+      if (parentFolder)
+	folders = [parentFolder objectForKey: @"children"];
+      else
+	folders = theFolders;
+
+      for (j = 0; j < [folders count]; j++)
+        {
+          currentFolder = [folders objectAtIndex: j];
+          if ([currentPath isEqualToString: [currentFolder objectForKey: @"path"]])
+            {
+              folder = currentFolder;
+              // Make sure all branches are ready to receive children
+              if (!last && ![folder objectForKey: @"children"])
+                  [folder setObject: [NSMutableArray array] forKey: @"children"];
+
+	      break;
+            }
+        }
+
+      // Check if the current folder is the "Other users" folder (shared mailboxes)
+      currentFolderName = [[pathComponents objectAtIndex: i] stringByDecodingImap4FolderName];
+      if (otherUsersFolderName
+          && [currentFolderName caseInsensitiveCompare: otherUsersFolderName] == NSOrderedSame)
+	isOtherUsersFolder = YES;
+      else
+	isOtherUsersFolder = NO;
+
+      if (folder == nil)
+        {
+          // Folder was not found; create it and add it to the folders list
+          if (parentIsOtherUsersFolder)
+            {
+              // Parent folder is the "Other users" folder; translate the user's mailbox name
+              // to the full name of the person
+              userManager = [SOGoUserManager sharedUserManager];
+              if ((fullName = [userManager getCNForUID: currentFolderName]) && [fullName length])
+                currentFolderName = fullName;
+	      else if ((fullName = [userManager getEmailForUID: currentFolderName]) && [fullName length])
+		currentFolderName = fullName;
+            }
+          else if (isOtherUsersFolder)
+	    currentFolderName = [self labelForKey: @"OtherUsersFolderName"];
+          else if (sharedFoldersName
+                   && [currentFolderName caseInsensitiveCompare: sharedFoldersName] == NSOrderedSame)
+	    currentFolderName = [self labelForKey: @"SharedFoldersName"];
+
+          flags = [NSMutableArray array];;
+
+          if (last)
+            folderType = [self _folderType: currentPath
+                                     flags: flags];
+          else
+            folderType = @"additional";
+
+	  if ([subscribedFolders objectForKey: folderPath])
+	    isSubscribed = YES;
+	  else
+	    isSubscribed = NO;
+
+          folder = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                          currentPath, @"path",
+                                          folderType, @"type",
+                                          currentFolderName, @"name",
+                                          [NSMutableArray array], @"children",
+					  flags, @"flags",
+					  [NSNumber numberWithBool: isSubscribed], @"subscribed",
+					  nil];
+
+          if (sieveFolderUTF8Encoding)
+            sievePath = [folderPath stringByDecodingImap4FolderName];
+          else
+            sievePath = folderPath;
+          [folder setObject: [sievePath substringFromIndex: 1] forKey: @"sievePath"];
+
+          // Either add this new folder to its parent or the list of root folders
+          [folders addObject: folder];
+        }
+
+      parentFolder = folder;
+      parentIsOtherUsersFolder = isOtherUsersFolder;
     }
-  
-  return nil;
+
+  return parentFolder;
 }
 
 //
+// Return a tree representation of the mailboxes
 //
-//
-- (NSArray *) allFoldersMetadata
+- (NSArray *) allFoldersMetadata: (SOGoMailListingMode) theListingMode
 {
-  NSString *currentFolder, *currentDecodedFolder, *currentDisplayName, *currentFolderType, *login, *fullName, *parent;
-  NSMutableArray *pathComponents, *folders;
-  SOGoUserManager *userManager;
+  NSString *currentFolder;
+  NSMutableArray *folders;
   NSEnumerator *rawFolders;
-  NSDictionary *folderData;
   NSAutoreleasePool *pool;
   NSArray *allFolderPaths;
 
-  allFolderPaths = [self allFolderPaths];
+  allFolderPaths = [self allFolderPaths: theListingMode];
   rawFolders = [allFolderPaths objectEnumerator];
-
   folders = [NSMutableArray array];
+
   while ((currentFolder = [rawFolders nextObject]))
     {
       // Using a local pool to avoid using too many file descriptors. This could
@@ -458,63 +594,16 @@ static NSString *inboxFolderName = @"INBOX";
       // lots of LDAP connections during its execution.
       pool = [[NSAutoreleasePool alloc] init];
 
-      currentDecodedFolder = [currentFolder stringByDecodingImap4FolderName];
-      currentFolderType = [self _folderType: currentFolder];
+      // Insert folder into folders tree
+      [self _insertFolder: currentFolder
+              foldersList: folders];
 
-      // We translate the "Other Users" and "Shared Folders" namespaces.
-      // While we're at it, we also translate the user's mailbox names
-      // to the full name of the person.
-      if (otherUsersFolderName && [currentDecodedFolder hasPrefix: [NSString stringWithFormat: @"/%@", otherUsersFolderName]])
-        {
-          // We have a string like /Other Users/lmarcotte/... under Cyrus, but we could
-          // also have something like /shared under Dovecot. So we swap the username only
-          // if we have one, of course.
-          pathComponents = [NSMutableArray arrayWithArray: [currentDecodedFolder pathComponents]];
-
-          if ([pathComponents count] > 2) 
-            {
-              login = [pathComponents objectAtIndex: 2];
-              userManager = [SOGoUserManager sharedUserManager];
-              fullName = [userManager getCNForUID: login];
-              [pathComponents removeObjectsInRange: NSMakeRange(0,3)];
-
-              currentDisplayName = [NSString stringWithFormat: @"/%@/%@/%@", 
-                                     [self labelForKey: @"OtherUsersFolderName"],
-                                     (fullName != nil ? fullName : login),
-                                     [pathComponents componentsJoinedByString: @"/"]];
-
-            }
-          else
-            {
-              currentDisplayName = [NSString stringWithFormat: @"/%@%@",
-                                     [self labelForKey: @"OtherUsersFolderName"],
-                                      [currentDecodedFolder substringFromIndex:
-                                        [otherUsersFolderName length]+1]];
-            }
-        }
-      else if (sharedFoldersName && [currentDecodedFolder hasPrefix: [NSString stringWithFormat: @"/%@", sharedFoldersName]])
-        currentDisplayName = [NSString stringWithFormat: @"/%@%@", [self labelForKey: @"SharedFoldersName"],
-                           [currentDecodedFolder substringFromIndex: [sharedFoldersName length]+1]];
-      else
-        currentDisplayName = currentDecodedFolder;
-
-      parent = [self _parentForFolder: currentFolder  foldersList: allFolderPaths];
-      
-      folderData = [NSDictionary dictionaryWithObjectsAndKeys:
-                                   currentFolder, @"path",
-                                 currentFolderType, @"type",
-                                 currentDisplayName, @"displayName",
-                                 parent, @"parent",
-                                 nil];
-      [folders addObject: folderData];
       [pool release];
     }
 
   return folders;
 }
 
-
-/* IMAP4 */
 - (NSDictionary *) _mailAccount
 {
   NSDictionary *mailAccount;
@@ -665,22 +754,33 @@ static NSString *inboxFolderName = @"INBOX";
 
 - (NSDictionary *) imapFolderGUIDs
 {
-  NSDictionary *result, *nresult, *namespaceDict;
+  NSDictionary *result, *nresult;
   NSMutableDictionary *folders;
   NGImap4Client *client;
+  SOGoUserDefaults *ud;
   NSArray *folderList;
   NSEnumerator *e;
   NSString *guid;
-  id object;
+  id currentFolder;
   
   BOOL hasAnnotatemore;
 
-  folderList = [self allFolderPaths];
+  ud = [[context activeUser] userDefaults];
+
+  // We skip the Junk folder here, as EAS doesn't know about this
+  if ([ud synchronizeOnlyDefaultMailFolders])
+    folderList = [[NSArray arrayWithObjects:
+                             [self inboxFolderNameInContext: context],
+                           [self draftsFolderNameInContext: context],
+                           [self sentFolderNameInContext: context],
+                           [self trashFolderNameInContext: context],
+                     nil] stringsWithFormat: @"/%@"];
+  else
+    folderList = [self allFolderPaths: SOGoMailStandardListing];
 
   folders = [NSMutableDictionary dictionary];
 
   client = [[self imap4Connection] client];
-  namespaceDict = [client namespace];
   hasAnnotatemore = [self hasCapability: @"annotatemore"];
 
   if (hasAnnotatemore)
@@ -689,43 +789,48 @@ static NSString *inboxFolderName = @"INBOX";
     result = [client lstatus: @"*" flags: [NSArray arrayWithObjects: @"x-guid", nil]];
   
   e = [folderList objectEnumerator];
-  
-  while ((object = [e nextObject]))
+
+  while ((currentFolder = [[e nextObject] substringFromIndex: 1]))
     {
       if (hasAnnotatemore)
-        guid = [[[[result objectForKey: @"FolderList"] objectForKey: [object substringFromIndex: 1]] objectForKey: @"/comment"] objectForKey: @"value.priv"];
+        guid = [[[[result objectForKey: @"FolderList"] objectForKey: currentFolder] objectForKey: @"/comment"] objectForKey: @"value.priv"];
       else
-        guid = [[[result objectForKey: @"status"] objectForKey: [object substringFromIndex: 1]] objectForKey: @"x-guid"];
+        guid = [[[result objectForKey: @"status"] objectForKey: currentFolder] objectForKey: @"x-guid"];
       
-      if (!guid)
+      if (!guid || [guid isNull])
         {
           // Don't generate a GUID for "Other users" and "Shared" namespace folders - user foldername instead
-          if ([[object substringFromIndex: 1] isEqualToString: [[[[namespaceDict objectForKey: @"other users"] lastObject] objectForKey: @"prefix"] substringFromIndex: 1]] ||
-              [[object substringFromIndex: 1] isEqualToString: [[[[namespaceDict objectForKey: @"shared"] lastObject] objectForKey: @"prefix"] substringFromIndex: 1]])
+          if ((otherUsersFolderName && [currentFolder isEqualToString: otherUsersFolderName]) ||
+              (sharedFoldersName && [currentFolder isEqualToString: sharedFoldersName]))
+             guid = [NSString stringWithFormat: @"%@", currentFolder];
+          // If Dovecot is used with mail_shared_explicit_inbox = yes we have to generate a guid for "shared/user".
+          // * LIST (\NonExistent \HasChildren) "/" shared
+          // * LIST (\NonExistent \HasChildren) "/" shared/jdoe@example.com
+          // * LIST (\HasNoChildren) "/" shared/jdoe@example.com/INBOX
+          else if (!hasAnnotatemore &&
+		   ([[[result objectForKey: @"list"] objectForKey: currentFolder] indexOfObject: @"nonexistent"] != NSNotFound &&
+		    [[[result objectForKey: @"list"] objectForKey: currentFolder] indexOfObject: @"haschildren"] != NSNotFound))
+            guid = [NSString stringWithFormat: @"%@", currentFolder];
+          else
             {
-              [folders setObject: [NSString stringWithFormat: @"folder%@", [object substringFromIndex: 1]] forKey: [NSString stringWithFormat: @"folder%@", [object substringFromIndex: 1]]];
-              continue;
+              // If folder doesn't exists - ignore it. 
+              nresult = [client status: currentFolder
+                                 flags: [NSArray arrayWithObject: @"UIDVALIDITY"]];
+              if (![[nresult valueForKey: @"result"] boolValue])
+                continue;
+              else if (hasAnnotatemore)
+                 {
+                   guid = [[NSProcessInfo processInfo] globallyUniqueString];
+                   nresult = [client annotation: currentFolder entryName: @"/comment" attributeName: @"value.priv" attributeValue: guid];
+                 }
+
+             // setannotation failed or annotatemore is not available
+             if ((hasAnnotatemore && ![[nresult objectForKey: @"result"] boolValue]) || !hasAnnotatemore)
+               guid = [NSString stringWithFormat: @"%@", currentFolder];
             }
-          
-          // if folder doesn't exists - ignore it
-          nresult = [client status: [object substringFromIndex: 1]
-                             flags: [NSArray arrayWithObject: @"UIDVALIDITY"]];
-          if (![[nresult valueForKey: @"result"] boolValue])
-            continue;
-          
-          if (hasAnnotatemore)
-            {
-              guid = [[NSProcessInfo processInfo] globallyUniqueString];
-              nresult = [client annotation: [object substringFromIndex: 1] entryName: @"/comment" attributeName: @"value.priv" attributeValue: guid];
-            }
-          
-          // setannotation failed or annotatemore is not available
-          if (![[nresult objectForKey: @"result"] boolValue] || !hasAnnotatemore)
-            guid = [NSString stringWithFormat: @"%@", [object substringFromIndex: 1]];
         }
       
-      [folders setObject: [NSString stringWithFormat: @"folder%@", guid] forKey: [NSString stringWithFormat: @"folder%@", [object substringFromIndex: 1]]];
-      
+      [folders setObject: [NSString stringWithFormat: @"folder%@", guid] forKey: [NSString stringWithFormat: @"folder%@", currentFolder]];
     }
   
   return folders;
@@ -734,6 +839,25 @@ static NSString *inboxFolderName = @"INBOX";
 
 /* name lookup */
 
+- (id) lookupNameByPaths: (NSArray *) _paths
+               inContext: (id)_ctx
+                 acquire: (BOOL) _flag
+{
+  NSString *folderName;
+  NSUInteger count, max;
+  SOGoMailBaseObject *folder;
+
+  max = [_paths count];
+  folder = self;
+  for (count = 0; folder && count < max; count++)
+    {
+      folderName = [_paths objectAtIndex: count];
+      folder = [folder lookupName: folderName inContext: _ctx acquire: _flag];
+    }
+
+  return folder;
+}
+
 - (id) lookupName: (NSString *) _key
 	inContext: (id)_ctx
 	  acquire: (BOOL) _flag
@@ -762,6 +886,9 @@ static NSString *inboxFolderName = @"INBOX";
       else if ([folderName
 		 isEqualToString: [self trashFolderNameInContext: _ctx]])
 	klazz = [SOGoTrashFolder class];
+      else if ([folderName
+		 isEqualToString: [self junkFolderNameInContext: _ctx]])
+	klazz = [SOGoJunkFolder class];
       else
 	klazz = [SOGoMailFolder class];
 
@@ -799,12 +926,12 @@ static NSString *inboxFolderName = @"INBOX";
   accounts = [user mailAccounts];
   accountIdx = [nameInContainer intValue];
   account = [accounts objectAtIndex: accountIdx];
-  folderName = [[account objectForKey: @"mailboxes"]
+  folderName = [[account objectForKey: @"specialMailboxes"]
                  objectForKey: purpose];
   if (!folderName && accountIdx > 0)
     {
       account = [accounts objectAtIndex: 0];
-      folderName = [[account objectForKey: @"mailboxes"]
+      folderName = [[account objectForKey: @"specialMailboxes"]
                      objectForKey: purpose];
     }
 
@@ -826,6 +953,11 @@ static NSString *inboxFolderName = @"INBOX";
   return [self _userFolderNameWithPurpose: @"Trash"];
 }
 
+- (NSString *) junkFolderNameInContext: (id)_ctx
+{
+  return [self _userFolderNameWithPurpose: @"Junk"];
+}
+
 - (NSString *) otherUsersFolderNameInContext: (id)_ctx
 {
   return otherUsersFolderName;
@@ -924,6 +1056,19 @@ static NSString *inboxFolderName = @"INBOX";
   return trashFolder;
 }
 
+- (SOGoJunkFolder *) junkFolderInContext: (id) _ctx
+{
+  if (!junkFolder)
+    {
+      junkFolder
+	= [self folderWithTraversal: [self junkFolderNameInContext: _ctx]
+                       andClassName: @"SOGoJunkFolder"];
+      [trashFolder retain];
+    }
+
+  return junkFolder;
+}
+
 /* account delegation */
 - (NSArray *) delegates
 {
@@ -971,7 +1116,7 @@ static NSString *inboxFolderName = @"INBOX";
       max = [newDelegates count];
       for (count = 0; count < max; count++)
         {
-          currentDelegate = [newDelegates objectAtIndex: 0];
+          currentDelegate = [newDelegates objectAtIndex: count];
           delegateUser = [SOGoUser userWithLogin: currentDelegate];
           if (delegateUser)
             {
@@ -998,13 +1143,11 @@ static NSString *inboxFolderName = @"INBOX";
       max = [oldDelegates count];
       for (count = 0; count < max; count++)
         {
-          currentDelegate = [oldDelegates objectAtIndex: 0];
+          currentDelegate = [oldDelegates objectAtIndex: count];
           delegateUser = [SOGoUser userWithLogin: currentDelegate];
+          [delegates removeObject: currentDelegate];
           if (delegateUser)
-            {
-              [delegates removeObject: currentDelegate];
-              [delegateUser removeMailDelegator: owner];
-            }
+            [delegateUser removeMailDelegator: owner];
         }
       
       [self _setDelegates: delegates];
