@@ -20,53 +20,57 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSData.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSURL.h>
 #import <Foundation/NSValue.h>
 #import <Foundation/NSTask.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSURL+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSFileManager+Extensions.h>
+#import <NGExtensions/NGHashMap.h>
+#import <NGExtensions/NSCalendarDate+misc.h>
 
-#import <DOM/DOMElement.h>
-#import <DOM/DOMProtocols.h>
 #import <SaxObjC/XMLNamespaces.h>
 
 #import <EOControl/EOSortOrdering.h>
 
+#import <NGMime/NGMimeBodyPart.h>
+#import <NGMime/NGMimeFileData.h>
+#import <NGMime/NGMimeMultipartBody.h>
+#import <NGMail/NGMimeMessage.h>
+#import <NGMail/NGMimeMessageGenerator.h>
+
 #import <NGImap4/NGImap4Connection.h>
 #import <NGImap4/NGImap4Client.h>
 #import <NGImap4/NSString+Imap4.h>
 
 #import <SOGo/DOMNode+SOGo.h>
 #import <SOGo/NSArray+Utilities.h>
+#import <SOGo/NSCalendarDate+SOGo.h>
+#import <SOGo/SOGoDAVAuthenticator.h>
 #import <SOGo/NSDictionary+Utilities.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/NSString+DAV.h>
 #import <SOGo/NSArray+DAV.h>
 #import <SOGo/NSObject+DAV.h>
-#import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoPermissions.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUser.h>
+#import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUserSettings.h>
 #import <SOGo/WORequest+SOGo.h>
 #import <SOGo/WOResponse+SOGo.h>
+#import <SOGo/SOGoMailer.h>
 
 #import "EOQualifier+MailDAV.h"
-#import "SOGoMailObject.h"
 #import "SOGoMailAccount.h"
-#import "SOGoMailManager.h"
-#import "SOGoMailFolder.h"
+#import "SOGoMailAccounts.h"
 #import "SOGoTrashFolder.h"
+#import "SOGoMailObject+Draft.h"
+
 
 #define XMLNS_INVERSEDAV @"urn:inverse:params:xml:ns:inverse-dav"
 
@@ -197,9 +201,9 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 {
   NSArray *subfolders;
 
-  subfolders = [[self subfolders] stringsWithFormat: @"folder%@"];
+  subfolders = [[self subfolders] resultsOfSelector: @selector (asCSSIdentifier)];
 
-  return [subfolders resultsOfSelector: @selector (asCSSIdentifier)];
+  return [subfolders stringsWithFormat: @"folder%@"];
 }
 
 - (NSArray *) subfolders
@@ -223,7 +227,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
   prefix = [self absoluteImap4Name];
 
-  result = [[self mailAccountFolder] allFolderPaths];
+  result = [[self mailAccountFolder] allFolderPaths: SOGoMailStandardListing];
   folderNames = [result objectEnumerator];
   while ((currentFolderName = [folderNames nextObject]))
     if ([currentFolderName hasPrefix: prefix])
@@ -524,22 +528,23 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   if (!archiveName)
     archiveName = @"SavedMessages.zip";
 
-#warning this method should be rewritten according to our coding styles  
   spoolPath = [self userSpoolFolderPath];
-  if (![self ensureSpoolFolderPath]) {
-    [self errorWithFormat: @"spool directory '%@' doesn't exist", spoolPath];
-    error = [NSException exceptionWithHTTPStatus: 500 
-                                          reason: @"spool directory does not exist"];
-    return (WOResponse *)error;
+  if (![self ensureSpoolFolderPath])
+    {
+      [self errorWithFormat: @"spool directory '%@' doesn't exist", spoolPath];
+      error = [NSException exceptionWithHTTPStatus: 500
+                                            reason: @"spool directory does not exist"];
+      return (WOResponse *)error;
   }
 
   zipPath = [[SOGoSystemDefaults sharedSystemDefaults] zipPath];
   fm = [NSFileManager defaultManager];
-  if (![fm fileExistsAtPath: zipPath]) {
-    error = [NSException exceptionWithHTTPStatus: 500 
-                                          reason: @"zip not available"];
-    return (WOResponse *)error;
-  }
+  if (![fm fileExistsAtPath: zipPath])
+    {
+      error = [NSException exceptionWithHTTPStatus: 500
+                                            reason: @"zip not available"];
+      return (WOResponse *)error;
+    }
   
   zipTask = [[NSTask alloc] init];
   [zipTask setCurrentDirectoryPath: spoolPath];
@@ -549,17 +554,18 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   [zipTaskArguments addObject: @"SavedMessages.zip"];
 
   msgs = (NSDictionary *)[self fetchUIDs: uids  
-                                   parts: [NSArray arrayWithObject: @"RFC822"]];
+                                   parts: [NSArray arrayWithObject: @"BODY.PEEK[]"]];
   messages = [msgs objectForKey: @"fetch"];
 
-  for (i = 0; i < [messages count]; i++) {
-    content = [[messages objectAtIndex: i] objectForKey: @"message"];
-    fileName = [NSString stringWithFormat:@"%@/%@.eml", spoolPath, [uids objectAtIndex: i]];;
-    [content writeToFile: fileName atomically: YES];
+  for (i = 0; i < [messages count]; i++)
+    {
+      content = [[[messages objectAtIndex: i] objectForKey: @"body[]"] objectForKey: @"data"];
+      fileName = [NSString stringWithFormat:@"%@/%@.eml", spoolPath, [uids objectAtIndex: i]];;
+      [content writeToFile: fileName atomically: YES];
     
-    [zipTaskArguments addObject: 
-      [NSString stringWithFormat:@"%@.eml", [uids objectAtIndex: i]]];
-  }
+      [zipTaskArguments addObject:
+                          [NSString stringWithFormat:@"%@.eml", [uids objectAtIndex: i]]];
+    }
   
   [zipTask setArguments: zipTaskArguments];
   [zipTask launch];
@@ -568,16 +574,16 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   [zipTask release];
   
   zipContent = [[NSData alloc] initWithContentsOfFile: 
-    [NSString stringWithFormat: @"%@/SavedMessages.zip", spoolPath]];
+                           [NSString stringWithFormat: @"%@/SavedMessages.zip", spoolPath]];
   
-  for(i = 0; i < [zipTaskArguments count]; i++) {
-    fileName = [zipTaskArguments objectAtIndex: i];
-    [fm removeFileAtPath: 
-      [NSString stringWithFormat: @"%@/%@", spoolPath, fileName] handler: nil];
-  }
-  
-  response = [context response];
+  for (i = 0; i < [zipTaskArguments count]; i++)
+    {
+      fileName = [zipTaskArguments objectAtIndex: i];
+      [fm removeFileAtPath:
+            [NSString stringWithFormat: @"%@/%@", spoolPath, fileName] handler: nil];
+    }
 
+  response = [context response];
   baseName = [archiveName stringByDeletingPathExtension];
   extension = [archiveName pathExtension];
   if ([extension length] > 0)
@@ -591,14 +597,14 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   [response setHeader: [NSString stringWithFormat: @"application/zip;"
                                  @" name=\"%@\"",
                                  qpFileName]
-               forKey:@"content-type"];
+               forKey: @"content-type"];
   [response setHeader: [NSString stringWithFormat: @"attachment; filename=\"%@\"",
                                  qpFileName]
                forKey: @"Content-Disposition"];
   [response setContent: zipContent];
 
   [zipContent release];
-  
+
   return response;
 }
 
@@ -632,7 +638,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 		inContext: (id) localContext
 {
   NSArray *folders;
-  NSString *currentFolderName, *currentAccountName;
+  NSString *currentFolderName, *currentAccountName, *destinationAccountName;
   NSMutableString *imapDestinationFolder;
   NGImap4Client *client;
   id result;
@@ -640,24 +646,24 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
 #warning this code will fail on implementation using something else than '/' as delimiter
   imapDestinationFolder = [NSMutableString string];
-  folders = [[destinationFolder componentsSeparatedByString: @"/"]
-              resultsOfSelector: @selector (fromCSSIdentifier)];
+  folders = [destinationFolder componentsSeparatedByString: @"/"];
   max = [folders count];
   if (max > 1)
     {
       currentAccountName = [[self mailAccountFolder] nameInContainer];
       client = [[self imap4Connection] client];
       [imap4 selectFolder: [self imap4URL]];
+      destinationAccountName = [[folders objectAtIndex: 1] fromCSSIdentifier];
 
       for (count = 2; count < max; count++)
         {
-          currentFolderName = [[folders objectAtIndex: count] substringFromIndex: 6];
+          currentFolderName = [[[folders objectAtIndex: count] substringFromIndex: 6] fromCSSIdentifier];
           [imapDestinationFolder appendFormat: @"/%@", currentFolderName];
         }
 
       if (client)
         {
-          if ([[folders objectAtIndex: 1] isEqualToString: currentAccountName])
+          if ([destinationAccountName isEqualToString: currentAccountName])
             {
               // We make sure the destination IMAP folder exist, if not, we create it.
               result = [[client status: imapDestinationFolder
@@ -686,7 +692,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
               
               userFolder = [[context activeUser] homeFolderInContext: context];
               accounts = [userFolder lookupName: @"Mail"  inContext: context  acquire: NO];
-              account = [accounts lookupName: [folders objectAtIndex: 1] inContext: localContext acquire: NO];
+              account = [accounts lookupName: destinationAccountName inContext: localContext acquire: NO];
 
               if ([account isKindOfClass: [NSException class]])
                 {
@@ -783,6 +789,117 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   return result;
 }
 
+- (WOResponse *) markMessagesAsJunkOrNotJunk: (NSArray *) uids
+                                        junk: (BOOL) isJunk
+{
+  NSDictionary *junkSettings;
+  NSString *recipient;
+  NSException *error;
+  unsigned int limit;
+
+  junkSettings = [[[context activeUser] domainDefaults] mailJunkSettings];
+  error = nil;
+
+  if ([[junkSettings objectForKey: @"vendor"] caseInsensitiveCompare: @"generic"] == NSOrderedSame)
+    {
+      if (isJunk)
+        recipient = [junkSettings objectForKey: @"junkEmailAddress"];
+      else
+        recipient = [junkSettings objectForKey: @"notJunkEmailAddress"];
+
+      limit = [[junkSettings objectForKey: @"limit"] intValue];
+
+      // If no limit is set, we only attach one mail at the time
+      // to reports sent by SOGo.
+      if (!limit)
+        limit = 1;
+
+      if ([recipient length])
+        {
+          NGMimeMessage *messageToSend;
+          SOGoMailObject *mailObject;
+          NGMimeBodyPart *bodyPart;
+          NGMutableHashMap *map;
+          NGMimeFileData *fdata;
+          NSArray *identities;
+          NSData *data;
+          id body;
+
+          int i;
+
+          identities = [[self mailAccountFolder] identities];
+
+          for (i = 0; i < [uids count]; i++)
+            {
+              // If we are starting or reaching the limit, we create
+              // a new mail message
+              if ((i%limit) == 0)
+                {
+                  map = [NGMutableHashMap hashMapWithCapacity: 5];
+
+#warning SOPE is just plain stupid here - if you change the case of keys, it will break the encoding of fields
+                  [map setObject: @"multipart/mixed" forKey: @"content-type"];
+                  [map setObject: @"1.0" forKey: @"MIME-Version"];
+                  [map setObject: [[identities objectAtIndex: 0] objectForKey: @"email"] forKey: @"from"];
+                  [map setObject: recipient forKey: @"to"];
+                  [map setObject: [[NSCalendarDate date] rfc822DateString]  forKey: @"date"];
+
+                  messageToSend = [[[NGMimeMessage alloc] initWithHeader: map] autorelease];
+                  body = [[[NGMimeMultipartBody alloc] initWithPart: messageToSend] autorelease];
+                }
+
+              mailObject = [self lookupName: [NSString stringWithFormat: @"%@", [uids objectAtIndex: i]]
+                                  inContext: context
+                                    acquire: NO];
+
+              // We skip emails that might have disappeared before we were able
+              // to perform the action
+              if ([mailObject isKindOfClass: [NSException class]])
+                continue;
+
+              map = [[[NGMutableHashMap alloc] initWithCapacity: 1] autorelease];
+              [map setObject: @"message/rfc822" forKey: @"content-type"];
+              [map setObject: @"8bit" forKey: @"content-transfer-encoding"];
+              [map addObject: [NSString stringWithFormat: @"attachment; filename=\"%@\"", [mailObject filenameForForward]] forKey: @"content-disposition"];
+              bodyPart = [[[NGMimeBodyPart alloc] initWithHeader: map] autorelease];
+
+              data = [mailObject content];
+              fdata = [[NGMimeFileData alloc] initWithBytes: [data bytes]  length: [data length]];
+
+              [bodyPart setBody: fdata];
+              RELEASE(fdata);
+              [body addBodyPart: bodyPart];
+
+              // We reached the limit or the end of the array
+              if ((i%limit) == 0 || i == [uids count]-1)
+                {
+                  id <SOGoAuthenticator> authenticator;
+                  NGMimeMessageGenerator *generator;
+
+                  [messageToSend setBody: body];
+
+                  generator = [[[NGMimeMessageGenerator alloc] init] autorelease];
+                  data = [generator generateMimeFromPart: messageToSend];
+
+                  authenticator = [SOGoDAVAuthenticator sharedSOGoDAVAuthenticator];
+
+                  error = [[SOGoMailer mailerWithDomainDefaults: [[context activeUser] domainDefaults]]
+                            sendMailData: data
+                            toRecipients: [NSArray arrayWithObject: recipient]
+                                  sender: [[identities objectAtIndex: 0] objectForKey: @"email"]
+                            withAuthenticator: authenticator
+                               inContext: context];
+
+                  if (error)
+                    break;
+                }
+            }
+        }
+    }
+
+  return (WOResponse *) error;
+}
+
 - (NSDictionary *) statusForFlags: (NSArray *) flags
 {
   NGImap4Client *client;
@@ -801,6 +918,33 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   return status;
 }
 
+- (unsigned int) unseenCount
+{
+  NSDictionary *imapResult;
+  NGImap4Connection *connection;
+  NGImap4Client *client;
+  EOQualifier *searchQualifier;
+  NSArray *searchResult;
+  unsigned int unseen;
+
+  connection = [self imap4Connection];
+  client = [connection client];
+
+  if ([connection selectFolder: [self imap4URL]])
+    {
+      searchQualifier
+        = [EOQualifier qualifierWithQualifierFormat: @"flags = %@ AND not flags = %@",
+                       @"unseen", @"deleted"];
+      imapResult = [client searchWithQualifier: searchQualifier];
+      searchResult = [[imapResult objectForKey: @"RawResponse"] objectForKey: @"search"];
+      unseen = [searchResult count];
+    }
+  else
+    unseen = 0;
+
+  return unseen;
+}
+
 - (NSArray *) fetchUIDsMatchingQualifier: (id) _q
 			    sortOrdering: (id) _so
 {
@@ -1393,11 +1537,12 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   path = [[self imap4Connection] imap4FolderNameForURL: [self imap4URL]];
 
   if ([self         _path: path
-           isInNamespaces: [mailAccount otherUsersFolderNamespaces]]
-      || [self         _path: path
-              isInNamespaces: [mailAccount sharedFolderNamespaces]])
+           isInNamespaces: [mailAccount otherUsersFolderNamespaces]] ||
+      [self         _path: path
+           isInNamespaces: [mailAccount sharedFolderNamespaces]])
     [acls addObject: SOGoRole_ObjectViewer];
   else
+    // Inside user's namespace, automatically owner
     [acls addObject: SoRole_Owner];
 
   return acls;
@@ -1410,23 +1555,22 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
   userLogin = [[context activeUser] login];
   if ([uid isEqualToString: userLogin])
+    // Login user wants her ACLs
     acls = [self _sharesACLs];
   else
+    // Login user wants the ACLs of another user
     acls = [NSMutableArray array];
 
-  if ([owner isEqualToString: userLogin])
-    {
-      if (!mailboxACL)
-        [self _readMailboxACL];
+  if (!mailboxACL)
+    [self _readMailboxACL];
 
-      if ([mailboxACL isKindOfClass: [NSDictionary class]])
-        {
-          userAcls = [mailboxACL objectForKey: uid];
-          if (!([userAcls length] || [uid isEqualToString: defaultUserID]))
-            userAcls = [mailboxACL objectForKey: defaultUserID];
-          if ([userAcls length])
-            [acls addObjectsFromArray: [self _imapAclsToSOGoAcls: userAcls]];
-        }
+  if ([mailboxACL isKindOfClass: [NSDictionary class]])
+    {
+      userAcls = [mailboxACL objectForKey: uid];
+      if (!([userAcls length] || [uid isEqualToString: defaultUserID]))
+        userAcls = [mailboxACL objectForKey: defaultUserID];
+      if ([userAcls length])
+        [acls addObjectsFromArray: [self _imapAclsToSOGoAcls: userAcls]];
     }
 
   return acls;
@@ -1553,7 +1697,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
 - (NSString *) displayName
 {
-  return [self relativeImap4Name];
+  return [[self relativeImap4Name] stringByDecodingImap4FolderName];
 }
 
 - (NSDictionary *) davIMAPFieldsTable
@@ -1645,7 +1789,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
   sortOrderings = [NSMutableArray array];
 
-  if ([self _sortElementIsAscending: sortElement])
+  if ([self _sortElementIsAscending: (NGDOMNodeWithChildren <DOMElement> *)sortElement])
     sortOrderingOrder = EOCompareAscending;
   else
     sortOrderingOrder = EOCompareDescending;
@@ -1931,6 +2075,11 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   return response;
 }
 
+- (NSString *) className
+{
+  return NSStringFromClass([self class]);
+}
+
 - (id) PUTAction: (WOContext *) _ctx
 {
   WORequest *rq;
@@ -2077,7 +2226,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 // * 5 FETCH (UID 559 MODSEQ (17))
 // * 6 FETCH (UID 560 MODSEQ (18))
 // * 7 FETCH (UID 561 MODSEQ (19))
-
+//
 //
 // fetchUIDsOfVanishedItems ..
 //
@@ -2087,9 +2236,10 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 // 
 // FIXME: refactor MAPIStoreMailFolder.m - synchroniseCache to use this method
 //
-- (NSArray *) syncTokenFieldsWithProperties: (NSArray *) theProperties
+- (NSArray *) syncTokenFieldsWithProperties: (NSDictionary *) theProperties
                           matchingSyncToken: (NSString *) theSyncToken
                                    fromDate: (NSCalendarDate *) theStartDate
+                                initialLoad: (BOOL) initialLoadInProgress
 {
   EOQualifier *searchQualifier;
   NSMutableArray *allTokens;
@@ -2097,18 +2247,13 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   NSDictionary *d;
   id fetchResults;
 
-  int uidnext, highestmodseq, i; 
+  int highestmodseq = 0, i;
 
   allTokens = [NSMutableArray array];
 
-  if ([theSyncToken isEqualToString: @"-1"])
-    {
-      uidnext = highestmodseq = 0;
-    }
-  else
+  if (![theSyncToken isEqualToString: @"-1"])
     {
       a = [theSyncToken componentsSeparatedByString: @"-"];
-      uidnext = [[a objectAtIndex: 0] intValue];
       highestmodseq = [[a objectAtIndex: 1] intValue];
     }
   
@@ -2166,6 +2311,9 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
 
   for (i = 0; i < [fetchResults count]; i++)
     { 
+      if ([[[fetchResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"] && initialLoadInProgress)
+        continue;
+
       d = [NSDictionary dictionaryWithObject: ([[[fetchResults objectAtIndex: i] objectForKey: @"flags"] containsObject: @"deleted"]) ? [NSNull null] : [[fetchResults objectAtIndex: i] objectForKey: @"modseq"]
                                       forKey: [[[fetchResults objectAtIndex: i] objectForKey: @"uid"] stringValue]];
       [allTokens addObject: d];
@@ -2176,7 +2324,7 @@ _compareFetchResultsByMODSEQ (id entry1, id entry2, void *data)
   if (highestmodseq == 0)
     highestmodseq = 1;
 
-  if (highestmodseq > 0)
+  if (highestmodseq > 0 && !initialLoadInProgress)
     {
       id uid;
 
