@@ -1,6 +1,6 @@
 /* SOGoMailForward.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,6 +49,12 @@
 @interface SOGoMailCatalanForward : SOGoMailForward
 @end
 
+@interface SOGoMailChineseTaiwanForward : SOGoMailForward
+@end
+
+@interface SOGoMailCroatianForward : SOGoMailForward
+@end
+
 @interface SOGoMailCzechForward : SOGoMailForward
 @end
 
@@ -79,12 +85,36 @@
 @interface SOGoMailItalianForward : SOGoMailForward
 @end
 
+@interface SOGoMailLithuanianForward : SOGoMailForward
+@end
+
+@interface SOGoMailMacedonianForward : SOGoMailForward
+@end
+
 @interface SOGoMailNorwegianBokmalForward : SOGoMailForward
 @end
 
 @interface SOGoMailNorwegianNynorskForward : SOGoMailForward
 @end
 
+@interface SOGoMailPolishForward : SOGoMailForward
+@end
+
+@interface SOGoMailPortugueseForward : SOGoMailForward
+@end
+
+@interface SOGoMailRussianForward : SOGoMailForward
+@end
+
+@interface SOGoMailSerbianForward : SOGoMailForward
+@end
+
+@interface SOGoMailSlovakForward : SOGoMailForward
+@end
+
+@interface SOGoMailSlovenianForward : SOGoMailForward
+@end
+
 @interface SOGoMailSpanishSpainForward : SOGoMailForward
 @end
 
@@ -94,13 +124,7 @@
 @interface SOGoMailSwedishForward : SOGoMailForward
 @end
 
-@interface SOGoMailPolishForward : SOGoMailForward
-@end
-
-@interface SOGoMailRussianForward : SOGoMailForward
-@end
-
-@interface SOGoMailSlovakForward : SOGoMailForward
+@interface SOGoMailTurkishTurkeyForward : SOGoMailForward
 @end
 
 @interface SOGoMailUkrainianForward : SOGoMailForward
