@@ -1,6 +1,6 @@
 /* SOGoMailForward.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -73,10 +73,10 @@
 
 - (NSString *) newLine
 {
-  NSString *rc = [NSString stringWithString: @" "];
+  NSString *rc = @" ";
   
   if (htmlComposition)
-    rc = [NSString stringWithString: @"<br/>"];
+    rc = @"<br/>";
 
   return rc;
 }
@@ -260,6 +260,12 @@
 @implementation SOGoMailCatalanForward
 @end
 
+@implementation SOGoMailChineseTaiwanForward
+@end
+
+@implementation SOGoMailCroatianForward
+@end
+
 @implementation SOGoMailCzechForward
 @end
 
@@ -290,12 +296,36 @@
 @implementation SOGoMailItalianForward
 @end
 
+@implementation SOGoMailLithuanianForward
+@end
+
+@implementation SOGoMailMacedonianForward
+@end
+
 @implementation SOGoMailNorwegianBokmalForward
 @end
 
 @implementation SOGoMailNorwegianNynorskForward
 @end
 
+@implementation SOGoMailPolishForward
+@end
+
+@implementation SOGoMailPortugueseForward
+@end
+
+@implementation SOGoMailRussianForward
+@end
+
+@implementation SOGoMailSerbianForward
+@end
+
+@implementation SOGoMailSlovakForward
+@end
+
+@implementation SOGoMailSlovenianForward
+@end
+
 @implementation SOGoMailSpanishSpainForward
 @end
 
@@ -305,13 +335,7 @@
 @implementation SOGoMailSwedishForward
 @end
 
-@implementation SOGoMailPolishForward
-@end
-
-@implementation SOGoMailRussianForward
-@end
-
-@implementation SOGoMailSlovakForward
+@implementation SOGoMailTurkishTurkeyForward
 @end
 
 @implementation SOGoMailUkrainianForward
