@@ -1,6 +1,6 @@
 /* SOGoMailReply.h - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -48,6 +48,12 @@
 @interface SOGoMailCatalanReply : SOGoMailReply
 @end
 
+@interface SOGoMailChineseTaiwanReply : SOGoMailReply
+@end
+
+@interface SOGoMailCroatianReply : SOGoMailReply
+@end
+
 @interface SOGoMailCzechReply : SOGoMailReply
 @end
 
@@ -78,12 +84,36 @@
 @interface SOGoMailItalianReply : SOGoMailReply
 @end
 
+@interface SOGoMailLithuanianReply : SOGoMailReply
+@end
+
+@interface SOGoMailMacedonianReply : SOGoMailReply
+@end
+
 @interface SOGoMailNorwegianBokmalReply : SOGoMailReply
 @end
 
 @interface SOGoMailNorwegianNynorskReply : SOGoMailReply
 @end
 
+@interface SOGoMailPolishReply : SOGoMailReply
+@end
+
+@interface SOGoMailPortugueseReply : SOGoMailReply
+@end
+
+@interface SOGoMailRussianReply : SOGoMailReply
+@end
+
+@interface SOGoMailSerbianReply : SOGoMailReply
+@end
+
+@interface SOGoMailSlovakReply : SOGoMailReply
+@end
+
+@interface SOGoMailSlovenianReply : SOGoMailReply
+@end
+
 @interface SOGoMailSpanishSpainReply : SOGoMailReply
 @end
 
@@ -93,13 +123,7 @@
 @interface SOGoMailSwedishReply : SOGoMailReply
 @end
 
-@interface SOGoMailPolishReply : SOGoMailReply
-@end
-
-@interface SOGoMailRussianReply : SOGoMailReply
-@end
-
-@interface SOGoMailSlovakReply : SOGoMailReply
+@interface SOGoMailTurkishTurkeyReply : SOGoMailReply
 @end
 
 @interface SOGoMailUkrainianReply : SOGoMailReply
