@@ -1,6 +1,6 @@
 /* SOGoMailReply.m - this file is part of SOGo
  *
- * Copyright (C) 2007-2013 Inverse inc.
+ * Copyright (C) 2007-2015 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,8 +22,6 @@
 #import <NGExtensions/NSString+misc.h>
 #import <NGImap4/NGImap4Envelope.h>
 
-#import <SoObjects/SOGo/SOGoDateFormatter.h>
-#import <SoObjects/SOGo/SOGoUser.h>
 
 #import "SOGoMailObject+Draft.h"
 #import "SOGoMailReply.h"
@@ -98,6 +96,12 @@
 @implementation SOGoMailCatalanReply
 @end
 
+@implementation SOGoMailChineseTaiwanReply
+@end
+
+@implementation SOGoMailCroatianReply
+@end
+
 @implementation SOGoMailCzechReply
 @end
 
@@ -128,12 +132,36 @@
 @implementation SOGoMailItalianReply
 @end
 
+@implementation SOGoMailLithuanianReply
+@end
+
+@implementation SOGoMailMacedonianReply
+@end
+
 @implementation SOGoMailNorwegianBokmalReply
 @end
 
 @implementation SOGoMailNorwegianNynorskReply
 @end
 
+@implementation SOGoMailPolishReply
+@end
+
+@implementation SOGoMailPortugueseReply
+@end
+
+@implementation SOGoMailRussianReply
+@end
+
+@implementation SOGoMailSerbianReply
+@end
+
+@implementation SOGoMailSlovakReply
+@end
+
+@implementation SOGoMailSlovenianReply
+@end
+
 @implementation SOGoMailSpanishSpainReply
 @end
 
@@ -143,13 +171,7 @@
 @implementation SOGoMailSwedishReply
 @end
 
-@implementation SOGoMailPolishReply
-@end
-
-@implementation SOGoMailRussianReply
-@end
-
-@implementation SOGoMailSlovakReply
+@implementation SOGoMailTurkishTurkeyReply
 @end
 
 @implementation SOGoMailUkrainianReply
