@@ -21,10 +21,8 @@
 #ifndef LDAPSOURCE_H
 #define LDAPSOURCE_H
 
-#import <Foundation/NSObject.h>
 
 #include "SOGoSource.h"
-#include "SOGoConstants.h"
 
 @class LDAPSourceSchema;
 @class NGLdapEntry;
@@ -120,8 +118,12 @@ andMultipleBookingsField: (NSString *) newMultipleBookingsField;
 - (void) setContactMapping: (NSDictionary *) newMapping
           andObjectClasses: (NSArray *) newObjectClasses;
 
-- (NGLdapEntry *) lookupGroupEntryByUID: (NSString *) theUID;
-- (NGLdapEntry *) lookupGroupEntryByEmail: (NSString *) theEmail;
+- (NGLdapEntry *) lookupGroupEntryByUID: (NSString *) theUID
+                               inDomain: (NSString *) domain;
+- (NGLdapEntry *) lookupGroupEntryByEmail: (NSString *) theEmail
+                                 inDomain: (NSString *) domain;
+
+- (void) applyContactMappingToResult: (NSMutableDictionary *) ldifRecord;
 
 @end
 
