@@ -18,12 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <ldap.h>
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSString.h>
 
 #import <NGExtensions/NSObject+Logs.h>
 #import <EOControl/EOControl.h>
@@ -38,11 +33,9 @@
 #import "NSString+Utilities.h"
 #import "NSString+Crypto.h"
 #import "SOGoCache.h"
-#import "SOGoDomainDefaults.h"
 #import "SOGoSystemDefaults.h"
 
 #import "LDAPSource.h"
-#import "../../Main/SOGo.h"
 
 static Class NSStringK;
 
@@ -162,6 +155,7 @@ static Class NSStringK;
   [multipleBookingsField release];
   [MSExchangeHostname release];
   [modifiers release];
+  [displayName release];
   [super dealloc];
 }
 
@@ -737,7 +731,15 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
       }
   NS_HANDLER
     {
-      [self logWithFormat: @"%@", localException];
+      if ([[localException name] isEqual: @"LDAPException"] &&
+          ([[[localException userInfo] objectForKey: @"error_code"] intValue] == LDAP_CONSTRAINT_VIOLATION))
+        {
+          *perr = PolicyInsufficientPasswordQuality;
+        }
+      else
+        {
+          [self logWithFormat: @"%@", localException];
+        }
     }
   NS_ENDHANDLER ;
   
@@ -904,7 +906,32 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     {
       allValues = [[ldapEntry attributeWithName: currentFieldName]
                     allStringValues];
-      [emails addObjectsFromArray: allValues];
+
+      // Special case handling for Microsoft Active Directory. proxyAddresses
+      // is generally prefixed with smtp: - if we find this (or any value preceeding
+      // the semi-colon), we strip it. See https://msdn.microsoft.com/en-us/library/ms679424(v=vs.85).aspx
+      if ([currentFieldName caseInsensitiveCompare: @"proxyAddresses"] == NSOrderedSame)
+	{
+	  NSRange r;
+	  int i;
+
+	  for (i = 0; i < [allValues count]; i++)
+	    {
+	      ldapValue = [allValues objectAtIndex: i];
+	      r = [ldapValue rangeOfString: @":"];
+
+	      if (r.length)
+		{
+		  // We only keep "smtp" ones
+		  if ([[ldapValue lowercaseString] hasPrefix: @"smtp"])
+		    [emails addObject: [ldapValue substringFromIndex: r.location+1]];
+		}
+	      else
+		[emails addObject: ldapValue];
+	    }
+	}
+      else
+	[emails addObjectsFromArray: allValues];
     }
   [ldifRecord setObject: emails forKey: @"c_emails"];
   [emails release];
@@ -963,7 +990,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 }
 
 /* conversion LDAP -> SOGo inetOrgPerson entry */
-- (void) _applyContactMappingToResult: (NSMutableDictionary *) ldifRecord
+- (void) applyContactMappingToResult: (NSMutableDictionary *) ldifRecord
 {
   NSArray *sourceFields;
   NSArray *keys;
@@ -997,7 +1024,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 }
 
 /* conversion SOGo inetOrgPerson entry -> LDAP */
-- (void) _applyContactMappingToOutput: (NSMutableDictionary *) ldifRecord
+- (void) applyContactMappingToOutput: (NSMutableDictionary *) ldifRecord
 {
   NSArray *sourceFields;
   NSArray *keys;
@@ -1081,7 +1108,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
         // We check if our entry is a group. If so, we set the
         // 'isGroup' custom attribute.
         gclasses = [groupObjectClasses objectEnumerator];
-        while (gclass = [gclasses nextObject])
+        while ((gclass = [gclasses nextObject]))
          if ([classes containsObject: [gclass lowercaseString]])
            {
              [ldifRecord setObject: [NSNumber numberWithInt: 1]
@@ -1159,9 +1186,11 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
           intoLDIFRecord: (NSMutableDictionary *) ldifRecord];
   [self _fillConstraints: ldapEntry forModule: @"Mail"
           intoLDIFRecord: (NSMutableDictionary *) ldifRecord];
+  [self _fillConstraints: ldapEntry forModule: @"ActiveSync"
+          intoLDIFRecord: (NSMutableDictionary *) ldifRecord];
 
   if (contactMapping)
-    [self _applyContactMappingToResult: ldifRecord];
+    [self applyContactMappingToResult: ldifRecord];
 
   return ldifRecord;
 }
@@ -1182,7 +1211,6 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
     {
       ldapConnection = [self _ldapConnection];
       qualifier = [self _qualifierForFilter: match];
-      // attributes = [self _searchAttributes];
       attributes = [NSArray arrayWithObject: @"*"];
 
       if ([_scope caseInsensitiveCompare: @"BASE"] == NSOrderedSame)
@@ -1208,10 +1236,9 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 - (NGLdapEntry *) _lookupLDAPEntry: (EOQualifier *) qualifier
 {
   NGLdapConnection *ldapConnection;
-  NSArray *attributes;
   NSEnumerator *entries;
+  NSArray *attributes;
 
-  // attributes = [self _searchAttributes];
   ldapConnection = [self _ldapConnection];
   attributes = [NSArray arrayWithObject: @"*"];
 
@@ -1232,6 +1259,7 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 }
 
 - (NSDictionary *) lookupContactEntry: (NSString *) theID
+                             inDomain: (NSString *) domain
 {
   NGLdapEntry *ldapEntry;
   EOQualifier *qualifier;
@@ -1277,12 +1305,19 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 {
   NGLdapConnection *ldapConnection;
   NGLdapEntry *entry;
+  EOQualifier *qualifier;
   NSString *login;
   
   login = nil;
+  qualifier = nil;
 
   ldapConnection = [self _ldapConnection];
+
+  if (_filter)
+    qualifier = [EOQualifier qualifierWithQualifierFormat: _filter];
+
   entry = [ldapConnection entryAtDN: theDN
+			  qualifier: qualifier
                          attributes: [NSArray arrayWithObject: UIDField]];
   if (entry)
     login = [[entry attributeWithName: UIDField] stringValueAtIndex: 0];
@@ -1329,12 +1364,14 @@ groupObjectClasses: (NSArray *) newGroupObjectClasses
 }
 
 - (NGLdapEntry *) lookupGroupEntryByUID: (NSString *) theUID
+                               inDomain: (NSString *) domain
 {
   return [self _lookupGroupEntryByAttributes: [NSArray arrayWithObject: UIDField]
 				    andValue: theUID];
 }
 
 - (NGLdapEntry *) lookupGroupEntryByEmail: (NSString *) theEmail
+                                 inDomain: (NSString *) domain
 {
   return [self _lookupGroupEntryByAttributes: mailFields
 				    andValue: theEmail];
@@ -1482,7 +1519,7 @@ _convertRecordToLDAPAttributes (LDAPSourceSchema *schema, NSDictionary *ldifReco
           [ldifRecord setObject: aId forKey: @"cn"];
         }
 
-      [self _applyContactMappingToOutput: ldifRecord];
+      [self applyContactMappingToOutput: ldifRecord];
 
       /* since the id might have changed due to the mapping above, we
          reload the record ID */
@@ -1590,7 +1627,7 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
       ldapConnection = [self _ldapConnection];
       ldifRecord = [roLdifRecord mutableCopy];
       [ldifRecord autorelease];
-      [self _applyContactMappingToOutput: ldifRecord];
+      [self applyContactMappingToOutput: ldifRecord];
       attributes = _convertRecordToLDAPAttributes (schema, ldifRecord);
 
       changes = _makeLDAPChanges (ldapConnection, dn, attributes);
@@ -1688,25 +1725,25 @@ _makeLDAPChanges (NGLdapConnection *ldapConnection,
               ab = [LDAPSource new];
               [ab setSourceID: [sourceRec objectForKey: @"ou"]];
               [ab setDisplayName: [sourceRec objectForKey: @"description"]];
-              [ab          setBindDN: bindDN
-                            password: password
-                            hostname: hostname
-                                port: [NSString stringWithFormat: @"%d", port]
-                          encryption: encryption
-                   bindAsCurrentUser: NO];
-              [ab                     setBaseDN: [entry dn]
-                                        IDField: @"cn"
-                                        CNField: @"displayName"
-                                       UIDField: @"cn"
-                                     mailFields: nil
-                                   searchFields: nil
-                             groupObjectClasses: nil
-                                  IMAPHostField: nil
-                                 IMAPLoginField: nil
-                                 SieveHostField: nil
-                                     bindFields: nil
-                                      kindField: nil
-                       andMultipleBookingsField: nil];
+              [ab setBindDN: bindDN
+		   password: password
+		   hostname: hostname
+		       port: [NSString stringWithFormat: @"%d", port]
+		 encryption: encryption
+		  bindAsCurrentUser: [NSString stringWithFormat: @"%d", NO]];
+              [ab setBaseDN: [entry dn]
+		    IDField: @"cn"
+		    CNField: @"displayName"
+		   UIDField: @"cn"
+		 mailFields: nil
+		  searchFields: nil
+		  groupObjectClasses: nil
+		  IMAPHostField: nil
+		  IMAPLoginField: nil
+		  SieveHostField: nil
+		 bindFields: nil
+		  kindField: nil
+		  andMultipleBookingsField: nil];
               [ab setListRequiresDot: NO];
               [ab setModifiers: modifier];
               [sources addObject: ab];
