@@ -44,23 +44,13 @@
  */
 
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSData.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSLock.h>
-#import <Foundation/NSValue.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSTimer.h>
 
-#import <NGObjWeb/SoObject.h>
 #import <NGExtensions/NSObject+Logs.h>
 
 #import "NSDictionary+Utilities.h"
 #import "NSString+Utilities.h"
 #import "SOGoObject.h"
 #import "SOGoSystemDefaults.h"
-#import "SOGoUser.h"
-#import "SOGoUserProfile.h"
 
 #import "SOGoCache.h"
 
@@ -515,6 +505,8 @@ static memcached_st *handle = NULL;
         {
           [d setObject: [NSNumber numberWithUnsignedInt: [[NSCalendarDate date] timeIntervalSince1970]] forKey: @"InitialDate"];
         }
+
+      [d setObject: [NSNumber numberWithUnsignedInt: [[NSCalendarDate date] timeIntervalSince1970]] forKey: @"LastRequestDate"];
       
       [d setObject: count  forKey: @"FailedCount"];
       [self _cacheValues: [d jsonRepresentation]
@@ -737,5 +729,58 @@ static memcached_st *handle = NULL;
 }
 
 
+//
+// SOGo request count for rate-limiting
+//
+- (void) setRequestCount: (int) theCount
+                forLogin: (NSString *) theLogin
+                interval: (unsigned int) theInterval
+{
+  NSMutableDictionary *d;
+  NSNumber *count;
+
+  if (theCount)
+    {
+      count = [NSNumber numberWithInt: theCount];
+
+      d = [NSMutableDictionary dictionaryWithDictionary: [self requestCountForLogin: theLogin]];
+
+      if (![d objectForKey: @"InitialDate"] || theInterval == 0)
+        [d setObject: [NSNumber numberWithUnsignedInt: [[NSCalendarDate date] timeIntervalSince1970]] forKey: @"InitialDate"];
+      else
+        [d setObject: [NSNumber numberWithUnsignedInt: theInterval] forKey: @"InitialDate"];
+
+      [d setObject: count  forKey: @"RequestCount"];
+      [self _cacheValues: [d jsonRepresentation]
+                  ofType:  @"requestcount"
+                  forKey: theLogin];
+    }
+  else
+    {
+      [self removeValueForKey: [NSString stringWithFormat: @"%@+failedlogins", theLogin]];
+    }
+}
+
+//
+// Returns a dictionary with two keys/values
+//
+// RequestCount ->
+// InitialDate ->
+//
+- (NSDictionary *) requestCountForLogin: (NSString *) theLogin
+{
+  NSDictionary *d;
+  NSString *s;
+
+  s = [self _valuesOfType: @"requestcount" forKey: theLogin];
+  d = nil;
+
+  if (s)
+    {
+      d = [s objectFromJSONString];
+    }
+
+  return d;
+}
 
 @end
