@@ -1,6 +1,6 @@
 /* SOGoDomainDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2015 Inverse inc.
+ * Copyright (C) 2009-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,7 +19,6 @@
  */
 
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
 #import <NGObjWeb/WOApplication.h>
 #import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WORequest.h>
@@ -27,8 +26,6 @@
 
 #import "SOGoSystemDefaults.h"
 
-#import "SOGoDomainDefaults.h"
-#import "SOGoConstants.h"
 
 @implementation SOGoDomainDefaults
 
@@ -50,6 +47,9 @@
                                         andParentSource: systemDefaults];
     }
 
+  if (!domainDefaults)
+    domainDefaults = [SOGoSystemDefaults sharedSystemDefaults];
+
   return domainDefaults;
 }
 
@@ -191,6 +191,11 @@
   return [self boolForKey: @"SOGoForceIMAPLoginWithEmail"];
 }
 
+- (BOOL) externalAvatarsEnabled
+{
+  return [self boolForKey: @"SOGoExternalAvatarsEnabled"];
+}
+
 - (BOOL) sieveScriptsEnabled
 {
   return [self boolForKey: @"SOGoSieveScriptsEnabled"];
@@ -215,6 +220,21 @@
   return [self boolForKey: @"SOGoVacationEnabled"];
 }
 
+- (NSString *) vacationDefaultSubject
+{
+  return [self stringForKey: @"SOGoVacationDefaultSubject"];
+}
+
+- (NSString *) vacationHeaderTemplateFile
+{
+  return [self stringForKey: @"SOGoVacationHeaderTemplateFile"];
+}
+
+- (NSString *) vacationFooterTemplateFile
+{
+  return [self stringForKey: @"SOGoVacationFooterTemplateFile"];
+}
+
 - (NSString *) mailingMechanism
 {
   NSString *mailingMechanism;
@@ -294,11 +314,6 @@
   return [self stringForKey: @"SOGoLDAPContactInfoAttribute"];
 }
 
-- (NSString *) calendarDefaultCategoryColor
-{
-  return [self stringForKey: @"SOGoCalendarDefaultCategoryColor"];
-}
-
 - (NSArray *) freeBusyDefaultInterval
 {
   return [self arrayForKey: @"SOGoFreeBusyDefaultInterval"];
@@ -363,4 +378,9 @@
   return [self boolForKey: @"SOGoNotifyOnExternalModifications"];
 }
 
+- (NSDictionary *) mailJunkSettings
+{
+  return [self objectForKey: @"SOGoMailJunkSettings"];
+}
+
 @end
