@@ -19,30 +19,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSKeyValueCoding.h>
-#import <Foundation/NSURL.h>
-#import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
-#import <NGObjWeb/SoObject.h>
 #import <NGObjWeb/SoObject+SoDAV.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
 #import <NGObjWeb/WOApplication.h>
-#import <NGObjWeb/WOResponse.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
-#import <DOM/DOMElement.h>
-#import <DOM/DOMProtocols.h>
 #import <EOControl/EOFetchSpecification.h>
 #import <EOControl/EOQualifier.h>
-#import <EOControl/EOSortOrdering.h>
 #import <GDLAccess/EOAdaptorChannel.h>
 #import <GDLAccess/EOAdaptorContext.h>
 #import <GDLContentStore/GCSChannelManager.h>
@@ -67,7 +53,6 @@
 #import "SOGoParentFolder.h"
 #import "SOGoPermissions.h"
 #import "SOGoUser.h"
-#import "SOGoUserDefaults.h"
 #import "SOGoUserSettings.h"
 #import "SOGoUserManager.h"
 #import "SOGoWebDAVAclManager.h"
@@ -301,6 +286,29 @@ static NSArray *childRecordFields = nil;
   return name;
 }
 
+- (BOOL) synchronize
+{
+  NSNumber *synchronize;
+
+  synchronize = [self folderPropertyValueInCategory: @"FolderSynchronize"];
+
+  return [synchronize boolValue];
+}
+
+- (void) setSynchronize: (BOOL) new
+{
+  NSNumber *synchronize;
+
+  if (new)
+    synchronize = [NSNumber numberWithBool: YES];
+  else
+    synchronize = nil;
+
+  [self setFolderPropertyValue: synchronize
+                    inCategory: @"FolderSynchronize"];
+}
+
+
 /* This method fetches the display name defined by the owner, but is also the
    fallback when a subscriber has not redefined the display name yet in their
    environment. */
@@ -376,12 +384,12 @@ static NSArray *childRecordFields = nil;
   if (!displayName)
     {
       if (activeUserIsOwner)
-        displayName = [self _displayNameFromOwner];
+        displayName = (NSMutableString *)[self _displayNameFromOwner];
       else
         {
-          displayName = [self _displayNameFromSubscriber];
+          displayName = (NSMutableString *)[self _displayNameFromSubscriber];
           if (!displayName)
-            displayName = [self _displayNameFromOwner];
+            displayName = (NSMutableString *)[self _displayNameFromOwner];
         }
       [displayName retain];
     }
@@ -467,9 +475,8 @@ static NSArray *childRecordFields = nil;
   davURL = [self realDavURL];
   newPath = [NSMutableArray arrayWithArray: [[davURL path] componentsSeparatedByString: @"/"]];
   [newPath insertObject: @"public" atIndex: 3];
-  davURL = [[NSURL alloc] initWithScheme: [davURL scheme]
-				    host: [davURL host]
-				    path: [newPath componentsJoinedByString: @"/"]];
+  davURL = [NSURL URLWithString: [newPath componentsJoinedByString: @"/"]
+                  relativeToURL: davURL];
 
   return davURL;
 }
@@ -492,10 +499,7 @@ static NSArray *childRecordFields = nil;
                        [container nameInContainer],
                        [self realNameInContainer]];
       currentDavURL = [self davURL];
-      realDavURL = [[NSURL alloc] initWithScheme: [currentDavURL scheme]
-                                            host: [currentDavURL host]
-                                            path: path];
-      [realDavURL autorelease];
+      realDavURL = [NSURL URLWithString: path relativeToURL: currentDavURL];
     }
   else
     realDavURL = [self davURL];
@@ -829,7 +833,7 @@ static NSArray *childRecordFields = nil;
     {
       currentID = [ids objectAtIndex: count];
       names = [[currentID componentsSeparatedByString: @"/"] objectEnumerator];
-      deleteObject = self;
+      deleteObject = (SOGoContentObject *)self;
       while ((currentName = [names nextObject]))
         {
           deleteObject = [deleteObject lookupName: currentName
@@ -1151,6 +1155,7 @@ static NSArray *childRecordFields = nil;
 - (NSArray *) syncTokenFieldsWithProperties: (NSDictionary *) properties
                           matchingSyncToken: (NSString *) syncToken
                                    fromDate: (NSCalendarDate *) theStartDate
+                                initialLoad: (BOOL) initialLoadInProgress
 {
   /* TODO:
      - validation:
@@ -1200,13 +1205,17 @@ static NSArray *childRecordFields = nil;
       mRecords = [NSMutableArray arrayWithArray: [self _fetchFields: fields
                                                       withQualifier: qualifier
                                                       ignoreDeleted: YES]];
-      qualifier = [EOQualifier qualifierWithQualifierFormat:
-                                 @"c_lastmodified > %d and c_deleted == 1",
-                               syncTokenInt];
-      fields = [NSMutableArray arrayWithObjects: @"c_name", @"c_lastmodified", @"c_deleted", nil];
-      [mRecords addObjectsFromArray: [self _fetchFields: fields
-                                          withQualifier: qualifier
-                                          ignoreDeleted: NO]];
+      if (!initialLoadInProgress)
+        {
+          qualifier = [EOQualifier qualifierWithQualifierFormat:
+                                   @"c_lastmodified > %d and c_deleted == 1",
+                                   syncTokenInt];
+          fields = [NSMutableArray arrayWithObjects: @"c_name", @"c_lastmodified", @"c_deleted", nil];
+          [mRecords addObjectsFromArray: [self _fetchFields: fields
+                                              withQualifier: qualifier
+                                              ignoreDeleted: NO]];
+        }
+
       records = mRecords;
     }
   else
@@ -1499,7 +1508,9 @@ static NSArray *childRecordFields = nil;
       properties = [self parseDAVRequestedProperties: propElement];
       records = [self syncTokenFieldsWithProperties: properties
                                   matchingSyncToken: syncToken
-                                           fromDate: nil];
+                                           fromDate: nil
+                                        initialLoad: NO];
+
       [self _appendComponentProperties: [properties allKeys]
                            fromRecords: records
                      matchingSyncToken: [syncToken intValue]
@@ -1710,7 +1721,7 @@ static NSArray *childRecordFields = nil;
 {
   EOQualifier *qualifier;
   NSString *uid, *uids, *qs, *objectPath, *domain;
-  NSMutableArray *usersAndGroups;
+  NSMutableArray *usersAndGroups, *groupsMembers;
   NSMutableDictionary *aclsForObject;
   SOGoGroup *group;
   unsigned int i;
@@ -1719,23 +1730,40 @@ static NSArray *childRecordFields = nil;
     {
       domain = [[context activeUser] domain];
       usersAndGroups = [NSMutableArray arrayWithArray: users];
+      groupsMembers = [NSMutableArray array];
       for (i = 0; i < [usersAndGroups count]; i++)
         {
           uid = [usersAndGroups objectAtIndex: i];
           if (![uid hasPrefix: @"@"])
             {
-              // Prefix the UID with the character "@" when dealing with a group
               group = [SOGoGroup groupWithIdentifier: uid inDomain: domain];
               if (group)
-                [usersAndGroups replaceObjectAtIndex: i
-                                          withObject: [NSString stringWithFormat: @"@%@", uid]];
+                {
+                  NSArray *members;
+                  SOGoUser *user;
+                  unsigned int j;
+
+                  // Fetch members to remove them from the cache along the group
+                  members = [group members];
+                  for (j = 0; j < [members count]; j++)
+                    {
+                      user = [members objectAtIndex: j];
+                      [groupsMembers addObject: [user login]];
+                    }
+
+                  // Prefix the UID with the character "@" when dealing with a group
+                  [usersAndGroups replaceObjectAtIndex: i
+                                            withObject: [NSString stringWithFormat: @"@%@", uid]];
+                }
             }
         }
       objectPath = [objectPathArray componentsJoinedByString: @"/"];
       aclsForObject = [[SOGoCache sharedCache] aclsForPath: objectPath];
       if (aclsForObject)
 	{
+          // Remove users, groups and groups members from the cache
 	  [aclsForObject removeObjectsForKeys: usersAndGroups];
+	  [aclsForObject removeObjectsForKeys: groupsMembers];
 	  [[SOGoCache sharedCache] setACLs: aclsForObject
 				   forPath: objectPath];
 	}
@@ -1763,11 +1791,18 @@ static NSArray *childRecordFields = nil;
   userRoles = [roles objectEnumerator];
   while ((currentRole = [userRoles nextObject]))
     {
-      SQL = [NSString stringWithFormat: @"INSERT INTO %@"
-                      @" (c_object, c_uid, c_role)"
-                      @" VALUES ('/%@', '%@', '%@')",
-                      [folder aclTableName],
-                      objectPath, uid, currentRole];
+      if ([GCSFolderManager singleStoreMode])
+        SQL = [NSString stringWithFormat: @"INSERT INTO %@"
+                        @" (c_object, c_uid, c_role, c_folder_id)"
+                        @" VALUES ('/%@', '%@', '%@', %@)",
+                        [folder aclTableName],
+                        objectPath, uid, currentRole, [folder folderId]];
+      else
+        SQL = [NSString stringWithFormat: @"INSERT INTO %@"
+                        @" (c_object, c_uid, c_role)"
+                        @" VALUES ('/%@', '%@', '%@')",
+                        [folder aclTableName],
+                        objectPath, uid, currentRole];
       [channel evaluateExpressionX: SQL];
     }
 
@@ -1783,6 +1818,11 @@ static NSArray *childRecordFields = nil;
   NSMutableArray *newRoles;
   SOGoGroup *group;
 
+  objectPath = [objectPathArray componentsJoinedByString: @"/"];
+
+  // We make sure we don't get unescaped uid - like foo%40bar.com
+  // or for groups - like %40team
+  uid = [uid stringByUnescapingURL];
   aUID = uid;
   if (![uid hasPrefix: @"@"])
     {
@@ -1790,7 +1830,12 @@ static NSArray *childRecordFields = nil;
       domain = [[context activeUser] domain];
       group = [SOGoGroup groupWithIdentifier: uid inDomain: domain];
       if (group)
-        aUID = [NSString stringWithFormat: @"@%@", uid];
+        {
+          aUID = [NSString stringWithFormat: @"@%@", uid];
+          // Remove all roles when defining ACLs for a group
+          [[SOGoCache sharedCache] setACLs: nil
+                                   forPath: objectPath];
+        }
     }
   [self removeAclsForUsers: [NSArray arrayWithObject: aUID]
            forObjectAtPath: objectPathArray];
@@ -1801,12 +1846,11 @@ static NSArray *childRecordFields = nil;
   [newRoles removeObject: SOGoRole_PublicUser];
   [newRoles removeObject: SOGoRole_AuthorizedSubscriber];
   [newRoles removeObject: SOGoRole_None];
-  objectPath = [objectPathArray componentsJoinedByString: @"/"];
   
   if (![newRoles count])
     [newRoles addObject: SOGoRole_None];
 
-  [self _cacheRoles: newRoles forUser: uid
+  [self _cacheRoles: newRoles forUser: aUID
 	      forObjectAtPath: objectPath];
 
   [self _commitRoles: newRoles forUID: aUID forObject: objectPath];
@@ -1843,9 +1887,9 @@ static NSArray *childRecordFields = nil;
 - (void) setRoles: (NSArray *) roles
           forUser: (NSString *) uid
 {
-  return [self    setRoles: roles
-                   forUser: uid
-           forObjectAtPath: [self pathArrayToFolder]];
+  return [self setRoles: roles
+		forUser: uid
+	       forObjectAtPath: [self pathArrayToFolder]];
 }
 
 - (void) removeAclsForUsers: (NSArray *) users
@@ -2054,7 +2098,7 @@ static NSArray *childRecordFields = nil;
       methodSel = SOGoSelectorForPropertyGetter (*currentProperty);
       if (methodSel && [sogoObject respondsToSelector: methodSel])
         *currentValue = [[sogoObject performSelector: methodSel]
-                          stringByEscapingXMLString];
+                          safeStringByEscapingXMLString];
       currentProperty++;
       currentValue++;
     }
