@@ -1,6 +1,6 @@
 /* SOGoSystemDefaults.h - this file is part of SOGo
  *
- * Copyright (C) 2009-2014 Inverse inc.
+ * Copyright (C) 2009-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -64,7 +64,11 @@
 - (BOOL) uixAdditionalPreferences;
 
 - (BOOL) uixDebugEnabled;
+- (BOOL) easDebugEnabled;
+- (BOOL) xsrfValidationEnabled;
+
 - (NSString *) pageTitle;
+- (NSString *) helpURL;
 
 - (NSArray *) supportedLanguages;
 - (NSString *) loginSuffix;
@@ -90,11 +94,18 @@
 - (int) maximumFailedLoginInterval;
 - (int) failedLoginBlockInterval;
 
+- (int) maximumMessageSizeLimit;
+
 - (int) maximumMessageSubmissionCount;
 - (int) maximumRecipientCount;
 - (int) maximumSubmissionInterval;
 - (int) messageSubmissionBlockInterval;
 
+- (int) maximumRequestCount;
+- (int) maximumRequestInterval;
+- (int) requestBlockInterval;
+
+
 - (int) maximumPingInterval;
 - (int) maximumSyncInterval;
 - (int) internalSyncInterval;
