@@ -1,6 +1,6 @@
 /* SOGoSystemDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2014 Inverse inc.
+ * Copyright (C) 2009-2015 Inverse inc.
  * Copyright (C) 2012 Jeroen Dekkers <jeroen@dekkers.ch>
  *
  * This file is free software; you can redistribute it and/or modify
@@ -20,13 +20,10 @@
  */
 
 #import <dlfcn.h>
-#import <unistd.h>
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSBundle.h>
 #import <Foundation/NSFileManager.h>
 #import <Foundation/NSFileManager.h>
-#import <Foundation/NSDictionary.h>
 #import <Foundation/NSUserDefaults.h>
 
 #import <NGExtensions/NSObject+Logs.h>
@@ -41,7 +38,6 @@
 @implementation SOGoSystemDefaults
 
 #if defined(LDAP_CONFIG)
-#import <SOGo/SOGoLDAPUserDefaults.h>
 #endif
 
 typedef void (*NSUserDefaultsInitFunction) ();
@@ -261,23 +257,25 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
 
 - (BOOL) enableDomainBasedUID
 {
-  return ([[self domainIds] count] > 0 && [self boolForKey: @"SOGoEnableDomainBasedUID"]);
+  return [self boolForKey: @"SOGoEnableDomainBasedUID"];
 }
 
 - (NSArray *) loginDomains
 {
   NSMutableArray *filteredLoginDomains;
   NSArray *domains;
-  NSEnumerator *objects;
   id currentObject;
+  int count;
   
   if (self->loginDomains == nil)
     {
       filteredLoginDomains = [NSMutableArray arrayWithArray: [self stringArrayForKey: @"SOGoLoginDomains"]];
       domains = [self domainIds];
-      objects = [filteredLoginDomains objectEnumerator];
-      while ((currentObject = [objects nextObject]))
+      count = [filteredLoginDomains count];
+      while (count > 0)
         {
+          count--;
+          currentObject = [filteredLoginDomains objectAtIndex: count];
           if (![domains containsObject: currentObject])
             {
               [filteredLoginDomains removeObject: currentObject];
@@ -427,11 +425,33 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return [self boolForKey: @"SOGoUIxDebugEnabled"];
 }
 
+- (BOOL) easDebugEnabled
+{
+  return [self boolForKey: @"SOGoEASDebugEnabled"];
+}
+
+- (BOOL) xsrfValidationEnabled
+{
+  id o;
+
+  if (!(o = [self objectForKey: @"SOGoXSRFValidationEnabled"]))
+    {
+      return YES;
+    }
+
+  return [o boolValue];
+}
+
 - (NSString *) pageTitle
 {
   return [self stringForKey: @"SOGoPageTitle"];
 }
 
+- (NSString *) helpURL
+{
+  return [self stringForKey: @"SOGoHelpURL"];
+}
+
 - (NSArray *) supportedLanguages
 {
   static NSArray *supportedLanguages = nil;
@@ -558,6 +578,14 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return v;
 }
 
+//
+//
+//
+- (int) maximumMessageSizeLimit
+{
+  return [self integerForKey: @"SOGoMaximumMessageSizeLimit"];
+}
+
 //
 //
 //
@@ -595,6 +623,42 @@ _injectConfigurationFromFile (NSMutableDictionary *defaultsDict,
   return v;
 }
 
+//
+// SOGo rate-limiting
+//
+- (int) maximumRequestCount
+{
+  return [self integerForKey: @"SOGoMaximumRequestCount"];
+}
+
+- (int) maximumRequestInterval
+{
+  int v;
+
+  v = [self integerForKey: @"SOGoMaximumRequestInterval"];
+
+  if (!v)
+    v = 30;
+
+  return v;
+}
+
+- (int) requestBlockInterval
+{
+  int v;
+
+  v = [self integerForKey: @"SOGoRequestBlockInterval"];
+
+  if (!v)
+    v = 300;
+
+  return v;
+}
+
+
+//
+// SOGo EAS settings
+//
 - (int) maximumPingInterval
 {
   int v;
