@@ -1,6 +1,6 @@
 /* SOGoUserDefaults.m - this file is part of SOGo
  *
- * Copyright (C) 2009-2014 Inverse inc.
+ * Copyright (C) 2009-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,10 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSSet.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSTimeZone.h>
 
 #import <NGImap4/NSString+Imap4.h>
@@ -30,11 +27,9 @@
 #import <NGObjWeb/WEClientCapabilities.h>
 
 #import "NSString+Utilities.h"
-#import "SOGoDomainDefaults.h"
 #import "SOGoSystemDefaults.h"
 #import "SOGoUserProfile.h"
 
-#import "SOGoUserDefaults.h"
 
 
 NSString *SOGoWeekStartJanuary1 = @"January1";
@@ -405,6 +400,16 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self boolForKey: @"SOGoMailShowSubscribedFoldersOnly"];
 }
 
+- (void) setSynchronizeOnlyDefaultMailFolders: (BOOL) newValue
+{
+  [self setBool: newValue forKey: @"SOGoMailSynchronizeOnlyDefaultFolders"];
+}
+
+- (BOOL) synchronizeOnlyDefaultMailFolders
+{
+  return [self boolForKey: @"SOGoMailSynchronizeOnlyDefaultFolders"];
+}
+
 - (void) setMailSortByThreads: (BOOL) newValue
 {
   [self setBool: newValue forKey: @"SOGoMailSortByThreads"];
@@ -448,6 +453,17 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
              stringByEncodingImap4FolderName];
 }
 
+- (void) setJunkFolderName: (NSString *) newValue
+{
+  [self setObject: newValue forKey: @"SOGoJunkFolderName"];
+}
+
+- (NSString *) junkFolderName
+{
+  return [[self stringForKey: @"SOGoJunkFolderName"]
+             stringByEncodingImap4FolderName];
+}
+
 - (void) setFirstDayOfWeek: (int) newValue
 {
   [self setInteger: newValue forKey: @"SOGoFirstDayOfWeek"];
@@ -498,6 +514,21 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self stringForKey: @"SOGoRefreshViewCheck"];
 }
 
+- (BOOL) gravatarEnabled
+{
+  return [self boolForKey: @"SOGoGravatarEnabled"];
+}
+
+- (void) setAlternateAvatar: (NSString *) newValue
+{
+  [self setObject: newValue forKey: @"SOGoAlternateAvatar"];
+}
+
+- (NSString *) alternateAvatar
+{
+  return [self stringForKey: @"SOGoAlternateAvatar"];
+}
+
 - (void) setMailComposeMessageType: (NSString *) newValue
 {
   [self setObject: newValue forKey: @"SOGoMailComposeMessageType"];
@@ -508,6 +539,16 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
   return [self stringForKey: @"SOGoMailComposeMessageType"];
 }
 
+- (void) setMailComposeFontSize: (NSString *) newValue
+{
+  [self setObject: newValue forKey: @"SOGoMailComposeFontSize"];
+}
+
+- (NSString *) mailComposeFontSize
+{
+  return [self stringForKey: @"SOGoMailComposeFontSize"];
+}
+
 - (void) setMailDisplayRemoteInlineImages: (NSString *) newValue
 {
   [self setObject: newValue forKey: @"SOGoMailDisplayRemoteInlineImages"];
@@ -702,7 +743,17 @@ NSString *SOGoWeekStartFirstFullWeek = @"FirstFullWeek";
 
 - (NSDictionary *) calendarCategoriesColors
 {
-  return [self dictionaryForKey: @"SOGoCalendarCategoriesColors"];
+  return [self objectForKey: @"SOGoCalendarCategoriesColors"];
+}
+
+- (void) setCalendarWeekdays: (NSArray *) newValues
+{
+  [self setObject: newValues forKey: @"SOGoCalendarWeekdays"];
+}
+
+- (NSArray *) calendarWeekdays
+{
+  return [self stringArrayForKey: @"SOGoCalendarWeekdays"];
 }
 
 - (void) setCalendarShouldDisplayWeekend: (BOOL) newValue
