@@ -1,15 +1,13 @@
 /*
   Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2007-2011 Inverse inc.
+  Copyright (C) 2007-2016 Inverse inc.
 
-  This file is part of OpenGroupware.org.
-
-  OGo is free software; you can redistribute it and/or modify it under
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
 
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
@@ -20,38 +18,24 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSURL.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
-#import <NGObjWeb/SoClassSecurityInfo.h>
 #import <NGObjWeb/SoSecurityManager.h>
-#import <NGObjWeb/WOApplication.h>
 #import <NGObjWeb/WOContext+SoObjects.h>
-#import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WORequest+So.h>
-#import <NGObjWeb/WOResponse.h>
 
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGExtensions/NSString+misc.h>
-#import <DOM/DOMDocument.h>
 #import <DOM/DOMNode.h>
-#import <DOM/DOMProtocols.h>
-#import <SaxObjC/SaxObjC.h>
-#import <SaxObjC/XMLNamespaces.h>
 
 #import <Appointments/SOGoAppointmentFolders.h>
-#import <Appointments/SOGoFreeBusyObject.h>
 #import <Appointments/SOGoCalendarProxy.h>
 #import <Contacts/SOGoContactFolders.h>
-#import <Mailer/SOGoMailAccounts.h>
 
-#import <SOGoUI/UIxComponent.h>
 
 #import "NSArray+Utilities.h"
 #import "NSDictionary+Utilities.h"
+#import "NSString+Utilities.h"
 #import "SOGoUserManager.h"
 #import "SOGoPermissions.h"
 #import "SOGoSystemDefaults.h"
@@ -286,7 +270,7 @@
       [r appendContentString: @"<D:status>HTTP/1.1 200 OK</D:status>"];
       [r appendContentString: @"<D:prop><D:displayname>"];
       data = [currentFolder objectForKey: @"displayName"];
-      [r appendContentString: [data stringByEscapingXMLString]];
+      [r appendContentString: [data safeStringByEscapingXMLString]];
       [r appendContentString: @"</D:displayname></D:prop></D:propstat>"];
 
       /* Remove this once extensions 0.8x are no longer used */
@@ -299,12 +283,12 @@
       ownerUser = [SOGoUser userWithLogin: [currentFolder objectForKey: @"owner"]
 			    roles: nil];
       data = [ownerUser cn];
-      [r appendContentString: [data stringByEscapingXMLString]];
+      [r appendContentString: [data safeStringByEscapingXMLString]];
       [r appendContentString: @"</ownerdisplayname>"];
 
       [r appendContentString: @"<D:displayname>"];
       data = [currentFolder objectForKey: @"displayName"];
-      [r appendContentString: [data stringByEscapingXMLString]];
+      [r appendContentString: [data safeStringByEscapingXMLString]];
       [r appendContentString: @"</D:displayname>"];
       /* end of temporary compatibility hack */
 
@@ -427,7 +411,8 @@
         {
           currentUser = [users objectAtIndex: i];
           field = [currentUser objectForKey: @"c_uid"];
-          if (enableDomainBasedUID)
+          if (enableDomainBasedUID &&
+              [field rangeOfString: @"@"].location == NSNotFound)
             field = [NSString stringWithFormat: @"%@@%@", field, domain];
           if (![field isEqualToString: login])
             {
@@ -435,14 +420,14 @@
                      [field stringByEscapingXMLString]];
               field = [currentUser objectForKey: @"cn"];
               [fetch appendFormat: @"<displayName>%@</displayName>",
-                     [field stringByEscapingXMLString]];
+                     [field safeStringByEscapingXMLString]];
               field = [currentUser objectForKey: @"c_email"];
               [fetch appendFormat: @"<email>%@</email>",
                      [field stringByEscapingXMLString]];
               field = [currentUser objectForKey: @"c_info"];
               if ([field length])
                 [fetch appendFormat: @"<info>%@</info>",
-                       [field stringByEscapingXMLString]];
+                       [field safeStringByEscapingXMLString]];
               [fetch appendString: @"</user>"];
             }
         }
