@@ -21,13 +21,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSException.h>
-#import <Foundation/NSObject.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSValue.h>
-#import <Foundation/NSURL.h>
 
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
@@ -37,7 +33,8 @@
 #import <GDLContentStore/EOQualifier+GCS.h>
 #import <GDLAccess/EOAdaptorChannel.h>
 
-#import "SOGoConstants.h"
+#import <SOGo/SOGoSystemDefaults.h>
+
 #import "NSString+Utilities.h"
 #import "NSString+Crypto.h"
 
@@ -98,6 +95,7 @@
       _kindField = nil;
       _multipleBookingsField = nil;
       _imapHostField = nil;
+      _sieveHostField = nil;
     }
 
   return self;
@@ -115,6 +113,7 @@
   [_multipleBookingsField release];
   [_domainField release];
   [_imapHostField release];
+  [_sieveHostField release];
    
   [super dealloc];
 }
@@ -131,6 +130,7 @@
   ASSIGN(_userPasswordAlgorithm, [udSource objectForKey: @"userPasswordAlgorithm"]);
   ASSIGN(_imapLoginField, [udSource objectForKey: @"IMAPLoginFieldName"]);
   ASSIGN(_imapHostField, [udSource objectForKey:  @"IMAPHostFieldName"]);
+  ASSIGN(_sieveHostField, [udSource objectForKey:  @"SieveHostFieldName"]);
   ASSIGN(_kindField, [udSource objectForKey: @"KindFieldName"]);
   ASSIGN(_multipleBookingsField, [udSource objectForKey: @"MultipleBookingsFieldName"]);
   ASSIGN(_domainField, [udSource objectForKey: @"DomainFieldName"]);
@@ -187,7 +187,10 @@
   pass = [plainPassword asCryptedPassUsingScheme: _userPasswordAlgorithm];
 
   if (pass == nil)
-    [self errorWithFormat: @"Unsupported user-password algorithm: %@", _userPasswordAlgorithm];
+    {
+      [self errorWithFormat: @"Unsupported user-password algorithm: %@", _userPasswordAlgorithm];
+      return nil;
+    }
 
   if (_prependPasswordScheme)
     result = [NSString stringWithFormat: @"{%@}%@", _userPasswordAlgorithm, pass];
@@ -308,18 +311,20 @@
   NSString *sqlstr;
   BOOL didChange;
   BOOL isOldPwdOk;
-  
+
   isOldPwdOk = NO;
   didChange = NO;
-  
+
   // Verify current password
   isOldPwdOk = [self checkLogin:login password:oldPassword perr:perr expire:0 grace:0];
-  
+
   if (isOldPwdOk)
     {
       // Encrypt new password
       NSString *encryptedPassword = [self _encryptPassword: newPassword];
-      
+      if(encryptedPassword == nil)
+        return NO;
+
       // Save new password
       login = [login stringByReplacingString: @"'"  withString: @"''"];
       cm = [GCSChannelManager defaultChannelManager];
@@ -479,10 +484,12 @@
                              forKey: [field substringFromIndex: 2]];
             }
 
+          // FIXME
           // We have to do this here since we do not manage modules
           // constraints right now over a SQL backend.
           [response setObject: [NSNumber numberWithBool: YES] forKey: @"CalendarAccess"];
           [response setObject: [NSNumber numberWithBool: YES] forKey: @"MailAccess"];
+          [response setObject: [NSNumber numberWithBool: YES] forKey: @"ActiveSyncAccess"];
 
 	  // We set the domain, if any
           value = nil;
@@ -508,7 +515,7 @@
 	      for (i = 0; i < [_mailFields count]; i++)
 		if ((s = [response objectForKey: [_mailFields objectAtIndex: i]]) &&
 		    [[s stringByTrimmingSpaces] length] > 0)
-		  [emails addObject: s];
+		  [emails addObjectsFromArray: [s componentsSeparatedByString: @" "]];
 	    }
 	  
 	  [response setObject: emails  forKey: @"c_emails"];
@@ -519,6 +526,13 @@
                 [response setObject: value forKey: @"c_imaphostname"];
             }
 
+          if (_sieveHostField)
+            {
+              value = [response objectForKey: _sieveHostField];
+              if ([value isNotNull])
+                [response setObject: value forKey: @"c_sievehostname"];
+            }
+
           // We check if the user can authenticate
           if (_authenticationFilter)
             {
@@ -601,8 +615,9 @@
 
 
 - (NSDictionary *) lookupContactEntry: (NSString *) theID
+                             inDomain: (NSString *) domain
 {
-  return [self _lookupContactEntry: theID  considerEmail: NO inDomain: nil];
+  return [self _lookupContactEntry: theID  considerEmail: NO inDomain: domain];
 }
 
 - (NSDictionary *) lookupContactEntryWithUIDorEmail: (NSString *) entryID
@@ -611,23 +626,94 @@
   return [self _lookupContactEntry: entryID  considerEmail: YES inDomain: domain];
 }
 
-- (NSArray *) allEntryIDs
+/* Returns an EOQualifier of the following form:
+ * (_domainField = domain OR _domainField = visibleDomain1 [...])
+ * Should only be called on SQL sources using _domainField name.
+ */
+- (EOQualifier *) _visibleDomainsQualifierFromDomain: (NSString *) domain
+{
+  int i;
+  EOQualifier *qualifier, *domainQualifier;
+  NSArray *visibleDomains;
+  NSMutableArray *qualifiers;
+  NSString *currentDomain;
+
+  SOGoSystemDefaults *sd;
+
+  /* Return early if no domain or if being called on a 'static' sql source */
+  if (!domain || !_domainField)
+    return nil;
+
+  sd = [SOGoSystemDefaults sharedSystemDefaults];
+  visibleDomains = [sd visibleDomainsForDomain: domain];
+  qualifier = nil;
+
+  domainQualifier =
+    [[EOKeyValueQualifier alloc] initWithKey: _domainField
+                            operatorSelector: EOQualifierOperatorEqual
+                                       value: domain];
+  [domainQualifier autorelease];
+
+  if ([visibleDomains count])
+    {
+      qualifiers = [NSMutableArray arrayWithCapacity: [visibleDomains count] + 1];
+      [qualifiers addObject: domainQualifier];
+      for(i = 0; i < [visibleDomains count]; i++)
+        {
+          currentDomain = [visibleDomains objectAtIndex: i];
+          qualifier =
+            [[EOKeyValueQualifier alloc] initWithKey: _domainField
+                                    operatorSelector: EOQualifierOperatorEqual
+                                               value: currentDomain];
+          [qualifier autorelease];
+          [qualifiers addObject: qualifier];
+        }
+      qualifier = [[EOOrQualifier alloc] initWithQualifierArray: qualifiers];
+      [qualifier autorelease];
+    }
+
+  return qualifier ? qualifier : domainQualifier;
+}
+
+
+- (NSArray *) allEntryIDsVisibleFromDomain: (NSString *) domain
 {
   EOAdaptorChannel *channel;
-  NSMutableArray *results;
+  EOQualifier *domainQualifier;
   GCSChannelManager *cm;
   NSException *ex;
-  NSString *sql;
-  
+  NSMutableArray *results;
+  NSMutableString *sql;
+
   results = [NSMutableArray array];
 
   cm = [GCSChannelManager defaultChannelManager];
   channel = [cm acquireOpenChannelForURL: _viewURL];
   if (channel)
     {
-      sql = [NSString stringWithFormat: @"SELECT c_uid FROM %@",
+      sql = [NSMutableString stringWithFormat: @"SELECT c_uid FROM %@",
                       [_viewURL gcsTableName]];
 
+      if (_domainField)
+        {
+          if ([domain length])
+            {
+              domainQualifier =
+                [self _visibleDomainsQualifierFromDomain: domain];
+              if (domainQualifier)
+                {
+                  [sql appendString: @" WHERE "];
+                  [domainQualifier _gcsAppendToString: sql];
+                }
+            }
+          else
+            {
+              /* Should not happen but avoid returning the whole table
+               * if a domain should have been defined */
+              [sql appendFormat: @" WHERE %@ is NULL", _domainField];
+            }
+        }
+
       ex = [channel evaluateExpressionX: sql];
       if (!ex)
         {
@@ -656,6 +742,11 @@
   return results;
 }
 
+- (NSArray *) allEntryIDs
+{
+  return [self allEntryIDsVisibleFromDomain: nil];
+}
+
 - (NSArray *) fetchContactsMatching: (NSString *) filter
                            inDomain: (NSString *) domain
 {
@@ -684,16 +775,26 @@
                       lowerFilter, lowerFilter];
       
       if (_mailFields && [_mailFields count] > 0)
-	{
-	  [sql appendString: [self _whereClauseFromArray: _mailFields  value: lowerFilter  exact: NO]];
-	}
+        {
+          [sql appendString: [self _whereClauseFromArray: _mailFields  value: lowerFilter  exact: NO]];
+        }
 
       [sql appendString: @")"];
 
       if (_domainField)
         {
           if ([domain length])
-            [sql appendFormat: @" AND %@ = '%@'", _domainField, domain];
+            {
+              EOQualifier *domainQualifier;
+              domainQualifier =
+                [self _visibleDomainsQualifierFromDomain: domain];
+              if (domainQualifier)
+                {
+                  [sql appendFormat: @" AND ("];
+                  [domainQualifier _gcsAppendToString: sql];
+                  [sql appendFormat: @")"];
+                }
+            }
           else
             [sql appendFormat: @" AND %@ IS NULL", _domainField];
         }
@@ -775,7 +876,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
@@ -788,7 +889,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
@@ -801,7 +902,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
@@ -827,7 +928,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
@@ -842,7 +943,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
@@ -856,7 +957,7 @@
 
   reason = [NSString stringWithFormat: @"method '%@' is not available"
                      @" for class '%@'", NSStringFromSelector (_cmd),
-                     NSStringFromClass (isa)];
+                     NSStringFromClass (object_getClass(self))];
 
   return [NSException exceptionWithName: @"SQLSourceIOException"
                                  reason: reason
