@@ -1,6 +1,6 @@
 /* SOGoEAlarmsNotifier.m - this file is part of SOGo
  *
- * Copyright (C) 2011-2014 Inverse inc.
+ * Copyright (C) 2011-2016 Inverse inc.
  *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,32 +20,24 @@
 
 #include <unistd.h>
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSProcessInfo.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSUserDefaults.h>
 
 #import <NGExtensions/NGHashMap.h>
 #import <NGExtensions/NGQuotedPrintableCoding.h>
 #import <NGExtensions/NSObject+Logs.h>
 #import <NGMail/NGMimeMessage.h>
-#import <NGMime/NGMimeBodyPart.h>
 
 #import <NGCards/iCalAlarm.h>
-#import <NGCards/iCalEntityObject.h>
-#import <NGCards/iCalPerson.h>
 
-#import "SOGo/SOGoCredentialsFile.h"
 #import <SOGo/NSCalendarDate+SOGo.h>
 #import <SOGo/NSString+Utilities.h>
-#import <SOGo/SOGoDomainDefaults.h>
+#import <SOGo/SOGoCredentialsFile.h>
 #import <SOGo/SOGoMailer.h>
 #import <SOGo/SOGoProductLoader.h>
-#import <SOGo/SOGoStaticAuthenticator.h>
-#import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/SOGoUser.h>
+#import <Appointments/iCalEntityObject+SOGo.h>
 #import <Appointments/iCalPerson+SOGo.h>
 #import <Appointments/SOGoEMailAlarmsManager.h>
 
@@ -81,8 +73,8 @@
   timestamp = (int) [[NSDate date] timeIntervalSince1970];
   pGUID = [[NSProcessInfo processInfo] globallyUniqueString];
 
-  messageID = [NSString stringWithFormat: @"<%0X-%0X-%0X-%0X@%u>",
-                        pid, timestamp, sequence, random(), [pGUID hash]];
+  messageID = [NSString stringWithFormat: @"<%0X-%0X-%0X-%0X@%lu>",
+                        pid, timestamp, sequence, (unsigned int)random(), [pGUID hash]];
 
   return [messageID lowercaseString];
 }
@@ -188,12 +180,15 @@
 
 - (BOOL) run
 {
-  SOGoEMailAlarmsManager *eaMgr;
-  SOGoCredentialsFile *cf;
   NSCalendarDate *startDate, *toDate;
-  NSArray *alarms;
-  NSMutableArray *owners;
+  SOGoEMailAlarmsManager *eaMgr;
+  NSMutableArray *metadata;
+  iCalEntityObject *entity;
+  SOGoCredentialsFile *cf;
   NSString *credsFilename;
+  NSDictionary *d;
+  NSArray *alarms;
+
   int count, max;
 
   if ([[NSUserDefaults standardUserDefaults] stringForKey: @"h"])
@@ -220,6 +215,7 @@
   eaMgr = [NSClassFromString (@"SOGoEMailAlarmsManager")
                              sharedEMailAlarmsManager];
 
+  metadata = [[NSMutableArray alloc] init];
   startDate = [NSCalendarDate calendarDate];
   toDate = [startDate addYear: 0 month: 0 day: 0
                          hour: 0 minute: 0
@@ -228,13 +224,26 @@
                                                       hour: 0 minute: -5
                                                     second: 0]
                                    toDate: toDate
-                               withOwners: &owners];
+                               withMetadata: metadata];
   max = [alarms count];
   for (count = 0; count < max; count++)
     [self _processAlarm: [alarms objectAtIndex: count]
-              withOwner: [owners objectAtIndex: count]];
+              withOwner: [[metadata objectAtIndex: count] objectForKey: @"owner"]];
+
+  // We now update the next alarm date (if any, for recurring
+  // events or tasks for example). This will also delete any emai
+  // alarms that are no longer relevant
+  max = [metadata count];
+
+  for (count = 0; count < max; count++)
+    {
+      d = [metadata objectAtIndex: count];
+      entity = [d objectForKey: @"entity"];
+      [entity quickRecordFromContent: nil
+			   container: [d objectForKey: @"container"]
+		     nameInContainer: [[d objectForKey: @"record"] objectForKey: @"c_name"]];
+    }
 
-  [eaMgr deleteAlarmsUntilDate: toDate];
 
   return YES;
 }
