@@ -20,17 +20,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <stdio.h>
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSAutoreleasePool.h>
 #import <Foundation/NSCalendarDate.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSObject.h>
-#import <Foundation/NSString.h>
-#import <Foundation/NSUserDefaults.h>
 #import <Foundation/NSValue.h>
 
 #import <NGCards/NGVList.h>
@@ -40,7 +32,6 @@
 #import <GDLAccess/EOAdaptorChannel.h>
 #import <GDLAccess/EOAdaptorContext.h>
 
-#import <GDLContentStore/GCSChannelManager.h>
 #import <GDLContentStore/GCSFolderManager.h>
 #import <GDLContentStore/GCSFolder.h>
 
@@ -175,7 +166,7 @@
 
   now = [NSCalendarDate date];
   delSql = [NSString stringWithFormat: @"UPDATE %@"
-                     @" SET c_deleted = 1, c_lastmodified = %d,"
+                     @" SET c_deleted = 1, c_lastmodified = %lu,"
                      @" c_content = ''"
 		     @" WHERE c_name = '%@'",
 		     tableName,
@@ -547,7 +538,7 @@
   fom = [GCSFolderManager defaultFolderManager];
   if (fom)
     rc = [self processFolder: folder ofUser: username
-	       withFoM: fom];
+		     withFoM: fom];
   else
     rc = NO;
 
