@@ -20,16 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <stdio.h>
 
 #import <Foundation/NSAutoreleasePool.h>
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
 #import <Foundation/NSException.h>
-#import <Foundation/NSString.h>
 
 #import <EOControl/EOQualifier.h>
-#import <GDLAccess/EOAdaptorChannel.h>
 #import <GDLAccess/EOAdaptorContext.h>
 
 #import <GDLContentStore/GCSChannelManager.h>
@@ -148,7 +144,7 @@
   if (sqlError)
     {
       [ac rollbackTransaction];
-      NSLog([sqlError reason]);
+      NSLog(@"%@", [sqlError reason]);
     }
   else
     rc = [ac commitTransaction];
@@ -201,7 +197,7 @@
   if (sqlError)
     {
       [ac rollbackTransaction];
-      NSLog([sqlError reason]);
+      NSLog(@"%@", [sqlError reason]);
     }
   else
     rc = [ac commitTransaction];
