@@ -20,21 +20,16 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCharacterSet.h>
 #import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSString.h>
 #import <Foundation/NSUserDefaults.h>
 
 #import <NGObjWeb/WOContext+SoObjects.h>
-#import <NGObjWeb/WOApplication.h>
 
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoProductLoader.h>
 #import "SOGo/SOGoCredentialsFile.h"
 #import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserFolder.h>
 #import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoUserSettings.h>
 #import <Mailer/SOGoMailAccounts.h>
@@ -115,7 +110,7 @@ typedef enum
       [theKey caseInsensitiveCompare: @"Vacation"] == NSOrderedSame)
     {
       /* credentials file handling */
-      NSString *credsFilename, *authname, *authpwd;
+      NSString *credsFilename=nil, *authname=nil, *authpwd=nil;
       SOGoCredentialsFile *cf;
 
       credsFilename = [[NSUserDefaults standardUserDefaults] stringForKey: @"p"];
