@@ -1,6 +1,5 @@
 /*
-  Copyright (C) 2004-2005 SKYRIX Software AG
-  Copyright (C) 2006-2013 Inverse inc.
+  Copyright (C) 2006-2016 Inverse inc.
 
   This file is part of SOGo
 
@@ -26,16 +25,17 @@
 #import <SoObjects/SOGo/NSString+Utilities.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WOResponse.h>
+#define COMPILING_NGOBJWEB 1 /* we want httpRequest for parsing multi-part
+                                form data */
 #import <NGObjWeb/WORequest.h>
+#undef COMPILING_NGOBJWEB
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NGBase64Coding.h>
+#import <NGHttp/NGHttpRequest.h>
+#import <NGMime/NGMimeMultipartBody.h>
 
-#import <Contacts/SOGoContactObject.h>
-#import <Contacts/SOGoContactFolder.h>
-#import <Contacts/SOGoContactFolders.h>
 #import <Contacts/NSDictionary+LDIF.h>
 
 #import <SoObjects/Contacts/NGVCard+SOGo.h>
@@ -44,18 +44,27 @@
 #import <SoObjects/Contacts/SOGoContactLDIFEntry.h>
 #import <SoObjects/Contacts/SOGoContactGCSList.h>
 #import <SoObjects/Contacts/SOGoContactGCSFolder.h>
-#import <GDLContentStore/GCSFolder.h>
+
+#import <SOGo/NSString+Utilities.h>
 
 #import "UIxContactFolderActions.h"
 
+static NSArray *photoTags = nil;
+
 @implementation UIxContactFolderActions
 
++ (void) initialize
+{
+  if (!photoTags)
+    {
+      photoTags = [[NSArray alloc] initWithObjects: @"jpegphoto", @"photo", @"thumbnailphoto", nil];
+    }
+}
 
 /* actions */
 
 - (id <WOActionResults>) exportAction
 {
-  WORequest *request;
   WOResponse *response;
   NSArray *contactsId;
   NSEnumerator *uids;
@@ -65,9 +74,9 @@
   NSMutableString *content;
 
   content = [NSMutableString string];
-  request = [context request];
   sourceFolder = [self clientObject];
-  contactsId = [request formValuesForKey: @"uid"];
+  contactsId = [[[[context request] contentAsString] objectFromJSONString] objectForKey: @"uids"];
+
   if (!contactsId)
     contactsId = [sourceFolder toOneRelationshipKeys];
 
@@ -78,9 +87,9 @@
                                     inContext: [self context]
                                       acquire: NO];
       if ([currentChild respondsToSelector: @selector (vCard)])
-        [content appendFormat: [[currentChild ldifRecord] ldifRecordAsString]];
+        [content appendFormat: @"%@", [[currentChild ldifRecord] ldifRecordAsString]];
       else if ([currentChild respondsToSelector: @selector (vList)])
-        [content appendFormat: [[currentChild vList] ldifString]];
+        [content appendFormat: @"%@", [[currentChild vList] ldifString]];
       [content appendString: @"\n"];
     }
 
@@ -109,16 +118,22 @@
 
   request = [context request];
   rc = [NSMutableDictionary dictionary];
-  data = [request formValueForKey: @"contactsFile"];
-  if ([data respondsToSelector: @selector(isEqualToString:)])
-    fileContent = (NSString *) data;
-  else
+  data = [[request httpRequest] body];
+
+  // We got an exception, that means the file upload limit
+  // has been reached.
+  if ([data isKindOfClass: [NSException class]])
     {
-      fileContent = [[NSString alloc] initWithData: (NSData *) data 
-                                          encoding: NSUTF8StringEncoding];
-      [fileContent autorelease];
+      response = [self responseWithStatus: 507];
+      return response;
     }
 
+  data = [[[data parts] lastObject] body];
+
+  fileContent = [[NSString alloc] initWithData: (NSData *) data 
+                                      encoding: NSUTF8StringEncoding];
+  [fileContent autorelease];
+
   if (fileContent && [fileContent length])
     {
       if ([fileContent hasPrefix: @"dn:"])
@@ -129,12 +144,10 @@
         imported = 0;
     }
 
-  [rc setObject: [NSNumber numberWithInt: imported]
-         forKey: @"imported"];
+  [rc setObject: [NSNumber numberWithInt: imported]  forKey: @"imported"];
 
   response = [self responseWithStatus: 200];
-  [response setHeader: @"text/html" 
-               forKey: @"content-type"];
+  [response setHeader: @"text/html"  forKey: @"content-type"];
   [(WOResponse*)response appendContentString: [rc jsonRepresentation]];
 
   return response;
@@ -142,24 +155,26 @@
 
 - (int) importLdifData: (NSString *) ldifData
 {
+  NSMutableDictionary *entry, *encodedEntry;
+  SOGoContactLDIFEntry *ldifEntry;
+  NSArray *ldifContacts, *lines;
   SOGoContactGCSFolder *folder;
-  NSString *key, *value;
-  NSArray *ldifContacts, *lines, *components;
-  NSMutableDictionary *entry;
+  NSEnumerator *keyEnumerator;
+  NSString *key, *uid, *line;
   NGVCard *vCard;
-  NSString *uid;
-  int i,j,count,linesCount;
-  int rc = 0;
+  id value;
+
+  NSRange r;
+  int i, j, count, linesCount;
+  int rc;
 
   folder = [self clientObject];
   ldifContacts = [ldifData componentsSeparatedByString: @"\ndn"];
   count = [ldifContacts count];
+  rc = 0;
 
   for (i = 0; i < count; i++)
     {
-      SOGoContactLDIFEntry *ldifEntry;
-      NSEnumerator *keyEnumerator;
-      NSMutableDictionary *encodedEntry;
       encodedEntry = [NSMutableDictionary dictionary];
       lines = [[ldifContacts objectAtIndex: i] 
                componentsSeparatedByString: @"\n"];
@@ -168,7 +183,6 @@
       linesCount = [lines count];
       for (j = 0; j < linesCount; j++)
         {
-          NSString *line;
           line = [lines objectAtIndex: j];
 
           /* skip embedded comment lines */
@@ -184,17 +198,17 @@
               if (key != NULL)
                 {
                   value = [[encodedEntry valueForKey: key]
-                           stringByAppendingString: [line substringFromIndex: 1]];
+			    stringByAppendingString: [line substringFromIndex: 1]];
                   [encodedEntry setValue: value forKey: key];
                 }
               continue;
             }
 
-          components = [line componentsSeparatedByString: @": "];
-          if ([components count] == 2)
+          r = [line rangeOfString: @": "];
+	  if (r.location != NSNotFound)
             {
-              key = [[components objectAtIndex: 0] lowercaseString];
-              value = [components objectAtIndex: 1];
+              key = [[line substringToIndex: r.location] lowercaseString];
+              value = [line substringFromIndex: NSMaxRange(r)];
 
               if ([key length] == 0)
                 key = @"dn";
@@ -216,8 +230,16 @@
           if ([key hasSuffix: @":"])
             {
               key = [key substringToIndex: [key length] - 1];
-              value = [value stringByDecodingBase64];
+	      if ([photoTags containsObject: key])
+		value = [value dataByDecodingBase64];
+	      else
+		value = [value stringByDecodingBase64];
             }
+
+	  // Standard key recognized in NGCards
+	  if ([photoTags containsObject: key])
+	    key = @"photo";
+
           [entry setValue: value forKey: key];
         }
 
@@ -238,17 +260,21 @@
 
 - (int) importVcardData: (NSString *) vcardData
 {
+  NSAutoreleasePool *pool;
   NSArray *allCards;
-  int rc;
+  int rc, count;
 
   rc = 0;
+
+  pool = [[NSAutoreleasePool alloc] init];
   allCards = [NGVCard parseFromSource: vcardData];
 
-  if (allCards && [allCards count])
+  count = [allCards count];
+  if (allCards && count)
     {
       int i;
 
-      for (i = 0; i < [allCards count]; i++)
+      for (i = 0; i < count; i++)
 	{
 	  if (![self importVcard: [allCards objectAtIndex: i]])
 	    {
@@ -260,29 +286,36 @@
 	}
     }
 
+  RELEASE(pool);
+
   return rc;
 }
 
 - (BOOL) importVcard: (NGVCard *) card
 {
-  NSString *uid;
   SOGoContactGCSFolder *folder;
   SOGoContactGCSEntry *contact;
+  NSAutoreleasePool *pool;
+  NSString *uid;
+
   BOOL rc = NO;
 
   if (card)
     {
+      pool = [[NSAutoreleasePool alloc] init];
       folder = [self clientObject];
       uid = [folder globallyUniqueObjectId];
 
       [card setUid: uid];
+      // TODO: shall we add .vcf as in [SOGoContactGCSEntry copyToFolder:]
       contact = [SOGoContactGCSEntry objectWithName: uid
                                         inContainer: folder];
       [contact setIsNew: YES];
       
-      [contact saveContentString: [card versitString]];
+      [contact saveComponent: card];
       
       rc = YES;
+      RELEASE(pool);
     }
 
   return rc;
