@@ -1,18 +1,18 @@
 /*
-  Copyright (C) 2004 SKYRIX Software AG
+  Copyright (C) 2005-2015 Inverse inc.
 
-  This file is part of OpenGroupware.org.
-
-  OGo is free software; you can redistribute it and/or modify it under
+  This file is part of SOGo.
+ 
+  SOGo is free software; you can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version.
-
-  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
+ 
+  SOGo is distributed in the hope that it will be useful, but WITHOUT ANY
   WARRANTY; without even the implied warranty of MERCHANTABILITY or
   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
   License for more details.
-
+ 
   You should have received a copy of the GNU Lesser General Public
   License along with OGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
@@ -35,7 +35,9 @@
   NSMutableArray *photosURL;
 }
 
-- (NSString *) fullName;
+//- (NSString *) fullName;
+- (NSArray *) orgUnits;
+- (NSString *) photoURL;
 
 @end
 
