@@ -18,24 +18,17 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSData.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSKeyValueCoding.h>
-#import <Foundation/NSValue.h>
 
-#import <SaxObjC/SaxAttributes.h>
-#import <SaxObjC/SaxContentHandler.h>
 #import <SaxObjC/SaxLexicalHandler.h>
-#import <SaxObjC/SaxXMLReader.h>
 #import <SaxObjC/SaxXMLReaderFactory.h>
 #import <NGExtensions/NSString+misc.h>
 #import <NGExtensions/NSString+Encoding.h>
-#import <NGObjWeb/SoObjects.h>
 
 #include <libxml/encoding.h>
 
+#import <SoObjects/SOGo/NSString+Utilities.h>
 #import <SoObjects/Mailer/SOGoMailObject.h>
 #import <SoObjects/Mailer/SOGoMailBodyPart.h>
 
@@ -161,10 +154,10 @@ static NSData* _sanitizeContent(NSData *theData)
       if (i < len-5)
 	{
 	  if ((*bytes == '<') &&
-	      (*(bytes+1) == 'm' || *(bytes+2) == 'M') &&
-	      (*(bytes+2) == 'e' || *(bytes+3) == 'E') &&
-	      (*(bytes+3) == 't' || *(bytes+4) == 'T') &&
-	      (*(bytes+4) == 'a' || *(bytes+5) == 'A') &&
+	      (*(bytes+1) == 'm' || *(bytes+1) == 'M') &&
+	      (*(bytes+2) == 'e' || *(bytes+2) == 'E') &&
+	      (*(bytes+3) == 't' || *(bytes+3) == 'T') &&
+	      (*(bytes+4) == 'a' || *(bytes+4) == 'A') &&
 	      (*(bytes+5) == ' '))
             in_meta = YES;
 	}
@@ -267,7 +260,7 @@ static NSData* _sanitizeContent(NSData *theData)
                   
                   tags = [VoidTags objectEnumerator];
                   tag = [tags nextObject];
-                  while (tag)
+                  while (tag && found_tag)
                     {
                       if ([tag caseInsensitiveCompare: found_tag] == NSOrderedSame)
                         {
@@ -420,9 +413,35 @@ static NSData* _sanitizeContent(NSData *theData)
   showWhoWeAre();
 }
 
+/**
+ * About CSS At-Rules (https://css-tricks.com/the-at-rules-of-css/)
+ *
+ * At-Rules follow two possible synthaxes:
+ *
+ *   @[KEYWORD] (RULE);
+ *     Examples:
+ *       @charset "UTF-8";
+ *       @import 'global.css';
+ *       @namespace svg url(http://www.w3.org/2000/svg);
+ *
+ *   @[KEYWORD] { (Nested Statements) }
+ *     Examples:
+ *       @font-face {
+ *         font-family: 'MyWebFont';
+ *         src:  url('myfont.woff2') format('woff2'),
+ *               url('myfont.woff') format('woff');
+ *       }
+ *       @media only screen
+ *         and (min-device-width: 320px)
+ *         and (max-device-width: 480px)
+ *         and (-webkit-min-device-pixel-ratio: 2) {
+ *           .module { width: 100%; }
+ *       }
+ */
 - (void) _appendStyle: (unichar *) _chars
                length: (NSUInteger) _len
 {
+  NSMutableString *declaration;
   NSUInteger count, length;
   unichar *start, *currentChar;
 
@@ -436,16 +455,59 @@ static NSData* _sanitizeContent(NSData *theData)
       currentChar = _chars + count;
       if (inCSSDeclaration)
         {
-          if (*currentChar == '}')
+          if (*currentChar < 32)
+            {
+              // Append substring since last valid character and reset start counter
+              if (currentChar > start)
+                [declaration appendString: [NSString stringWithCharacters: start
+                                                                   length: (currentChar - start)]];
+              start = currentChar + 1;
+            }
+          else if (*currentChar == '}')
             {
               inCSSDeclaration = NO;
-              hasEmbeddedCSS = NO;
+              if (hasEmbeddedCSS)
+                {
+                  // End of at-rule definition; remove it from the stylesheet
+                  hasEmbeddedCSS = NO;
+                  start = currentChar + 1;
+                }
+              else
+                {
+                  // Prefix CSS rule
+                  length = (currentChar - start);
+                  [declaration appendString: [NSString stringWithCharacters: start length: length]];
+                  [css appendFormat: @".SOGoHTMLMail-CSS-Delimiter %@\n", declaration];
+                  start = currentChar;
+                }
+            }
+          else if (*currentChar == ';')
+            {
+              // Add !important
+              if ((currentChar < _chars - 10) ||
+                  !((*(currentChar-1) == 't' || *(currentChar-1) == 'T') &&
+                    (*(currentChar-2) == 'n' || *(currentChar-2) == 'N') &&
+                    (*(currentChar-3) == 'a' || *(currentChar-3) == 'A') &&
+                    (*(currentChar-4) == 't' || *(currentChar-4) == 'T') &&
+                    (*(currentChar-5) == 'r' || *(currentChar-5) == 'R') &&
+                    (*(currentChar-6) == 'o' || *(currentChar-6) == 'O') &&
+                    (*(currentChar-7) == 'p' || *(currentChar-7) == 'P') &&
+                    (*(currentChar-8) == 'm' || *(currentChar-8) == 'M') &&
+                    (*(currentChar-9) == 'i' || *(currentChar-9) == 'I') &&
+                    *(currentChar-10) == '!'))
+                {
+                  length = (currentChar - start);
+                  [declaration appendFormat: @"%@ !important;",
+                               [NSString stringWithCharacters: start length: length]];
+                  start = currentChar + 1;
+                }
             }
         }
       else
         {
           if (*currentChar < 32)
             {
+              // Append substring since last valid character and reset start counter
               if (currentChar > start)
                 [css appendString: [NSString stringWithCharacters: start
                                                            length: (currentChar - start)]];
@@ -454,26 +516,36 @@ static NSData* _sanitizeContent(NSData *theData)
           else
             {
               if (*currentChar == '{')
-                inCSSDeclaration = YES;
+                {
+                  // Start of rule declaration
+                  inCSSDeclaration = YES;
+                  declaration = [NSMutableString string];
+                }
               if (*currentChar == '}')
                 // CSS syntax error: ending declaration character while not in a CSS declaration.
                 // Ignore eveything from last CSS declaration.
                 start = currentChar + 1;
-              else if (*currentChar == ',')
-                hasEmbeddedCSS = NO;
-              else if (!hasEmbeddedCSS)
+              else if (hasEmbeddedCSS)
                 {
-                  if (*currentChar == '@')
-                    hasEmbeddedCSS = YES;
-                  else
-                    if (*currentChar > 32)
-                      {
-                        length = (currentChar - start);
-                        [css appendFormat: @"%@\n.SOGoHTMLMail-CSS-Delimiter ",
-                             [NSString stringWithCharacters: start length: length]];
-                        hasEmbeddedCSS = YES;
-                        start = currentChar;
-                      }
+                  if (*currentChar == ';')
+                    {
+                      // End of at-rule definition; remove it from the stylesheet
+                      hasEmbeddedCSS = NO;
+                      start = currentChar + 1;
+                    }
+                }
+              else if (*currentChar == ',')
+                {
+                  // Prefix CSS selector
+                  length = (currentChar - start);
+                  [css appendFormat: @" .SOGoHTMLMail-CSS-Delimiter %@,",
+                       [NSString stringWithCharacters: start length: length]];
+                  start = currentChar + 1;
+                }
+              else if (*currentChar == '@')
+                {
+                  // Start of at-rule definition
+                  hasEmbeddedCSS = YES;
                 }
             }
         }
@@ -551,9 +623,9 @@ static NSData* _sanitizeContent(NSData *theData)
                   else
                     skipAttribute = YES;
                 }
-              else if ([name isEqualToString: @"background"] ||
+              else if (([name isEqualToString: @"background"] ||
                        ([name isEqualToString: @"data"]
-                        || [name isEqualToString: @"classid"])
+                        || [name isEqualToString: @"classid"]))
                        && [lowerName isEqualToString: @"object"])
                 {
                   value = [_attributes valueAtIndex: count];
@@ -627,21 +699,10 @@ static NSData* _sanitizeContent(NSData *theData)
 
 - (void) _finishCSS
 {
-  NSRange excessiveDelimiter;
-
   [css replaceString: @"<!--" withString: @""];
   [css replaceString: @"-->" withString: @""];
   [css replaceString: @".SOGoHTMLMail-CSS-Delimiter body"
        withString: @".SOGoHTMLMail-CSS-Delimiter"];
-  [css replaceString: @";" withString: @" !important;"];
-
-  excessiveDelimiter = [css rangeOfString: @".SOGoHTMLMail-CSS-Delimiter "
-                                  options: NSBackwardsSearch];
-  if (excessiveDelimiter.location != NSNotFound)
-    {
-      if (NSMaxRange (excessiveDelimiter) == [css length])
-        [css deleteCharactersInRange: excessiveDelimiter];
-    }
 }
 
 - (void) endElement: (NSString *) _localName
@@ -741,7 +802,7 @@ static NSData* _sanitizeContent(NSData *theData)
 
 /* SaxLexicalHandler */
 - (void) comment: (unichar *) _chars
-          length: (NSUInteger) _len
+          length: (int) _len
 {
   showWhoWeAre();
   if (inStyle)
@@ -807,7 +868,7 @@ static NSData* _sanitizeContent(NSData *theData)
     }
   [dump appendFormat: @"--- end ---\n"];
 
-  NSLog(dump);
+  NSLog(@"%@", dump);
   [dump release];
 }
 
@@ -848,6 +909,8 @@ static NSData* _sanitizeContent(NSData *theData)
   NSObject <SaxXMLReader> *parser;
   NSData *preparsedContent;
   SOGoMailObject *mail;
+  NSString *s;
+
   xmlCharEncoding enc;
 
   mail = [self clientObject];
@@ -857,7 +920,7 @@ static NSData* _sanitizeContent(NSData *theData)
              createXMLReaderForMimeType: @"text/html"];
 
   handler = [_UIxHTMLMailContentHandler new];
-  [handler setAttachmentIds: [mail fetchFileAttachmentIds]];
+  [handler setAttachmentIds: attachmentIds];
 
   // We check if we got an unsupported charset. If so
   // we convert everything to UTF-16{LE,BE} so it passes
@@ -866,12 +929,23 @@ static NSData* _sanitizeContent(NSData *theData)
   enc = [self _xmlCharEncoding];
   if (enc == XML_CHAR_ENCODING_ERROR)
     {
-      NSString *s;
-
       s = [NSString stringWithData: preparsedContent
 		    usingEncodingNamed: [[bodyInfo objectForKey:@"parameterList"]
 					  objectForKey: @"charset"]];
 
+      // In some rare cases (like #3276), we can get utterly broken email messages where
+      // HTML parts are wrongly encoded. We try to fall back to UTF-8 if that happens and
+      // if it still happens, we fall back to ISO-Latin-1.
+      if (!s)
+        {
+          s = [[NSString alloc] initWithData: preparsedContent  encoding: NSUTF8StringEncoding];
+
+          if (!s)
+            s = [[NSString alloc] initWithData: preparsedContent  encoding: NSISOLatin1StringEncoding];
+
+          AUTORELEASE(s);
+        }
+
 #if BYTE_ORDER == BIG_ENDIAN
       preparsedContent = [s dataUsingEncoding: NSUTF16BigEndianStringEncoding];
       enc = XML_CHAR_ENCODING_UTF16BE;
@@ -881,6 +955,14 @@ static NSData* _sanitizeContent(NSData *theData)
 #endif
     }
 
+  // Let's sanitize the string to make sure libxml doesn't go havoc
+  if (enc == XML_CHAR_ENCODING_UTF8)
+    {
+      s = [[NSString alloc] initWithData: preparsedContent  encoding: NSUTF8StringEncoding];
+      preparsedContent = [[s safeString] dataUsingEncoding: NSUTF8StringEncoding];
+      RELEASE(s);
+    }
+
   [handler setContentEncoding: enc];
 
   [parser setContentHandler: handler];
@@ -965,7 +1047,7 @@ static NSData* _sanitizeContent(NSData *theData)
     encoding = @"us-ascii";
 
   handler = [_UIxHTMLMailContentHandler new];
-  [handler setAttachmentIds: [mail fetchFileAttachmentIds]];
+  [handler setAttachmentIds: attachmentIds];
 
   // We check if we got an unsupported charset. If so
   // we convert everything to UTF-16{LE,BE} so it passes
