@@ -1,5 +1,5 @@
 /*
-  Copyright (C) 2007-2013 Inverse inc.
+  Copyright (C) 2007-2015 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -15,22 +15,20 @@
   License for more details.
 
   You should have received a copy of the GNU Lesser General Public
-  License along with OGo; see the file COPYING.  If not, write to the
+  License along with SOGo; see the file COPYING.  If not, write to the
   Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
   02111-1307, USA.
 */
 
-#import <Foundation/NSArray.h>
 #import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
 
-#import <NGExtensions/NGBase64Coding.h>
 #import <NGExtensions/NSNull+misc.h>
 #import <NGExtensions/NSObject+Logs.h>
-#import <NGExtensions/NGQuotedPrintableCoding.h>
 #import <NGExtensions/NSString+Encoding.h>
 #import <NGExtensions/NSString+misc.h>
 
+#import <NGObjWeb/WOResponse.h>
+
 #import <SOGo/NSString+Utilities.h>
 #import <Mailer/NSData+Mail.h>
 #import <Mailer/NSDictionary+Mail.h>
@@ -39,12 +37,21 @@
 #import "MailerUI/WOContext+UIxMailer.h"
 #import "UIxMailRenderingContext.h"
 #import "UIxMailSizeFormatter.h"
-#import "SOGoUI/UIxComponent.h"
 
 #import "UIxMailPartViewer.h"
 
 @implementation UIxMailPartViewer
 
+- (id) init
+{
+  if ((self = [super init]))
+    {
+      attachmentIds = nil;
+    }
+
+  return self;
+}
+
 - (void) dealloc
 {
   [flatContent release];
@@ -141,6 +148,26 @@
   return currentObject;
 }
 
+- (id) renderedPart
+{
+  NSString *type;
+
+  type = [NSString stringWithFormat: @"%@/%@",
+                            [bodyInfo objectForKey: @"type"],
+                            [bodyInfo objectForKey: @"subtype"]];
+
+  return [NSDictionary dictionaryWithObjectsAndKeys:
+                         [self className], @"type",
+                       type, @"contentType",
+                       [[self generateResponse] contentAsString], @"content",
+                       nil];
+}
+
+- (void) setAttachmentIds: (NSDictionary *) newAttachmentIds
+{
+  attachmentIds = newAttachmentIds;
+}
+
 - (NSData *) content
 {
   return [[self clientObject] fetchBLOB];
@@ -267,7 +294,11 @@
   NSString *extension;
 
   filename = [NSMutableString stringWithString: [self filename]];
-  if (![filename length])
+  if ([filename length])
+    // We replace any slash by a dash since Apache won't allow encoded slashes by default.
+    // See http://httpd.apache.org/docs/2.2/mod/core.html#allowencodedslashes
+    filename = [NSMutableString stringWithString: [filename stringByReplacingString: @"/" withString: @"-"]];
+  else
     [filename appendFormat: @"%@-%@",
 	      [self labelForKey: @"Untitled"],
 	      [bodyPart nameInContainer]];
@@ -282,7 +313,7 @@
   return [filename stringByEscapingURL];
 }
 
-- (NSString *) pathToAttachment
+- (NSString *) _pathForAttachmentOrDownload: (BOOL) forDownload
 {
   NSMutableString *url;
   NSString *s, *attachment;
@@ -294,16 +325,41 @@
   if (![url hasSuffix: @"/"])
     [url appendString: @"/"];
 
-//   s = [[self partPath] componentsJoinedByString: @"/"];
   if ([bodyPart isKindOfClass: [SOGoMailBodyPart class]])
     attachment = [self _filenameForAttachment: bodyPart];
   else
     attachment = @"0";
+
+  if (forDownload)
+    [url appendString: @"asAttachment/"];
+    
   [url appendString: attachment];
 
   return url;
 }
 
+- (NSString *) pathToAttachmentFromMessage
+{
+  SOGoMailBodyPart *bodyPart;
+
+  bodyPart = [self clientPart];
+
+  if ([bodyPart isKindOfClass: [SOGoMailBodyPart class]])
+    return [NSString stringWithFormat: @"%@/%@", [bodyPart nameInContainer], [self _filenameForAttachment: bodyPart]];
+
+  return @"0";
+}
+
+- (NSString *) pathToAttachment
+{
+  return [self _pathForAttachmentOrDownload: NO];
+}
+
+- (NSString *) pathForDownload
+{
+  return [self _pathForAttachmentOrDownload: YES];
+}
+
 - (NSString *) mimeImageURL
 {
   NSString *mimeImageFile, *mimeImageUrl;
