@@ -1,6 +1,6 @@
 /*
 
-  Copyright (C) 2006-2014 Inverse inc. 
+  Copyright (C) 2006-2015 Inverse inc.
   Copyright (C) 2004-2005 SKYRIX Software AG
 
   This file is part of SOGo.
@@ -21,16 +21,9 @@
   02111-1307, USA.
 */
 
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSException.h>
-#import <Foundation/NSTimeZone.h>
-#import <Foundation/NSURL.h>
-#import <Foundation/NSValue.h>
 
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WOApplication.h>
-#import <NGObjWeb/WOContext.h>
 #import <NGObjWeb/WOCookie.h>
 #import <NGObjWeb/WORequest.h>
 #import <NGObjWeb/WOResponse.h>
@@ -43,12 +36,11 @@
 
 #import <Appointments/SOGoAppointmentFolders.h>
 
+#import <SOGo/NSString+Crypto.h>
 #import <SOGo/NSString+Utilities.h>
 #import <SOGo/SOGoBuild.h>
 #import <SOGo/SOGoCache.h>
 #import <SOGo/SOGoCASSession.h>
-#import <SOGo/SOGoConstants.h>
-#import <SOGo/SOGoDomainDefaults.h>
 #if defined(SAML2_CONFIG)
 #import <SOGo/SOGoSAML2Session.h>
 #endif /* SAML2_ENABLE */
@@ -80,10 +72,10 @@
 
 /* accessors */
 
-- (NSString *) connectURL
-{
-  return [NSString stringWithFormat: @"%@/connect", [self applicationPath]];
-}
+//- (NSString *) connectURL
+//{
+//  return [NSString stringWithFormat: @"%@/connect", [self applicationPath]];
+//}
 
 - (NSString *) cookieUsername
 {
@@ -175,18 +167,18 @@
 //
 //
 //
-- (id <WOActionResults>) connectAction
+- (WOResponse *) connectAction
 {
   WOResponse *response;
   WORequest *request;
-  WOCookie *authCookie;
+  WOCookie *authCookie, *xsrfCookie;
   SOGoWebAuthenticator *auth;
   SOGoAppointmentFolders *calendars;
   SOGoUserDefaults *ud;
-  SOGoSystemDefaults *sd;
   SOGoUser *loggedInUser;
+  NSDictionary *params;
   NSString *username, *password, *language, *domain, *remoteHost;
-  NSArray *supportedLanguages;
+  NSArray *supportedLanguages, *creds;
   
   SOGoPasswordPolicyError err;
   int expire, grace;
@@ -195,14 +187,15 @@
   err = PolicyNoError;
   expire = grace = -1;
   
-  auth = [[WOApplication application]
-	    authenticatorInContext: context];
+  auth = [[WOApplication application] authenticatorInContext: context];
   request = [context request];
-  username = [request formValueForKey: @"userName"];
-  password = [request formValueForKey: @"password"];
-  language = [request formValueForKey: @"language"];
-  rememberLogin = [[request formValueForKey: @"rememberLogin"] boolValue];
-  domain = [request formValueForKey: @"domain"];
+  params = [[request contentAsString] objectFromJSONString];
+
+  username = [params objectForKey: @"userName"];
+  password = [params objectForKey: @"password"];
+  language = [params objectForKey: @"language"];
+  rememberLogin = [[params objectForKey: @"rememberLogin"] boolValue];
+  domain = [params objectForKey: @"domain"];
   /* this will always be set to something more or less useful by
    * [WOHttpTransaction applyAdaptorHeadersWithHttpRequest] */
   remoteHost = [request headerForKey:@"x-webobjects-remote-host"];
@@ -219,27 +212,40 @@
 
       [self logWithFormat: @"successful login from '%@' for user '%@' - expire = %d  grace = %d", remoteHost, username, expire, grace];
       
-      json = [NSDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithInt: expire], @"expire",
-                                [NSNumber numberWithInt: grace], @"grace", nil];
-      
-      response = [self responseWithStatus: 200
-		       andJSONRepresentation: json];
-      
+      // We get the proper username for cookie creation. If we are using a multidomain
+      // environment with SOGoEnableDomainBasedUID, we could have to append the domain
+      // to the username. Also when SOGoEnableDomainBasedUID is enabled, we could be in
+      // the DomainLessLogin situation, so we would NOT add the domain. -getUIDForEmail
+      // has all the logic for this, so lets use it.
       if ([domain isNotNull])
         {
-          sd = [SOGoSystemDefaults sharedSystemDefaults];
-          if ([sd enableDomainBasedUID])
-            username = [NSString stringWithFormat: @"%@@%@", username, domain];
+          username = [[SOGoUserManager sharedUserManager] getUIDForEmail: username];
         }
 
+      loggedInUser = [SOGoUser userWithLogin: username];
+
+      json = [NSDictionary dictionaryWithObjectsAndKeys:
+                             [loggedInUser cn], @"cn",
+                                [NSNumber numberWithInt: expire], @"expire",
+                                [NSNumber numberWithInt: grace], @"grace", nil];
+
+      response = [self responseWithStatus: 200
+                    andJSONRepresentation: json];
+
       authCookie = [auth cookieWithUsername: username
                                 andPassword: password
                                   inContext: context];
       [response addCookie: authCookie];
 
+      // We prepare the XSRF protection cookie
+      creds = [auth parseCredentials: [authCookie value]];
+      xsrfCookie = [WOCookie cookieWithName: @"XSRF-TOKEN"
+                                      value: [[SOGoSession valueForSessionKey: [creds lastObject]] asSHA1String]];
+      [xsrfCookie setPath: [NSString stringWithFormat: @"/%@/", [[context request] applicationName]]];
+      [response addCookie: xsrfCookie];
+
       supportedLanguages = [[SOGoSystemDefaults sharedSystemDefaults]
                              supportedLanguages];
-      loggedInUser = [SOGoUser userWithLogin: username];
       [context setActiveUser: loggedInUser];
       if (language && [supportedLanguages containsObject: language])
 	{
@@ -545,8 +551,8 @@
 - (WOResponse *) changePasswordAction
 {
   NSString *username, *domain, *password, *newPassword, *value;
+  WOCookie *authCookie, *xsrfCookie;
   NSDictionary *message;
-  WOCookie *authCookie;
   NSArray *creds;
   SOGoUserManager *um;
   SOGoPasswordPolicyError error;
@@ -587,7 +593,8 @@
       if ([domain isNotNull])
         {
           sd = [SOGoSystemDefaults sharedSystemDefaults];
-          if ([sd enableDomainBasedUID])
+          if ([sd enableDomainBasedUID] &&
+              [username rangeOfString: @"@"].location == NSNotFound)
             username = [NSString stringWithFormat: @"%@@%@", username, domain];
         }
       
@@ -596,6 +603,12 @@
                                 andPassword: newPassword
                                   inContext: context];
       [response addCookie: authCookie];
+
+      // We update the XSRF protection cookie
+      creds = [auth parseCredentials: [authCookie value]];
+      xsrfCookie = [WOCookie cookieWithName: @"XSRF-TOKEN"
+                                      value: [[SOGoSession valueForSessionKey: [creds lastObject]] asSHA1String]];
+      [response addCookie: xsrfCookie];
     }
   else
     response = [self _responseWithLDAPPolicyError: error];
