@@ -18,13 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSArray.h>
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSURL.h>
-#import <Foundation/NSTimeZone.h>
-#import <Foundation/NSValue.h>
 #import <NGObjWeb/NSException+HTTP.h>
 #import <NGObjWeb/WOCookie.h>
 #import <NGObjWeb/WORequest.h>
@@ -34,16 +27,20 @@
 
 #import <Appointments/SOGoFreeBusyObject.h>
 
+#import <Mailer/SOGoMailAccount.h>
+#import <Mailer/SOGoMailAccounts.h>
+
 #import <SOGo/SOGoCache.h>
 #import <SOGo/SOGoCASSession.h>
 #if defined(SAML2_CONFIG)
 #import <SOGo/SOGoSAML2Session.h>
 #endif
+#import <SOGo/SOGoDomainDefaults.h>
 #import <SOGo/SOGoUserManager.h>
 #import <SOGo/SOGoWebAuthenticator.h>
 #import <SOGo/SOGoUser.h>
-#import <SOGo/SOGoUserDefaults.h>
 #import <SOGo/SOGoUserFolder.h>
+#import <SOGo/SOGoUserProfile.h>
 #import <SOGo/SOGoSession.h>
 #import <SOGo/SOGoSystemDefaults.h>
 #import <SOGo/NSCalendarDate+SOGo.h>
@@ -52,7 +49,9 @@
 
 #import <SBJson/NSObject+SBJSON.h>
 
-#define intervalSeconds 900 /* 15 minutes */
+#define INTERVALSECONDS 900 /* 15 minutes */
+#define PADDING 8
+#define HALFPADDING PADDING/2
 
 @interface SOGoUserHomePage : UIxComponent
 
@@ -64,10 +63,16 @@
 {
   SOGoUserFolder *co;
   NSString *loginModule;
+  SOGoSystemDefaults *sd;
+  SOGoDomainDefaults *dd;
   SOGoUserDefaults *ud;
+  NSArray *filters;
   NSURL *moduleURL;
 
+  sd = [SOGoSystemDefaults sharedSystemDefaults];
+  dd = [[context activeUser] domainDefaults];
   ud = [[context activeUser] userDefaults];
+
   loginModule = [ud loginModule];
   if (!([loginModule isEqualToString: @"Calendar"]
         || [loginModule isEqualToString: @"Contacts"]
@@ -78,6 +83,24 @@
       loginModule = @"Calendar";
     }
 
+  // We check if we must write the Sieve scripts to the server
+  // upon first login if no user preferences are found, and the SOGo
+  // admin has defined SOGoSieveFilters in the domain or system settings
+  if ([dd sieveScriptsEnabled] && [[[ud source] values] count] == 0 &&
+      ((filters = [[dd source] objectForKey: @"SOGoSieveFilters"]) || (filters = [[sd source] objectForKey: @"SOGoSieveFilters"])))
+    {
+      SOGoMailAccount *account;
+      SOGoMailAccounts *folder;
+
+      [ud setSieveFilters: filters];
+      [ud synchronize];
+
+      folder = [[self clientObject] mailAccountsFolder: @"Mail"
+					     inContext: context];
+      account = [folder lookupName: @"0" inContext: context acquire: NO];
+      [account updateFilters];
+    }
+
   co = [self clientObject];
   moduleURL = [NSURL URLWithString: loginModule
 		     relativeToURL: [co soURL]];
@@ -85,20 +108,42 @@
   return [self redirectToLocation: [moduleURL absoluteString]];
 }
 
-- (void) _fillFreeBusyItems: (unsigned int *) items
-		      count: (unsigned int) itemCount
-                withRecords: (NSArray *) records
-              fromStartDate: (NSCalendarDate *) startDate
-                  toEndDate: (NSCalendarDate *) endDate
+- (NSDictionary *) _freeBusyFromStartDate: (NSCalendarDate *) startDate
+                            toEndDate: (NSCalendarDate *) endDate
+                          forFreeBusy: (SOGoFreeBusyObject *) fb
+                           andContact: (NSString *) uid
 {
+  NSCalendarDate *start, *end;
+  NSMutableDictionary *freeBusy, *dayData, *hourData;
+  NSArray *records;
   NSArray *emails, *partstates;
-  NSCalendarDate *currentDate;
+  NSCalendarDate *currentDate, *currentStartDate, *currentEndDate;
   NSDictionary *record;
+  NSString *dayKey, *hourKey, *minuteKey;
+  NSEnumerator *freeBusyList, *dayList, *hourList;
   SOGoUser *user;
+  int quarter, lastQuarter, recordCount, recordMax, count, i, type, maxBookings, isResource;
 
-  int recordCount, recordMax, count, startInterval, endInterval, i, type, maxBookings, isResource, delta;
+  // We "copy" the start/end date because -fetchFreeBusyInfosFrom will mess
+  // with the timezone and we don't want that to properly calculate the delta
+  // DO NOT USE -copy HERE - it'll simply return [self retain].
+  start = [NSCalendarDate dateWithYear: [startDate yearOfCommonEra]
+                                 month: [startDate monthOfYear]
+                                   day: [startDate dayOfMonth]
+                                  hour: [startDate hourOfDay]
+                                minute: [startDate minuteOfHour]
+                                second: [startDate secondOfMinute]
+                              timeZone: [startDate timeZone]];
 
-  recordMax = [records count];
+  end = [NSCalendarDate dateWithYear: [endDate yearOfCommonEra]
+                               month: [endDate monthOfYear]
+                                 day: [endDate dayOfMonth]
+                                hour: [endDate hourOfDay]
+                              minute: [endDate minuteOfHour]
+                              second: [endDate secondOfMinute]
+                            timeZone: [endDate timeZone]];
+
+  freeBusy = [NSMutableDictionary dictionary];
   user = [SOGoUser userWithLogin: [[self clientObject] ownerInContext: context] roles: nil];
   maxBookings = [user numberOfSimultaneousBookings];
   isResource = [user isResource];
@@ -106,6 +151,8 @@
   // Fetch freebusy information if the user is NOT a resource or if multiplebookings isn't unlimited
   if (!isResource || maxBookings != 0)
     {
+      records = [fb fetchFreeBusyInfosFrom: start to: end forContact: uid];
+      recordMax = [records count];
       for (recordCount = 0; recordCount < recordMax; recordCount++)
         {
           record = [records objectAtIndex: recordCount];
@@ -145,35 +192,111 @@
 
               if (type == 1)
                 {
-                  // User is busy for this event; update items bit string
-                  currentDate = [record objectForKey: @"startDate"];
-                  if ([currentDate earlierDate: startDate] == currentDate)
-                    startInterval = 0;
-                  else
-                    startInterval = ([currentDate timeIntervalSinceDate: startDate]
-                                     / intervalSeconds);
-
-                  delta = [[currentDate timeZoneDetail] timeZoneSecondsFromGMT] - [[startDate timeZoneDetail] timeZoneSecondsFromGMT];
-                  startInterval += (delta/60/15);
-                  startInterval = (startInterval < -4 ? -4 : startInterval);
-
-                  currentDate = [record objectForKey: @"endDate"];
-                  if ([currentDate earlierDate: endDate] == endDate)
-                    endInterval = itemCount - 1;
-                  else
-                    endInterval = ([currentDate timeIntervalSinceDate: startDate]
-                                   / intervalSeconds);
-                  
-                  delta = [[currentDate timeZoneDetail] timeZoneSecondsFromGMT] - [[startDate timeZoneDetail] timeZoneSecondsFromGMT];
-                  endInterval += (delta/60/15);
-                  endInterval = (endInterval < 0 ? 0 : endInterval);
-                  endInterval = (endInterval > itemCount+4 ? itemCount+4 : endInterval);
-
-                  // Update bit string representation
-                  // If the user is a resource with restristed amount of bookings, keep the sum of overlapping events
-                  for (count = startInterval; count < endInterval; count++)
+                  // User is busy for this event
+                  currentStartDate = [record objectForKey: @"startDate"];
+                  currentEndDate = [record objectForKey: @"endDate"];
+                  dayKey = [currentStartDate shortDateString];
+                  dayData = [freeBusy objectForKey: dayKey];
+                  if (!dayData)
                     {
-                      *(items + count) = (isResource && maxBookings > 0) ? *(items + count) + 1 : 1;
+                      dayData = [NSMutableDictionary dictionary];
+                      [freeBusy setObject: dayData forKey: dayKey];
+                    }
+                  if ([currentStartDate earlierDate: startDate] == currentStartDate)
+                    currentStartDate = startDate;
+
+                  currentDate = [NSCalendarDate dateWithYear: [currentStartDate yearOfCommonEra]
+                                                       month: [currentStartDate monthOfYear]
+                                                         day: [currentStartDate dayOfMonth]
+                                                        hour: [currentStartDate hourOfDay]
+                                                      minute: 0
+                                                      second: 0
+                                                    timeZone: [currentStartDate timeZone]];
+
+                  // Increment counters for quarters of first hour
+                  hourKey = [NSString stringWithFormat: @"%u", (unsigned int)[currentDate hourOfDay]];
+                  hourData = [dayData objectForKey: hourKey];
+                  if (!hourData)
+                    {
+                      hourData = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                                        [NSNumber numberWithInt: 0], @"0",
+                                                      [NSNumber numberWithInt: 0], @"15",
+                                                      [NSNumber numberWithInt: 0], @"30",
+                                                      [NSNumber numberWithInt: 0], @"45",
+                                                      nil];
+                      [dayData setObject: hourData forKey: hourKey];
+                    }
+                  quarter = (int)([currentStartDate minuteOfHour] / 15 + 0.5);
+                  if ([currentEndDate timeIntervalSinceDate: currentDate] < 3600)
+                    lastQuarter = (int)([currentEndDate minuteOfHour] / 15 + 0.5);
+                  else
+                    lastQuarter = 4;
+                  for (i = 0; i < lastQuarter; i++) {
+                    if (i >= quarter)
+                      {
+                        minuteKey = [NSString stringWithFormat: @"%u", i*15];
+                        count = [[hourData objectForKey: minuteKey] intValue] + 1;
+                        [hourData setObject: [NSNumber numberWithInt: count] forKey: minuteKey];
+                      }
+                  }
+                  currentDate = [currentDate dateByAddingYears:0 months:0 days:0 hours:1 minutes:0 seconds:0];
+
+                  // Increment counters of fully busy hours
+                  while ([currentDate compare: currentEndDate] == NSOrderedAscending &&
+                         [currentEndDate timeIntervalSinceDate: currentDate] >= 3600) // 1 hour
+                    {
+                      hourKey = [NSString stringWithFormat: @"%u", (unsigned int)[currentDate hourOfDay]];
+                      hourData = [dayData objectForKey: hourKey];
+                      if (!hourData)
+                        {
+                          hourData = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                                               [NSNumber numberWithInt: 1], @"0",
+                                                               [NSNumber numberWithInt: 1], @"15",
+                                                               [NSNumber numberWithInt: 1], @"30",
+                                                               [NSNumber numberWithInt: 1], @"45",
+                                                          nil];
+                          [dayData setObject: hourData forKey: hourKey];
+                        }
+                      else
+                        {
+                          for (i = 0; i < 4; i++)
+                            {
+                              minuteKey = [NSString stringWithFormat: @"%u", i*15];
+                              count = [[hourData objectForKey: minuteKey] intValue] + 1;
+                              [hourData setObject: [NSNumber numberWithInt: count] forKey: minuteKey];
+                            }
+                        }
+                      currentDate = [currentDate dateByAddingYears:0 months:0 days:0 hours:1 minutes:0 seconds:0];
+                    }
+
+                  // Increment counters for quarters of last hour
+                  if ([currentEndDate timeIntervalSinceDate: currentDate] > 0)
+                    {
+                      hourKey = [NSString stringWithFormat: @"%u", (unsigned int)[currentDate hourOfDay]];
+                      hourData = [dayData objectForKey: hourKey];
+                      if (!hourData)
+                        {
+                          hourData = [NSMutableDictionary dictionaryWithObjectsAndKeys:
+                                                               [NSNumber numberWithInt: 0], @"0",
+                                                               [NSNumber numberWithInt: 0], @"15",
+                                                               [NSNumber numberWithInt: 0], @"30",
+                                                               [NSNumber numberWithInt: 0], @"45",
+                                                          nil];
+                          [dayData setObject: hourData forKey: hourKey];
+                        }
+                      quarter = (int)([currentEndDate minuteOfHour] / 15 + 0.5);
+                      for (i = 0; i < 4; i++)
+                        {
+                          if (i < quarter)
+                            {
+                              minuteKey = [NSString stringWithFormat: @"%u", i*15];
+                              if (isResource)
+                                count = [[hourData objectForKey: minuteKey] intValue] + 1;
+                              else
+                                count = 1;
+                              [hourData setObject: [NSNumber numberWithInt: count] forKey: minuteKey];
+                            }
+                        }
                     }
                 }
             }
@@ -181,88 +304,44 @@
       if (maxBookings > 0)
         {
           // Reset the freebusy for the periods that are bellow the maximum number of bookings
-          for (count = 0; count < itemCount; count++)
+          freeBusyList = [freeBusy objectEnumerator];
+          while ((dayData = [freeBusyList nextObject]))
             {
-              if (*(items + count) < maxBookings)
-                *(items + count) = 0;
-              else
-                *(items + count) = 1;
+              dayList = [dayData objectEnumerator];
+              while ((hourData = [dayList nextObject]))
+                {
+                  hourList = [hourData keyEnumerator];
+                  while ((minuteKey = [hourList nextObject]))
+                    {
+                      count = [[hourData objectForKey: minuteKey] intValue];
+                      if (count < maxBookings)
+                        i = 0;
+                      else
+                        i = 1;
+                      if (i != count)
+                        [hourData setObject: [NSNumber numberWithInt: i] forKey: minuteKey];
+                    }
+                }
             }
         }
     }
-}
 
-//
-//
-//
-- (NSString *) _freeBusyFromStartDate: (NSCalendarDate *) startDate
-                            toEndDate: (NSCalendarDate *) endDate
-                          forFreeBusy: (SOGoFreeBusyObject *) fb
-                           andContact: (NSString *) uid
-{
-  NSCalendarDate *start, *end;
-  NSMutableArray *freeBusy;
-  unsigned int *freeBusyItems;
-  NSTimeInterval interval;
-  unsigned int count, intervals;
-
-  // We "copy" the start/end date because -fetchFreeBusyInfosFrom will mess
-  // with the timezone and we don't want that to properly calculate the delta
-  // DO NOT USE -copy HERE - it'll simply return [self retain].
-  start = [NSCalendarDate dateWithYear: [startDate yearOfCommonEra]
-                                 month: [startDate monthOfYear]
-                                   day: [startDate dayOfMonth]
-                                  hour: [startDate hourOfDay]
-                                minute: [startDate minuteOfHour]
-                                second: [startDate secondOfMinute]
-                              timeZone: [startDate timeZone]];
-  
-  end = [NSCalendarDate dateWithYear: [endDate yearOfCommonEra]
-                               month: [endDate monthOfYear]
-                                 day: [endDate dayOfMonth]
-                                hour: [endDate hourOfDay]
-                              minute: [endDate minuteOfHour]
-                              second: [endDate secondOfMinute]
-                            timeZone: [endDate timeZone]];
-
-  interval = [endDate timeIntervalSinceDate: startDate] + 60;
-
-  // Slices of 15 minutes. The +8 is to take into account that we can
-  // have a timezone change during the freebusy lookup. We have +4 at the
-  // beginning and +4 at the end.
-  intervals = interval / intervalSeconds + 8;
-
-  // Build a bit string representation of the freebusy data for the period
-  freeBusyItems = calloc(intervals, sizeof (unsigned int));
-  [self _fillFreeBusyItems: (freeBusyItems+4)
-                     count: (intervals-4)
-	       withRecords: [fb fetchFreeBusyInfosFrom: start to: end forContact: uid]
-             fromStartDate: startDate
-                 toEndDate: endDate];
-
-  // Convert bit string to a NSArray. We also skip by the default the non-requested information.
-  freeBusy = [NSMutableArray arrayWithCapacity: intervals];
-  for (count = 4; count < (intervals-4); count++)
-    {
-      [freeBusy addObject: [NSString stringWithFormat: @"%d", *(freeBusyItems + count)]];
-    }
-  free(freeBusyItems);
-
-  // Return a NSString representation
-  return [freeBusy componentsJoinedByString: @","];
+  return freeBusy;
 }
 
 - (id <WOActionResults>) readFreeBusyAction
 {
-  WOResponse *response;
   SOGoFreeBusyObject *freebusy;
   NSCalendarDate *startDate, *endDate;
+  NSDictionary *jsonResponse;
   NSString *queryDay, *uid;
   NSTimeZone *uTZ;
   SOGoUser *user;
+  unsigned int httpStatus;
 
   user = [context activeUser];
   uTZ = [[user userDefaults] timeZone];
+  httpStatus = 200;
 
   uid = [self queryParameterForKey: @"uid"];
   queryDay = [self queryParameterForKey: @"sday"];
@@ -279,28 +358,42 @@
                                                  inTimeZone: uTZ];
 
           if ([startDate earlierDate: endDate] == endDate)
-            response = [self responseWithStatus: 403
-                                      andString: @"Start date is later than end date."];
+            {
+              httpStatus = 403;
+              jsonResponse = [NSDictionary dictionaryWithObjectsAndKeys:
+                                             @"failure", @"status",
+                                           @"Start date is later than end date.", @"message",
+                                           nil];
+            }
           else
             {
-             freebusy = [self clientObject];
-              response
-                = [self responseWithStatus: 200
-                                 andString: [self _freeBusyFromStartDate: startDate
-                                                               toEndDate: endDate
-                                                             forFreeBusy: freebusy
-                                                              andContact: uid]];
+              freebusy = [self clientObject];
+              jsonResponse = [self _freeBusyFromStartDate: startDate
+                                                toEndDate: endDate
+                                              forFreeBusy: freebusy
+                                               andContact: uid];
             }
         }
       else
-        response = [self responseWithStatus: 403
-                                  andString: @"Invalid end date."];
+        {
+          httpStatus = 403;
+          jsonResponse = [NSDictionary dictionaryWithObjectsAndKeys:
+                                         @"failure", @"status",
+                                       @"Invalid end date.", @"message",
+                                       nil];
+        }
     }
   else
-    response = [self responseWithStatus: 403
-                              andString: @"Invalid start date."];
+    {
+      httpStatus = 403;
+      jsonResponse = [NSDictionary dictionaryWithObjectsAndKeys:
+                                     @"failure", @"status",
+                                   @"Invalid start date.", @"message",
+                                   nil];
+    }
 
-  return response;
+  return [self responseWithStatus: httpStatus
+            andJSONRepresentation: jsonResponse];
 }
 
 - (NSString *) _logoutRedirectURL
@@ -391,8 +484,8 @@
 
 - (id <WOActionResults>) logoffAction
 {
-  SOGoWebAuthenticator *auth;
   NSString *userName, *value;
+  SOGoWebAuthenticator *auth;
   WOResponse *response;
   NSCalendarDate *date;
   WOCookie *cookie;
@@ -422,6 +515,12 @@
   if (cookie)
     [response addCookie: cookie];
 
+  // We remove the XSRF cookie
+  cookie = [WOCookie cookieWithName: @"XSRF-TOKEN"  value: @"discard"];
+  [cookie setPath: [NSString stringWithFormat: @"/%@/", [[context request] applicationName]]];
+  [cookie setExpires: [date yesterday]];
+  [response addCookie: cookie];
+
   [response setHeader: [date rfc822DateString] forKey: @"Last-Modified"];
   [response setHeader: @"no-store, no-cache, must-revalidate,"
             @" max-age=0, post-check=0, pre-check=0"
@@ -437,13 +536,14 @@
 {
   NSString *uid;
   NSDictionary *contact;
-  NSString *contactInfo, *login;
-  NSMutableArray *jsonResponse, *jsonLine;
+  NSString *contactInfo, *owner;
+  NSMutableArray *jsonResponse;
+  NSMutableDictionary *jsonLine;
   NSArray *allUsers;
   int count, max;
   BOOL activeUserIsInDomain;
 
-  login = [[context activeUser] login];
+  owner = [[self clientObject] ownerInContext: context];
   activeUserIsInDomain = ([domain length] == 0 || [[[context activeUser] domain] isEqualToString: domain]);
 
   // We sort our array - this is pretty useful for the Web
@@ -457,19 +557,19 @@
       contact = [allUsers objectAtIndex: count];
       uid = [contact objectForKey: @"c_uid"];
 
-      // We do NOT return the current authenticated user
-      if (!activeUserIsInDomain || ![uid isEqualToString: login])
+      // We do NOT return the owner from which the search is performed
+      if (!activeUserIsInDomain || ![uid isEqualToString: owner])
         {
-          jsonLine = [NSMutableArray arrayWithCapacity: 4];
+          jsonLine = [NSMutableDictionary dictionary];
           if ([domain length])
             uid = [NSString stringWithFormat: @"%@@%@", uid, domain];
-          [jsonLine addObject: uid];
-          [jsonLine addObject: [contact objectForKey: @"cn"]];
-          [jsonLine addObject: [contact objectForKey: @"c_email"]];
-          [jsonLine addObject: [NSNumber numberWithBool: [[contact objectForKey: @"isGroup"] boolValue]]];
+          [jsonLine setObject: uid forKey: @"uid"];
+          [jsonLine setObject: [contact objectForKey: @"cn"] forKey: @"cn"];
+          [jsonLine setObject: [contact objectForKey: @"c_email"] forKey: @"c_email"];
+          [jsonLine setObject: [NSNumber numberWithBool: [[contact objectForKey: @"isGroup"] boolValue]] forKey: @"isGroup"];
           contactInfo = [contact objectForKey: @"c_info"];
           if (contactInfo)
-            [jsonLine addObject: contactInfo];
+            [jsonLine setObject: contactInfo forKey: @"c_info"];
           [jsonResponse addObject: jsonLine];
         }
     }
@@ -477,10 +577,28 @@
   return jsonResponse;
 }
 
+/**
+ * @api {get} /so/:username/usersSearch?search=:search Search for users
+ * @apiVersion 1.0.0
+ * @apiName GetUsersSearch
+ * @apiGroup Common
+ * @apiExample {curl} Example usage:
+ *     curl -i http://localhost/SOGo/so/sogo1/usersSearch?search=john
+ *
+ * @apiParam {String} search Substring to match against username or email address
+ *
+ * @apiSuccess (Success 200) {Object[]} users        List of matching users
+ * @apiSuccess (Success 200) {String} users.uid      User ID
+ * @apiSuccess (Success 200) {String} users.c_email  Main email address
+ * @apiSuccess (Success 200) {String} users.cn       Common name
+ * @apiSuccess (Success 200) {Number} users.isGroup  1 if the user is a group
+ * @apiError   (Error 400) {Object} error            The error message
+ */
 - (id <WOActionResults>) usersSearchAction
 {
   NSMutableArray *users;
   NSArray *currentUsers;
+  NSDictionary *message;
   NSString *contact, *domain, *uidDomain;
   NSEnumerator *visibleDomains;
   id <WOActionResults> result;
@@ -509,11 +627,15 @@
               [users addObjectsFromArray: currentUsers];
             }
         }
-      result = [self responseWithStatus: 200 andJSONRepresentation: users];
+      result = [self responseWithStatus: 200
+                  andJSONRepresentation: [NSDictionary dictionaryWithObject: users forKey: @"users"]];
     }
   else
-    result = [NSException exceptionWithHTTPStatus: 400
-                          reason: @"missing 'search' parameter"];
+    {
+      message = [NSDictionary dictionaryWithObject: [self labelForKey: @"Missing search parameter"]
+                                          forKey: @"error"];
+      result = [self responseWithStatus: 400 andJSONRepresentation: message];
+    }
 
   return result;
 }
@@ -527,27 +649,80 @@
   [response setHeader: @"text/plain; charset=utf-8"
 	    forKey: @"Content-Type"];
 
-  [response appendContentString: [folders JSONRepresentation]];
+  [response appendContentString: [[NSDictionary dictionaryWithObject: folders
+                                                              forKey: @"folders"] JSONRepresentation]];
 
   return response;
 }
 
+/**
+ * @api {get} /so/:username/foldersSearch?type=:type Search for folders
+ * @apiVersion 1.0.0
+ * @apiName GetFoldersSearch
+ * @apiGroup Common
+ * @apiExample {curl} Example usage:
+ *     curl -i http://localhost/SOGo/so/sogo1/foldersSearch?type=contact
+ *
+ * @apiParam {String} type Either 'calendar' or 'contact'. If nothing is specifed, its both.
+ *
+ * @apiSuccess (Success 200) {Object[]} folders           List of matching folders
+ * @apiSuccess (Success 200) {String} folders.name        Path of folder
+ * @apiSuccess (Success 200) {String} folders.displayName Human readable name
+ * @apiSuccess (Success 200) {String} folders.owner       Username of owner
+ * @apiSuccess (Success 200) {String} folders.type        Either 'calendar' or 'contact'
+ * @apiError   (Error 400) {Object} error                 The error message
+ */
 - (id <WOActionResults>) foldersSearchAction
 {
   NSString *folderType;
-  NSArray *folders;
+  NSMutableArray *folders;
   id <WOActionResults> result;
   SOGoUserFolder *userFolder;
 
+
   folderType = [self queryParameterForKey: @"type"];
+  folders = [NSMutableArray array];
   userFolder = [self clientObject];
-  folders = [userFolder foldersOfType: folderType
-                               forUID: [userFolder ownerInContext: context]];
-  result = [self _foldersResponseForResults: folders];
+
+  if ([folderType length])
+    {
+      [folders addObjectsFromArray: [userFolder foldersOfType: folderType
+                                                       forUID: [userFolder ownerInContext: context]]];
+    }
+  else
+    {
+      [folders addObjectsFromArray: [userFolder foldersOfType: @"calendar"
+                                                        forUID: [userFolder ownerInContext: context]]];
+      [folders addObjectsFromArray: [userFolder foldersOfType: @"contact"
+                                                       forUID: [userFolder ownerInContext: context]]];
+    }
   
+  result = [self _foldersResponseForResults: folders];
+
   return result;
 }
 
+/**
+ * @api {get} /so/:username/date Get current day
+ * @apiVersion 1.0.0
+ * @apiName GetCurrentDate
+ * @apiGroup Common
+ * @apiExample {curl} Example usage:
+ *     curl -i http://localhost/SOGo/so/sogo1/date
+ *
+ * @apiSuccess (Success 200) {String} weekday      Full weekday name according to user's locale
+ * @apiSuccess (Success 200) {String} month        Full month name according to user's locale
+ * @apiSuccess (Success 200) {String} day          Day of month as two digit decimal number (leading zero)
+ * @apiSuccess (Success 200) {String} year         Year as a decimal number with century
+ * @apiSuccess (Success 200) {Object} abbr         Abbreviations
+ * @apiSuccess (Success 200) {String} abbr.weekday Abbreviated weekday name according to user's locale
+ * @apiSuccess (Success 200) {String} abbr.month   Abbreviated month name according to user's locale
+ */
+- (id <WOActionResults>) dateAction
+{
+  return [self responseWithStatus: 200 andJSONRepresentation: [[context activeUser] currentDay]];
+}
+
 - (id) recoverAction
 {
   return [self responseWithStatus: 200
