@@ -31,7 +31,6 @@
 
 #import <SoObjects/SOGo/SOGoMailer.h>
 #import <SoObjects/SOGo/SOGoUser.h>
-#import <SoObjects/SOGo/SOGoObject.h>
 #import <SoObjects/SOGo/SOGoUserManager.h>
 #import <SoObjects/SOGo/NSCalendarDate+SOGo.h>
 #import <SoObjects/SOGo/NSString+Utilities.h>
@@ -140,7 +139,7 @@
 
   part = [NGMimeBodyPart bodyPartWithHeader: headerMap];
   body = [[self body] dataUsingEncoding: NSUTF8StringEncoding];
-  [part setBody: [self body]];
+  [part setBody: body];
 
   return part;
 }
@@ -205,9 +204,11 @@
 
   dd = [activeUser domainDefaults];
   [[SOGoMailer mailerWithDomainDefaults: dd]
-    sendMimePart: message
-    toRecipients: [NSArray arrayWithObject: recipient]
-          sender: from];
+           sendMimePart: message
+           toRecipients: [NSArray arrayWithObject: recipient]
+                 sender: from
+      withAuthenticator: [self authenticatorInContext: context]
+              inContext: context];
 }
 
 @end
