@@ -2,8 +2,6 @@
  *
  * Copyright (C) 2007 Inverse inc.
  *
- * Author: Wolfgang Sourdeau <wsourdeau@inverse.ca>
- *
  * This file is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
@@ -20,10 +18,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#import <Foundation/NSCalendarDate.h>
-#import <Foundation/NSDictionary.h>
-#import <Foundation/NSEnumerator.h>
-#import <Foundation/NSNull.h>
 #import <Foundation/NSValue.h>
 
 #import <NGExtensions/NGCalendarDateRange.h>
@@ -120,8 +114,8 @@
   NSComparisonResult result;
   unsigned int selfTime, otherTime;
 
-  result = [self _compareCompletionWithStatus1: [self objectAtIndex: taskCalendarNameIndex]
-                                    andStatus2: [otherTask objectAtIndex: taskCalendarNameIndex]];
+  result = [self _compareCompletionWithStatus1: [self objectAtIndex: taskStatusIndex]
+                                    andStatus2: [otherTask objectAtIndex: taskStatusIndex]];
   if (result == NSOrderedSame)
   {
     // End date
@@ -188,7 +182,41 @@
   return [selfTitle caseInsensitiveCompare: otherTitle];
 }
 
-- (NSComparisonResult) compareTasksEndAscending: (NSArray *) otherTask
+- (NSComparisonResult) compareTasksStartDateAscending: (NSArray *) otherTask
+{
+  NSComparisonResult result;
+  unsigned int selfTime, otherTime;
+
+  // Start date
+  selfTime = [[self objectAtIndex: taskStartDateIndex] intValue];
+  otherTime = [[otherTask objectAtIndex: taskStartDateIndex] intValue];
+  if (selfTime && !otherTime)
+    result = NSOrderedAscending;
+  else if (!selfTime && otherTime)
+    result = NSOrderedDescending;
+  else
+  {
+    if (selfTime > otherTime)
+      result = NSOrderedDescending;
+    else if (selfTime < otherTime)
+      result = NSOrderedAscending;
+    else
+    {
+      // Calendar ID
+      result = [[self objectAtIndex: taskFolderIndex]
+                  compare: [otherTask objectAtIndex: taskFolderIndex]];
+      if (result == NSOrderedSame)
+        // Task name
+        result = [[self objectAtIndex: taskTitleIndex]
+                    compare: [otherTask objectAtIndex: taskTitleIndex]
+                    options: NSCaseInsensitiveSearch];
+    }
+  }
+
+  return result;
+}
+
+- (NSComparisonResult) compareTasksEndDateAscending: (NSArray *) otherTask
 {
   NSComparisonResult result;
   unsigned int selfTime, otherTime;
